
//
//
//
//
//  Extinction Suite (v3.2.4.7)
//           by LUKAS PAYNE
//
// This macro uses "the DCRAW reader plugin," by David Coffin, to allow ImageJ to open RAW
// files.
//
// A portion is based on the BatchConvert.txt macro at http://rsb.info.nih.gov/ij/plugins/batch-converter.html,
// with significant alterations.
//
// It may be used to open RAW (or otherwise) files in ImageJ, split the color channels if desired,
// and then save opened files as TIFF, 8-bit TIFF,
// JPEG, GIF, PNG, PGM, BMP, FITS, Text Image, ZIP or "Raw"

// Following the conversion/splitting, the program may be used to average and, if desired, re-merge the
// the averaged channels.
//
// The resulting averaged color channel images can then be put through the extinction analysis process to create
// 3 extinction images.
//
// Finally, the extinction images may be further analysed to produce single and ensemble nanoparticle extinction data.


showMessage("Any open images should be closed prior to running the Extinction Suite");

Dialog.create("Extinction Suite Mode Selection");
Modechoices=newArray("Unpolarized","Polarized");
operation=newArray("Run Folder Mapping Module","Run Extinction Suite");
Dialog.addChoice("Choose Image Processing and Analysis Mode",Modechoices,Modechoices[0]);
Dialog.addChoice("Choose Operation",operation,operation[1]);
Dialog.show();

Extmode=Dialog.getChoice();
operationchoice=Dialog.getChoice();

if(Extmode=="Unpolarized"){
	polarization=false;
}
if(Extmode=="Polarized"){
	polarization=true;
}
if(operationchoice==operation[0]){
	mapping=true;
	extinctionsuite=false;
}
if(operationchoice==operation[1]){
	mapping=false;
	extinctionsuite=true;
}

if(mapping==true){
	showMessage("Choose Location in which to keep New Base folder for imaging/analysis");

	compdirec=getDirectory("Choose Computer Folder");
	Dialog.create("Folder Options");
	Dialog.setInsets(0,0,0);
	Dialog.addString("Input Title of New Base Folder", "e.g.2015_01_01_goldNP");
	Dialog.setInsets(10,0,0);
	Dialog.addCheckbox("Will you acquire Darkfield images?",true);
	Dialog.setInsets(0,0,0);
	Dialog.addCheckbox("Will you acquire BG Images with darkfield exposure times?",true);
	Dialog.setInsets(0,0,0);
	Dialog.addCheckbox("Will you need to convert images?",true);
	color=newArray("Color","BW");
	Dialog.setInsets(15,0,0);
	Dialog.addChoice("Choose Image Type",color,color[0]);
	Dialog.show();

	basetitle=Dialog.getString();
	darkfolderoption=Dialog.getCheckbox();
	darkBGopt=Dialog.getCheckbox();;
	convert=Dialog.getCheckbox();;;
	colorchoice=Dialog.getChoice();
	File.makeDirectory(compdirec+basetitle);
	colortitles=newArray("B","G","R");
	count=3;
	if(polarization==false){
		if(darkfolderoption==true){
			count=4;
			if(darkBGopt==true){
				count=5;
			}
		}
		foldertitles=newArray("Focus","Defocus","BG","Dark","DarkBG");
		BASEdirec=compdirec+basetitle+File.separator;
		for(i=0;i<count;i++){
			File.makeDirectory(BASEdirec+foldertitles[i]);
		}
		if(convert==false && colorchoice==color[0]){
			for(i=0;i<count;i++){
				for(j=0;j<3;j++){
					File.makeDirectory(BASEdirec+foldertitles[i]+File.separator+colortitles[j]);
				}
			}
		}
	}

	if(polarization==true){
		Dialog.create("Polarization Options");
		Dialog.addNumber("Initial Angle",0);
		Dialog.addNumber("Final Focus/Defocus Angle",180);
		Dialog.addNumber("Angle Step Size",10);
		Dialog.show();

		polstart=Dialog.getNumber();
		polend=Dialog.getNumber();;
		polstep=Dialog.getNumber();;;

		polarnum=((polend-polstart)/polstep)+1;
		polnames=newArray(polarnum+2);
		polnums=newArray(polarnum);

		for(poloop=0;poloop<polarnum;poloop++){
			polnames[poloop]=d2s(poloop*(polend/(polarnum-1)),0);
			polnums[poloop]=poloop*(polend/(polarnum-1));
		}
		polnames[polarnum]="BG";
		polnames[polarnum+1]="DarkBG";
		foldertitles=newArray("Focus","Defocus","Dark");
		bgcount=1;
		if(darkfolderoption==true){
			if(darkBGopt==true){
				bgcount=2;
			}
		}

		for(poloop=0;poloop<polarnum+bgcount;poloop++){
			BASEdirec=compdirec+basetitle+File.separator+polnames[poloop]+File.separator;
			File.makeDirectory(BASEdirec);
			if(poloop<polarnum){
				File.makeDirectory(BASEdirec+"Focus");
				File.makeDirectory(BASEdirec+"Defocus");
				if(darkfolderoption==true){
					File.makeDirectory(BASEdirec+"Dark");
				}
			}
			if(poloop>=polarnum){
				count=1;
			}
			if(convert==false && colorchoice==color[0]){
				for(i=0;i<count;i++){
					for(j=0;j<3;j++){
						if(poloop<polarnum){
							File.makeDirectory(BASEdirec+foldertitles[i]+File.separator+colortitles[j]);
						}
						else{
							File.makeDirectory(BASEdirec+File.separator+colortitles[j]);
						}
					}
				}
			}
		}
	}
}


if(extinctionsuite==true){
	Dialog.create("Extinction Suite");
	Dialog.addCheckbox("Perform Image Conversion?",true);
	Dialog.addCheckbox("Perform Image Averaging?",true);
	Dialog.addCheckbox("Perform Extinction Analysis?",true);
	Dialog.addCheckbox("Perform Particle Analysis?",true);
	Dialog.show();
	converter=Dialog.getCheckbox();
	averager=Dialog.getCheckbox();;
	extanalysis=Dialog.getCheckbox();;;
	partanalysis=Dialog.getCheckbox();;;;
	
	if(converter|averager==true){
		if(extanalysis==true || extanalysis==false){
			darkBGsubopt=newArray("No Darkfield images aqcuired","No darkfield BG correction","Subtract BG Image","Subtract Numerical offset");
			formats=newArray("Consumer RAW Formats", "TIFF", "JPEG", "GIF", "PNG","PGM", "BMP", "FITS", "Text Image", "ZIP", "8-bit TIFF");
			imstyle=newArray("Single frame per input image file","Multiple frames (a stack) per input image file");
			coloropt=newArray("Color","BW");
			Dialog.create("Input Image Options");
			Dialog.addChoice("Specify Input Image File Format",formats,formats[0]);
			Dialog.addChoice("Specify Input Image Color Format",coloropt,coloropt[0]);
			Dialog.addMessage("Please indicate below, the method by wich the acquired images have been saved.\nSome cameras, e.g. the Canon 40D, save individual frames per saved file.\nOther cameras, e.g. the PCO Edge 5.5 sCMOS save multiple frames (an image stack) per saved file.");
			Dialog.addChoice("Save Style of Input Images",imstyle,imstyle[0]);
			Dialog.addMessage("You may indicate below whether or not you acquired any darkfield images, or, if you\nacquired darkfield images, how you would like to correct the BG offset of the \ndarkfield image. You can choose to correct it via BG images acquired using darkfield\nexposure settings, or to correct it with a simple numerical offset");
			Dialog.addChoice("Choose Dark BG Subtraction Option",darkBGsubopt,darkBGsubopt[2]);
			Dialog.show();
	
			inform=Dialog.getChoice();
			usrcolor=Dialog.getChoice();
			usrstyle=Dialog.getChoice();
			extdarkchoice=Dialog.getChoice();
	
			imageoption=newArray("RAW","Standard RGB","B/W");
			if(inform==formats[0]){
				raw=imageoption[0];
			}
			else{
				if(inform!=formats[0] && usrcolor==coloropt[0]){
					raw=imageoption[1];
				}
				else{
					if(inform!=formats[0] && usrcolor==coloropt[1]){
						raw=imageoption[2];
					}
				}
			}
			splitter=false;
			if(raw==imageoption[2]){
					selectedtitle="Extinction";
			}
			
			if(inform==formats[0]) exten=newArray(".3FR",".ARI",".ARW",".SRF",".SR2",".BAY",".CRW",".CR2",".CAP",".IIQ",".EIP",".DCS",".DCR",".DRF",".K25",".KDC",".DNG",".ERF",".FFF",".MEF",".MDC",".MOS",".MRW",".NEF",".NRW",".ORF",".PEF",".PTX",".PXN",".R3D",".RAF",".RAW",".RW2",".RWL",".RWZ",".SRW",".X3F");
			if(inform==formats[1] || inform==formats[10]) exten=newArray(".tif");
			if(inform==formats[2]) exten=newArray(".jpg");
			if(inform==formats[3]) exten=newArray(".gif");
			if(inform==formats[4]) exten=newArray(".png");
			if(inform==formats[5]) exten=newArray(".pgm");
			if(inform==formats[6]) exten=newArray(".bmp");
			if(inform==formats[7]) exten=newArray(".fits");
			if(inform==formats[8]) exten=newArray(".txt");
			if(inform==formats[9]) exten=newArray(".zip");
			
			if(extdarkchoice==darkBGsubopt[3]){
				Dialog.create("Dark BG Offset");
				Dialog.addNumber("Input Offset Level to be subtracted from Darkfield Image",100);
				Dialog.show();
				DBGoffset=Dialog.getNumber();
			}
			if(extdarkchoice==darkBGsubopt[0]){
				darkdefault=false;
				darkchoice=false;
				darkBGchoice=false;
				avgdarkchoice=false;
				avgdarkBGchoice=false;
				darkfolder="";
				darkBGfolder="";
			}
			if(extdarkchoice!=darkBGsubopt[0]){
				darkdefault=true;
				if(extdarkchoice==darkBGsubopt[1]){
					darkBGdefault=false;
					darkBGchoice=false;
					avgdarkBGchoice=false;
					darkBGfolder="";
				}
				if(extdarkchoice==darkBGsubopt[2]){
					darkBGdefault=true;
				}
				if(extdarkchoice==darkBGsubopt[3]){
					darkBGdefault=false;
					darkBGchoice=false;
					avgdarkBGchoice=false;
					darkBGfolder="";
				}
			}
		}
	}
	
	if(converter==false && averager==false){
		if(extanalysis==true){
			darkBGsubopt=newArray("No Darkfield images aqcuired","No darkfield BG correction","Subtract BG Image","Subtract Numerical offset");
			Dialog.create("Extinction and Darkfield Options");
			imageoption=newArray("RAW","Standard RGB","BW");
			Dialog.addChoice("Image Type",imageoption,imageoption[0]);
			Dialog.addMessage("You may indicate below whether or not you acquired any darkfield images, or, if you\nacquired darkfield images, how you would like to correct the BG offset of the \ndarkfield image. You can choose to correct it via BG images acquired using darkfield\nexposure settings, or to correct it with a simple numerical offset");
			Dialog.addChoice("Choose Dark BG Subtraction Option",darkBGsubopt,darkBGsubopt[2]);
			Dialog.show();
			extchoices=Dialog.getChoice();
			extdarkchoice=Dialog.getChoice();
		
			if(extdarkchoice==darkBGsubopt[3]){
				Dialog.create("Dark BG Offset");
				Dialog.addNumber("Input Offset Level to be subtracted from Darkfield Image",100);
				Dialog.show();
				DBGoffset=Dialog.getNumber();
			}
			if(extchoices==imageoption[2]){
				selectedtitle="Extinction";
			}
			if(extdarkchoice==darkBGsubopt[0]){
					darkdefault=false;
					darkchoice=false;
					darkBGchoice=false;
					avgdarkchoice=false;
					avgdarkBGchoice=false;
					darkfolder="";
					darkBGfolder="";
				}
			if(extdarkchoice!=darkBGsubopt[0]){
				darkdefault=true;
				if(extdarkchoice==darkBGsubopt[1]){
					darkBGdefault=false;
					darkBGchoice=false;
					avgdarkBGchoice=false;
					darkBGfolder="";
				}
				if(extdarkchoice==darkBGsubopt[2]){
					darkBGdefault=true;
				}
				if(extdarkchoice==darkBGsubopt[3]){
					darkBGdefault=false;
					darkBGchoice=false;
					avgdarkBGchoice=false;
					darkBGfolder="";
				}
			}
		}
		if(extanalysis==false && partanalysis==true){
			ImageType=newArray("Color","B/W");
			normMethod=newArray("Defocus","Shifted Sample");
			Dialog.create("Analysis Options");
			Dialog.addChoice("Choose Image Type to Analyze",ImageType,ImageType[0]);
			Dialog.addCheckbox("Run Darkfield Registration and Correlation?",true);
			Dialog.addCheckbox("Include Single Radius (Mean) Method?",false);
			Dialog.addCheckbox("Include Single Radius (Regional) Method?",false);
			Dialog.addCheckbox("Include Double Radius (Local) Method?",true);
			Dialog.addCheckbox("Single NP size?",true);
			Dialog.addChoice("Select Normalization Method",normMethod,normMethod[0]);
			mat=newArray("Silver","Gold","Non-Metallic");
			Dialog.addChoice("Choose Material to Analyze",mat,mat[1]);
			if(polarization==false){
				Dialog.setInsets(30,20,10);
				Dialog.addMessage("One-time Only Calibration Schemes:");
				Dialog.addCheckbox("Calibrate the particle measurement radius (Ri)?",false);
				Dialog.addCheckbox("Calibrate the Eta factor for Dark-Field?",false);
			}
			Dialog.show();
			type=Dialog.getChoice();
			dark=Dialog.getCheckbox();
			singleradius=Dialog.getCheckbox();;
			regional=Dialog.getCheckbox();;;
			doubleradius=Dialog.getCheckbox();;;;
			NPsize=Dialog.getCheckbox;;;;;
			normchoice=Dialog.getChoice();;
			if(type==ImageType[0]){
			matchoice=Dialog.getChoice();;;
			}
			else{
				if(type==ImageType[1]){
					matchoice="Non-Metallic";
				}
			}
			if(dark==true){
				darkdefault=true;
				darkBGdefault=true;
			}
			else{
				darkdefault=false;
				darkBGdefault=false;
			}
			if(polarization==false){
				radiuscalibration=Dialog.getCheckbox();;;;;;
				Etacalibration=Dialog.getCheckbox();;;;;;;
			}
		}
	}
	
	
	if(converter==true){
		Dialog.create("Converter");
		Dialog.setInsets(10,20,7);
		Dialog.addMessage("Choose Image sets to Convert");
		Dialog.addCheckbox("Complete for focus image set?",true);
		Dialog.addCheckbox("Complete for defocus image set?",true);
		Dialog.addCheckbox("Complete for background image set?",true);
		if(darkdefault==true){
			Dialog.addCheckbox("Complete for darkfield image set?",darkdefault);
			if(darkBGdefault==true){
				Dialog.addCheckbox("Complete for darkfield background image set?",darkBGdefault);
			}
		}
		Dialog.show();
		focuschoice=Dialog.getCheckbox();
		defocuschoice=Dialog.getCheckbox();;
		background=Dialog.getCheckbox();;;
		if(darkdefault==true){
			darkchoice=Dialog.getCheckbox();;;;
			if(darkBGdefault==true){
				darkBGchoice=Dialog.getCheckbox();;;;;
			}
		}
	}
	
	if(averager==true){
		Dialog.create("Averager");
		Dialog.setInsets(10,20,7);
		Dialog.addMessage("Choose Image sets to Average");
		Dialog.addCheckbox("Complete for focus image set?",true);
		Dialog.addCheckbox("Complete for defocus image set?",true);
		Dialog.addCheckbox("Complete for background image set?",true);
		if(darkdefault==true){
			Dialog.addCheckbox("Complete for darkfield image set?",darkdefault);
			if(darkBGdefault==true){
				Dialog.addCheckbox("Complete for darkfield background image set?",darkBGdefault);
			}
		}
		Dialog.show();
		avgfocuschoice=Dialog.getCheckbox();
		avgdefocuschoice=Dialog.getCheckbox();;
		avgbackground=Dialog.getCheckbox();;;
		if(darkdefault==true){
			avgdarkchoice=Dialog.getCheckbox();;;;
			if(darkBGdefault==true){
				avgdarkBGchoice=Dialog.getCheckbox();;;;;
			}
		}
	}
	
	
	if(converter==true && averager==false){
		if(raw==imageoption[0] || raw==imageoption[1]){
			Dialog.create("RGB Conversion Options");
			Dialog.addCheckbox("Split RGB Image?",true);
			Dialog.show();
			splitter = Dialog.getCheckbox();
		}
		if(raw==imageoption[2]){
			splitter=false;
		}
		avgfocuschoice=false;
		avgdefocuschoice=false;
		avgbackground=false;
		avgdarkchoice=false;
		avgdarkBGchoice=false;
	}
	
	else{
		if(converter==true){
			if(raw==imageoption[0] || raw==imageoption[1]){
				if(averager==true){
					splitter=true;
				}
			}
			if(raw==imageoption[2]){
				splitter = false;
			}
		}
	}
	if(converter==false){
		focuschoice=false;
		defocuschoice=false;
		background=false;
		darkchoice=false;
		darkBGchoice=false;
	}
	if(converter==false && averager==false){
		//imageoption=newArray("RAW","Standard RGB","B/W");
		//raw=imageoption[0];
		focuschoice=false;
		defocuschoice=false;
		background=false;
		darkchoice=false;
		avgfocuschoice=false;
		avgdefocuschoice=false;
		avgbackground=false;
		avgdarkchoice=false;
		avgdarkBGchoice=false;
	}
	
	if(mapping==true){
		usermapping=false;
	}
	
	if(mapping==false){
		Dialog.create("Folder Information");
		Dialog.addCheckbox("Did you create your folder mapping using the Extinction Suite?",true);
		Dialog.show();
		usermapping=Dialog.getCheckbox();
	
		if(Extmode=="Unpolarized"){
			BASEdirec=getDirectory("Choose BASE Folder");
			polarnum=1;
		}
		if(Extmode=="Polarized"){
			polbase=getDirectory("Choose BASE Folder");
		}
		if(usermapping==true){
			focusfolder="Focus";
			defocusfolder="Defocus";
			BGfolder="BG";
			if(darkdefault==true){
				darkfolder="Dark";
				if(darkBGdefault==true){
					darkBGfolder="DarkBG";
				}
			}
		}
		if(usermapping==false){
			if(converter|averager|extanalysis|partanalysis==true){
				Dialog.create("Folder Titles");
				Dialog.addString("Type title of Focus Folder", "Focus");
				Dialog.addString("Type title of Defocus Folder", "Defocus");
				Dialog.addString("Type title of BG Folder", "BG");
				if(darkdefault==true){
					Dialog.addString("Type title of Darkfield Folder","Dark");
					if(darkBGdefault==true){
						Dialog.addString("Type title of Darkfield-BG Folder","DarkBG");
					}
				}
				Dialog.show();
	
				focusfolder=Dialog.getString();
				defocusfolder=Dialog.getString();;
				BGfolder=Dialog.getString();;;
				if(darkdefault==true){
					darkfolder=Dialog.getString();;;;
					if(darkBGdefault==true){
						darkBGfolder=Dialog.getString();;;;;
					}
				}
			}
		}
	}
	if(converter|averager|extanalysis|partanalysis==true && polarization==true){
		Dialog.create("Polarization Options");
		Dialog.addNumber("Initial Angle",0);
		Dialog.addNumber("Final Focus/Defocus Angle",180);
		Dialog.addNumber("Angle Step Size",10);
		Dialog.show();
	
		polstart=Dialog.getNumber();
		polend=Dialog.getNumber();;
		polstep=Dialog.getNumber();;;
	
		polarnum=((polend-polstart)/polstep)+1;
		polnames=newArray(polarnum+2);
		polnums=newArray(polarnum);
	
		for(poloop=0;poloop<polarnum;poloop++){
			polnames[poloop]=d2s(polstart+poloop*polstep,0);
			polnums[poloop]=polstart+poloop*polstep;
		}
		polnames[polarnum]="BG";
		polnames[polarnum+1]="DarkBG";
	}
	
	imagesets=newArray(focuschoice,defocuschoice,background,darkchoice,darkBGchoice);
	setnames=newArray("focuschoice","defocuschoice","background","darkchoice","darkBGchoice");
	avgsets=newArray(avgfocuschoice,avgdefocuschoice,avgbackground,avgdarkchoice,avgdarkBGchoice);
	avgsetnames=newArray("avgfocuschoice","avgdefocuschoice","avgbackground","avgdarkchoice","avgdarkBGchoice");
	
	function listrebuilder(array,valuearray){
		numoccur=0;
		for (i=0;i<array.length;i++){
			for(j=0;j<valuearray.length;j++){
				if(endsWith(array[i],valuearray[j])){
					numoccur++;
				}
			}
		}
		if (numoccur==0){
			valueposition=newArray(1);
			valueposition[0]=0;
			return valueposition;
		}
		if (numoccur>0){
			valueposition=newArray(numoccur);
			newlist=newArray(numoccur);
			numoccur=0;
			for(i=0;i<array.length;i++){
				for(j=0;j<valuearray.length;j++){
					if(endsWith(array[i],valuearray[j])){
						valueposition[numoccur]=i;
						numoccur++;
					}
				}
			}
			for(i=0;i<numoccur;i++){
				newlist[i]=array[valueposition[i]];
			}
			return newlist;
		}
	}
	
	//////Begin Converter/Averager/Extinction Polarization Loop
	if(converter|averager|extanalysis==true){
	
		for(poloop=0;poloop<polarnum;poloop++){
			if(polarization==true){
				BASEdirec=polbase+polnames[poloop]+File.separator;
			}
			if(usermapping==true){
				focusfolder="Focus";
				defocusfolder="Defocus";
				BGfolder="BG";
				if(darkdefault==true){
					darkfolder="Dark";
					if(darkBGdefault==true){
						darkBGfolder="DarkBG";
					}
				}
			}
			if(usermapping==true || usermapping==false){
				focsrc=BASEdirec+focusfolder+File.separator+"RAW"+File.separator;
				defsrc=BASEdirec+defocusfolder+File.separator+"RAW"+File.separator;
				if(polarization==false){
					BGsrc=BASEdirec+BGfolder+File.separator+"RAW"+File.separator;
					BGdest=BASEdirec+BGfolder+File.separator;
					if(extdarkchoice==darkBGsubopt[2]){
						darkBGsrc=BASEdirec+darkBGfolder+File.separator+"RAW"+File.separator;
						darkBGdest=BASEdirec+darkBGfolder+File.separator;
					}
				}
				if(polarization==true){
					BGsrc=polbase+BGfolder+File.separator+"RAW"+File.separator;
					BGdest=polbase+BGfolder+File.separator;
					if(extdarkchoice==darkBGsubopt[2]){
						darkBGsrc=polbase+darkBGfolder+File.separator+"RAW"+File.separator;
						darkBGdest=polbase+darkBGfolder+File.separator;
					}
				}
				darsrc=BASEdirec+darkfolder+File.separator+"RAW"+File.separator;
			}
	
			focusrawexists=File.exists(BASEdirec+focusfolder+File.separator+"RAW");
			defocusrawexists=File.exists(BASEdirec+defocusfolder+File.separator+"RAW");
			if(polarization==false){
				BGrawexists=File.exists(BASEdirec+BGfolder+File.separator+"RAW");
				if(extdarkchoice==darkBGsubopt[2]){
					darkBGrawexists=File.exists(BASEdirec+darkBGfolder+File.separator+"RAW");
				}
			}
			if(polarization==true){
				BGrawexists=File.exists(polbase+BGfolder+File.separator+"RAW");
				if(extdarkchoice==darkBGsubopt[2]){
					darkBGrawexists=File.exists(polbase+darkBGfolder+File.separator+"RAW");
				}
			}
			darkrawexists=File.exists(BASEdirec+darkfolder+File.separator+"RAW");
	
			if((focuschoice==true || (focuschoice==false && avgfocuschoice==true && raw==imageoption[2])) && focusrawexists==false){
				File.makeDirectory(BASEdirec+"Focus_processing");
				File.rename(BASEdirec+focusfolder,BASEdirec+"Focus_processing"+File.separator+"RAW");
				wait(30);
				File.rename(BASEdirec+"Focus_processing",BASEdirec+focusfolder);
			}
			if((defocuschoice==true || (defocuschoice==false && avgdefocuschoice==true && raw==imageoption[2])) && defocusrawexists==false){
				File.makeDirectory(BASEdirec+"Defocus_processing");
				File.rename(BASEdirec+defocusfolder,BASEdirec+"Defocus_processing"+File.separator+"RAW");
				wait(30);
				File.rename(BASEdirec+"Defocus_processing",BASEdirec+defocusfolder);
			}
			if((background==true || (background==false && avgbackground==true && raw==imageoption[2])) && BGrawexists==false && poloop==0){
				if(polarization==false){
					File.makeDirectory(BASEdirec+"BG_processing");
					File.rename(BASEdirec+BGfolder,BASEdirec+"BG_processing"+File.separator+"RAW");
					wait(30);
					File.rename(BASEdirec+"BG_processing",BASEdirec+BGfolder);
				}
				if(polarization==true){
					File.makeDirectory(polbase+"BG_processing");
					File.rename(polbase+BGfolder,polbase+"BG_processing"+File.separator+"RAW");
					wait(30);
					File.rename(polbase+"BG_processing",polbase+BGfolder);
				}
			}
			if((darkBGchoice==true || (darkBGchoice==false && avgdarkBGchoice==true && raw==imageoption[2])) && poloop==0){
				if(darkBGrawexists==false){
					if(polarization==false){
						File.makeDirectory(BASEdirec+"darkBG_processing");
						File.rename(BASEdirec+darkBGfolder,BASEdirec+"darkBG_processing"+File.separator+"RAW");
						wait(30);
						File.rename(BASEdirec+"darkBG_processing",BASEdirec+darkBGfolder);
					}
					if(polarization==true){
						File.makeDirectory(polbase+"darkBG_processing");
						File.rename(polbase+darkBGfolder,polbase+"darkBG_processing"+File.separator+"RAW");
						wait(30);
						File.rename(polbase+"darkBG_processing",polbase+darkBGfolder);
					}
				}
			}
			if((darkchoice==true || (darkchoice==false && avgdarkchoice==true && raw==imageoption[2])) && darkrawexists==false){
				File.makeDirectory(BASEdirec+"Dark_processing");
				File.rename(BASEdirec+darkfolder,BASEdirec+"Dark_processing"+File.separator+"RAW");
				wait(30);
				File.rename(BASEdirec+"Dark_processing",BASEdirec+darkfolder);
			}
	
	// Converter Portion of Macro
	
			if(converter==true){
	/////////////////////////  general converter command section START
				function convertTo8Bit() {
					if (bitDepth==24)
						run("8-bit Color", "number=256");
					else
						run("8-bit");
				}
	
	////////////////////////  general converter command section END
	
	///////////////////////////// Focus command Start
				if(focuschoice==true){
	
		 			dir1 = focsrc;
		 			if(dir1==""){
		  				beep();
		  				exit("No Focus Image Source Specified");
		 			}
	
		  			if(averager==false){
		  				if (splitter==false || raw==imageoption[2]){
							File.makeDirectory(BASEdirec+focusfolder+File.separator+"converted");
							dir5=BASEdirec+focusfolder+File.separator+"converted"+File.separator;
						}
						if (splitter==true){
							File.makeDirectory(BASEdirec+focusfolder+File.separator+"B");
							File.makeDirectory(BASEdirec+focusfolder+File.separator+"G");
							File.makeDirectory(BASEdirec+focusfolder+File.separator+"R");
							dir2 = BASEdirec+focusfolder+File.separator+"B"+File.separator;
							dir3 = BASEdirec+focusfolder+File.separator+"G"+File.separator;
							dir4 = BASEdirec+focusfolder+File.separator+"R"+File.separator;
						}
					}
	
					if(averager==true){
	
						if(raw==imageoption[2]){
							File.makeDirectory(BASEdirec+focusfolder+File.separator+"converted");
							dir5=BASEdirec+focusfolder+File.separator+"converted"+File.separator;
						}
	
						if (raw==imageoption[0] || raw==imageoption[1]){
							File.makeDirectory(BASEdirec+focusfolder+File.separator+"B");
							File.makeDirectory(BASEdirec+focusfolder+File.separator+"G");
							File.makeDirectory(BASEdirec+focusfolder+File.separator+"R");
							dir2 = BASEdirec+focusfolder+File.separator+"B"+File.separator;
							dir3 = BASEdirec+focusfolder+File.separator+"G"+File.separator;
							dir4 = BASEdirec+focusfolder+File.separator+"R"+File.separator;
						}
					}
					prelist=getFileList(dir1);
					list=listrebuilder(prelist,exten);
					
					if(poloop==0){
						formats = newArray("TIFF", "8-bit TIFF", "JPEG", "GIF", "PNG","PGM", "BMP", "FITS", "Text Image", "ZIP", "Raw");
						Dialog.create("Focus Set Image Conversion Options");
						Dialog.addChoice("Convert to:", formats,"TIFF");
						Dialog.addNumber("Initial Image",1);
						Dialog.addNumber("Number of Images", list.length);
						Dialog.show();;
						format=Dialog.getChoice();
						imagestart = Dialog.getNumber();
						imagenumber = Dialog.getNumber();;
					}
				}
	///////////////////////focus command end
	
	///////////////////////////////// Defocus Command Start
				if(defocuschoice==true){
	
		  			dir6 = defsrc;
					if(dir6==""){
						beep();
						exit("No Defocus Image Source Specified");
					}
	
					if(averager==false){
			  			if (splitter==false || raw==imageoption[2]){
					  		File.makeDirectory(BASEdirec+defocusfolder+File.separator+"converted");
					  		dir10=BASEdirec+defocusfolder+File.separator+"converted"+File.separator;
				  		}
	
						if (splitter==true){
							File.makeDirectory(BASEdirec+defocusfolder+File.separator+"B");
							File.makeDirectory(BASEdirec+defocusfolder+File.separator+"G");
							File.makeDirectory(BASEdirec+defocusfolder+File.separator+"R");
							dir7 = BASEdirec+defocusfolder+File.separator+"B"+File.separator;
							dir8 = BASEdirec+defocusfolder+File.separator+"G"+File.separator;
							dir9 = BASEdirec+defocusfolder+File.separator+"R"+File.separator;
						}
					}
	
					if(averager==true){
	
		 				if(raw==imageoption[2]){
							File.makeDirectory(BASEdirec+defocusfolder+File.separator+"converted");
							dir10=BASEdirec+defocusfolder+File.separator+"converted"+File.separator;
						}
	
						if (raw==imageoption[0]|| raw==imageoption[1]){
							File.makeDirectory(BASEdirec+defocusfolder+File.separator+"B");
							File.makeDirectory(BASEdirec+defocusfolder+File.separator+"G");
							File.makeDirectory(BASEdirec+defocusfolder+File.separator+"R");
							dir7 = BASEdirec+defocusfolder+File.separator+"B"+File.separator;
							dir8 = BASEdirec+defocusfolder+File.separator+"G"+File.separator;
							dir9 = BASEdirec+defocusfolder+File.separator+"R"+File.separator;
						}
					}
	
					prelist1=getFileList(dir6);
					list1=listrebuilder(prelist1,exten);
		  			
					if(poloop==0){
						formats = newArray("TIFF", "8-bit TIFF", "JPEG", "GIF", "PNG","PGM", "BMP", "FITS", "Text Image", "ZIP", "Raw");
						Dialog.create("Defocus Set Image Conversion Options");
						Dialog.addChoice("Convert to:", formats,"TIFF");
						Dialog.addNumber("Initial Image",1);
						Dialog.addNumber("Number of Images", list1.length);
						Dialog.show();;
						format1=Dialog.getChoice();
						imagestart1 = Dialog.getNumber();
						imagenumber1 = Dialog.getNumber();;
					}
				}
	///////////////////defocus command end
	
	//////////////////////BG Command Start
				if(poloop==0){
	
					if(background==true){
	
						dir11 = BGsrc;
						if(dir11==""){
							beep();
							exit("No Image Source Specified");
						}
	
						if(averager==false){
							if (splitter==false || raw==imageoption[2]){
								File.makeDirectory(BGdest+"converted");
								dir15=BGdest+"converted"+File.separator;
							}
	
							if (splitter==true){
								File.makeDirectory(BGdest+"B");
								File.makeDirectory(BGdest+"G");
								File.makeDirectory(BGdest+"R");
								dir12 = BGdest+"B"+File.separator;
								dir13 = BGdest+"G"+File.separator;
								dir14 = BGdest+"R"+File.separator;
							}
						}
	
						if(averager==true){
							if(raw==imageoption[2]){
								File.makeDirectory(BGdest+"converted");
								dir15=BGdest+"converted"+File.separator;
							}
	
							if (raw==imageoption[0]|| raw==imageoption[1]){
								File.makeDirectory(BGdest+"B");
								File.makeDirectory(BGdest+"G");
								File.makeDirectory(BGdest+"R");
								dir12 = BGdest+"B"+File.separator;
								dir13 = BGdest+"G"+File.separator;
								dir14 = BGdest+"R"+File.separator;
							}
						}
	
						prelist2=getFileList(dir11);
						list2=listrebuilder(prelist2,exten);

						formats = newArray("TIFF", "8-bit TIFF", "JPEG", "GIF", "PNG","PGM", "BMP", "FITS", "Text Image", "ZIP", "Raw");
						Dialog.create("Background Set Image Conversion Options");
						Dialog.addChoice("Convert to:", formats,"TIFF");
						Dialog.addNumber("Initial Image",1);
						Dialog.addNumber("Number of Images", list2.length);
						Dialog.show();;
						format2=Dialog.getChoice();
						imagestart2 = Dialog.getNumber();
						imagenumber2 = Dialog.getNumber();;
					}
				}
	///////////////////BG Command End
	
	///////////////////Darkfield Command Start
				if(darkchoice==true){
	
					dir16 = darsrc;
					if(dir16==""){
						beep();
						exit("No Image Source Specified");
					}
	
					if(averager==false){
						if (splitter==false || raw==imageoption[2]){
							File.makeDirectory(BASEdirec+darkfolder+File.separator+"converted");
							dir20=BASEdirec+darkfolder+File.separator+"converted"+File.separator;
						}
	
						if (splitter==true){
							File.makeDirectory(BASEdirec+darkfolder+File.separator+"B");
							File.makeDirectory(BASEdirec+darkfolder+File.separator+"G");
							File.makeDirectory(BASEdirec+darkfolder+File.separator+"R");
							dir17 = BASEdirec+darkfolder+File.separator+"B"+File.separator;
							dir18 = BASEdirec+darkfolder+File.separator+"G"+File.separator;
							dir19 = BASEdirec+darkfolder+File.separator+"R"+File.separator;
						}
					}
	
					if(averager==true){
						if(raw==imageoption[2]){
							File.makeDirectory(BASEdirec+darkfolder+File.separator+"converted");
							dir20=BASEdirec+darkfolder+File.separator+"converted"+File.separator;
						}
	
						if (raw==imageoption[0]|| raw==imageoption[1]){
							File.makeDirectory(BASEdirec+darkfolder+File.separator+"B");
							File.makeDirectory(BASEdirec+darkfolder+File.separator+"G");
							File.makeDirectory(BASEdirec+darkfolder+File.separator+"R");
							dir17 = BASEdirec+darkfolder+File.separator+"B"+File.separator;
							dir18 = BASEdirec+darkfolder+File.separator+"G"+File.separator;
							dir19 = BASEdirec+darkfolder+File.separator+"R"+File.separator;
						}
					}
	
					prelist3=getFileList(dir16);
					list3=listrebuilder(prelist3,exten);
					
					if(poloop==0){
						formats = newArray("TIFF", "8-bit TIFF", "JPEG", "GIF", "PNG","PGM", "BMP", "FITS", "Text Image", "ZIP", "Raw");
						Dialog.create("Darkfield Set Image Conversion Options");
						Dialog.addChoice("Convert to:", formats,"TIFF");
						Dialog.addNumber("Initial Image",1);
						Dialog.addNumber("Number of Images", list3.length);
						Dialog.show();;
						format3=Dialog.getChoice();
						imagestart3 = Dialog.getNumber();
						imagenumber3 = Dialog.getNumber();;
					}
				}
	///////////////////Darkfield Command End
	
	//////////////////////DarkBG Command Start
				if(poloop==0){
	
					if(darkBGchoice==true){
	
						dir21 = darkBGsrc;
						if(dir21==""){
							beep();
							exit("No Image Source Specified");
						}
	
						if(averager==false){
							if (splitter==false || raw==imageoption[2]){
							File.makeDirectory(darkBGdest+"converted");
							dir25=darkBGdest+"converted"+File.separator;
							}
	
							if(splitter==true){
								File.makeDirectory(darkBGdest+"B");
								File.makeDirectory(darkBGdest+"G");
								File.makeDirectory(darkBGdest+"R");
								dir22 = darkBGdest+"B"+File.separator;
								dir23 = darkBGdest+"G"+File.separator;
								dir24 = darkBGdest+"R"+File.separator;
							}
						}
	
						if(averager==true){
							if(raw==imageoption[2]){
								File.makeDirectory(darkBGdest+"converted");
								dir25=darkBGdest+"converted"+File.separator;
							}
	
							if(raw==imageoption[0]|| raw==imageoption[1]){
								File.makeDirectory(darkBGdest+"B");
								File.makeDirectory(darkBGdest+"G");
								File.makeDirectory(darkBGdest+"R");
								dir22 = darkBGdest+"B"+File.separator;
								dir23 = darkBGdest+"G"+File.separator;
								dir24 = darkBGdest+"R"+File.separator;
							}
						}
						prelist4=getFileList(dir21);
						list4=listrebuilder(prelist4,exten);
						
						formats = newArray("TIFF", "8-bit TIFF", "JPEG", "GIF", "PNG","PGM", "BMP", "FITS", "Text Image", "ZIP", "Raw");
						Dialog.create("DarkBG Set Image Conversion Options");
						Dialog.addChoice("Convert to:", formats,"TIFF");
						Dialog.addNumber("Initial Image",1);
						Dialog.addNumber("Number of Images", list4.length);
						Dialog.show();;
						format4=Dialog.getChoice();
						imagestart4 = Dialog.getNumber();
						imagenumber4 = Dialog.getNumber();;
					}
				}
	///////////////////DarkBG Command End
			}
	/////////////////////////////////////////////////////Converter Command END
	
	////////////////////////////////////////////////////Averager Command START
	//////////////////////  general averager command section START
			if(averager==true){
	
				if(raw==imageoption[2]){
					File.makeDirectory(BASEdirec+"Averages");
					if(polarization==true && poloop==0){
						File.makeDirectory(BGdest+"Averages");
						if(avgdarkBGchoice==true && poloop==0){
							File.makeDirectory(darkBGdest+"Averages");
						}
					}
				}
	//////////////////////  general averager command section END
	
	////////////////////////////////////////Averager Focus Command START
				if(avgfocuschoice==true){
					selection=raw;
	
					if(selection==imageoption[0] || selection==imageoption[1]){
	
						if(converter==false){
							dir2=BASEdirec+focusfolder+File.separator+"B"+File.separator;
							dir3=BASEdirec+focusfolder+File.separator+"G"+File.separator;
							dir4=BASEdirec+focusfolder+File.separator+"R"+File.separator;
	
							prelistB=getFileList(dir2);
							prelistG=getFileList(dir3);
							prelistR=getFileList(dir4);
							
							listblue=listrebuilder(prelistB,exten);
							listgreen=listrebuilder(prelistG,exten);
							listred=listrebuilder(prelistR,exten);
						}
						if(converter==true){
							if(focuschoice==false){
								dir2=BASEdirec+focusfolder+File.separator+"B"+File.separator;
								dir3=BASEdirec+focusfolder+File.separator+"G"+File.separator;
								dir4=BASEdirec+focusfolder+File.separator+"R"+File.separator;
	
								prelistB=getFileList(dir2);
								prelistG=getFileList(dir3);
								prelistR=getFileList(dir4);
								
								listblue=listrebuilder(prelistB,exten);
								listgreen=listrebuilder(prelistG,exten);
								listred=listrebuilder(prelistR,exten);
							}
						}
	
						if(poloop==0){
							Dialog.create("Focus Set Image options");
							Dialog.addCheckbox("Include Merged Averaged Image?",false);
							Dialog.addCheckbox("Delete Individual Converted Files after Averaging?",false);
							Dialog.addNumber("Initial Image",1);
							if(converter==true){
								if(focuschoice==true){
									Dialog.addNumber("Number of Images", imagenumber);
								}
							}
							if(converter==false){
								if(usrstyle==imstyle[0]){
									Dialog.addNumber("Number of Images", listblue.length);
								}
								if(usrstyle==imstyle[1]){
									Dialog.addNumber("Number of Image Stacks to open", listblue.length);
									Dialog.addNumber("Number of Slices to Average",3);
								}
							}
							if(converter==true){
								if(focuschoice==false){
									if(usrstyle==imstyle[0]){
										Dialog.addNumber("Number of Images", listblue.length);
									}
									if(usrstyle==imstyle[1]){
										Dialog.addNumber("Number of Image Stacks to open", listblue.length);
										Dialog.addNumber("Number of Slices to Average",3);
									}
								}
							}
							Dialog.show();
							merge=Dialog.getCheckbox();
							condelete=Dialog.getCheckbox();;
							imagestart11=Dialog.getNumber();
							imagenumber11=Dialog.getNumber();;
							if(usrstyle==imstyle[1]){
								slicecount=Dialog.getNumber();;;
							}
						}
						File.makeDirectory(BASEdirec+focusfolder+File.separator+"Averages");
						solodir=BASEdirec+focusfolder+File.separator+"Averages"+File.separator;
					}
					else{
						if(selection==imageoption[2]){
	
							if(converter==false){
								singledir=focsrc;
								prelist=getFileList(singledir);
								listsingle=listrebuilder(prelist,exten);
							}
	
							if(converter==true){
								if(focuschoice==false){
									singledir=focsrc;
									prelist=getFileList(singledir);
									listsingle=listrebuilder(prelist,exten);
								}
							}
	
							if(converter==true){
								if(focuschoice==true){
									singledir=dir5;
									prelist=getFileList(singledir);
									listsingle=listrebuilder(prelist,exten);
								}
							}
	
							savedir=BASEdirec+"Averages"+File.separator;
	
							if(poloop==0){
								Dialog.create("Average Focus Image Options");
								Dialog.addCheckbox("Delete Individual Converted Files after Averaging?",false);
								if(usrstyle==imstyle[1]){
									Dialog.addMessage("You have indicated that input images are stack files.\nPlease input below the initial file and number of stack files to open.\nThe default number of stack files to open is 1. However, you may choose to open\nseveral seperate stack files, which ImageJ will concatenate into a single larger stack.\nSo, finally, please indicate the total number of slices you wish to average over.");
								}
								Dialog.addNumber("Select Initial Image",1);
								if(converter==true){
									if(focuschoice==true){
										Dialog.addNumber("Number of Images", imagenumber);
									}
								}
								if(converter==false){
									if(usrstyle==imstyle[0]){
										Dialog.addNumber("Number of Images", listsingle.length);
									}
									if(usrstyle==imstyle[1]){
										Dialog.addNumber("Number of Image Stacks to open", listsingle.length);
										Dialog.addNumber("Number of Slices to Average",3);
									}
								}
								if(converter==true){
									if(focuschoice==false){
										if(usrstyle==imstyle[0]){
											Dialog.addNumber("Number of Images", listsingle.length);
										}
										if(usrstyle==imstyle[1]){
											Dialog.addNumber("Number of Image Stacks to open", listsingle.length);
											Dialog.addNumber("Number of Slices to Average",3);
										}
									}
								}
								Dialog.show();
								condelete=Dialog.getCheckbox();
								preferredtitle="AVG_focus";
								imagestart11=Dialog.getNumber();
								imagenumber11=Dialog.getNumber();;
								if(usrstyle==imstyle[1]){
									slicecount=Dialog.getNumber();;;
								}
							}
						}
					}
				}
	//////////////////////////////Averager Focus Command END
	
	///////////////////////////////////////////////////////////Average deFocus Command START
				if(avgdefocuschoice==true){
	
					selection1=raw;
	
					if(selection1==imageoption[0] || selection1==imageoption[1]){
	
						if(converter==false){
							dir7=BASEdirec+defocusfolder+File.separator+"B"+File.separator;
							dir8=BASEdirec+defocusfolder+File.separator+"G"+File.separator;
							dir9=BASEdirec+defocusfolder+File.separator+"R"+File.separator;
	
							prelistB=getFileList(dir7);
							prelistG=getFileList(dir8);
							prelistR=getFileList(dir9);
							
							listblue=listrebuilder(prelistB,exten);
							listgreen=listrebuilder(prelistG,exten);
							listred=listrebuilder(prelistR,exten);
						}
	
						if(converter==true){
							if(defocuschoice==false){
								dir7=BASEdirec+defocusfolder+File.separator+"B"+File.separator;
								dir8=BASEdirec+defocusfolder+File.separator+"G"+File.separator;
								dir9=BASEdirec+defocusfolder+File.separator+"R"+File.separator;
	
								prelistB=getFileList(dir7);
								prelistG=getFileList(dir8);
								prelistR=getFileList(dir9);
								
								listblue=listrebuilder(prelistB,exten);
								listgreen=listrebuilder(prelistG,exten);
								listred=listrebuilder(prelistR,exten);
							}
						}
	
						if(poloop==0){
							Dialog.create("Defocus Set Image Options");
							Dialog.addCheckbox("Include Merged Averaged Image?",false);
							Dialog.addCheckbox("Delete Individual Converted Files after Averaging?",false);
							Dialog.addNumber("Initial Image",1);
							if(converter==true){
								if(defocuschoice==true){
									Dialog.addNumber("Number of Images", imagenumber1);
								}
							}
							if(converter==false){
								if(usrstyle==imstyle[0]){
									Dialog.addNumber("Number of Images", listblue.length);
								}
								if(usrstyle==imstyle[1]){
									Dialog.addNumber("Number of Image Stacks to open", listblue.length);
									Dialog.addNumber("Number of Slices to Average",3);
								}
							}
							if(converter==true){
								if(focuschoice==false){
									if(usrstyle==imstyle[0]){
										Dialog.addNumber("Number of Images", listblue.length);
									}
									if(usrstyle==imstyle[1]){
										Dialog.addNumber("Number of Image Stacks to open", listblue.length);
										Dialog.addNumber("Number of Slices to Average",3);
									}
								}
							}
							Dialog.show();
							merge1=Dialog.getCheckbox();
							condelete1=Dialog.getCheckbox();;
							imagestart12=Dialog.getNumber();
							imagenumber12=Dialog.getNumber();;
							if(usrstyle==imstyle[1]){
								slicecount1=Dialog.getNumber();;;
							}
						}
	
						File.makeDirectory(BASEdirec+defocusfolder+File.separator+"Averages")
						solodir1=BASEdirec+defocusfolder+File.separator+"Averages"+File.separator;
	
					}
					else{
						if(selection1==imageoption[2]){
	
							if(converter==false){
								singledir1=defsrc;
								prelist1=getFileList(singledir1);
								listsingle1=listrebuilder(prelist1,exten);
							}
	
							if(converter==true){
								if(defocuschoice==false){
									singledir1=defsrc;
									prelist1=getFileList(singledir1);
									listsingle1=listrebuilder(prelist1,exten);
								}
							}
	
							if(converter==true){
								if(defocuschoice==true){
									singledir1=dir10;
									prelist1=getFileList(singledir1);
									listsingle1=listrebuilder(prelist1,exten);
								}
							}
	
							savedir1=BASEdirec+"Averages"+File.separator;
	
							if(poloop==0){
								Dialog.create("Average Defocus Image Options");
								Dialog.addCheckbox("Delete Individual Converted Files after Averaging?",false);
								if(usrstyle==imstyle[1]){
									Dialog.addMessage("You have indicated that input images are stack files.\nPlease input below the initial file and number of stack files to open.\nThe default number of stack files to open is 1. However, you may choose to open\nseveral seperate stack files, which ImageJ will concatenate into a single larger stack.\nSo, finally, please indicate the total number of slices you wish to average over.");
								}
								Dialog.addNumber("Select Initial Image",1);
								if(converter==true){
									if(defocuschoice==true){
										Dialog.addNumber("Number of Images", imagenumber1);
									}
								}
								if(converter==false){
									if(usrstyle==imstyle[0]){
										Dialog.addNumber("Number of Images", listsingle1.length);
									}
									if(usrstyle==imstyle[1]){
										Dialog.addNumber("Number of Image Stacks to open", listsingle1.length);
										Dialog.addNumber("Number of Slices to Average",3);
									}
								}
								if(converter==true){
									if(focuschoice==false){
										if(usrstyle==imstyle[0]){
											Dialog.addNumber("Number of Images", listsingle1.length);
										}
										if(usrstyle==imstyle[1]){
											Dialog.addNumber("Number of Image Stacks to open", listsingle1.length);
											Dialog.addNumber("Number of Slices to Average",3);
										}
									}
								}
								Dialog.show();
								condelete1=Dialog.getCheckbox();
								preferredtitle1="AVG_defocus";
								imagestart12=Dialog.getNumber();
								imagenumber12=Dialog.getNumber();;
								if(usrstyle==imstyle[1]){
									slicecount1=Dialog.getNumber();;;
								}
							}
						}
					}
				}
	////////////////////////////////////////////////////////////Average deFocus Command END
	
	//////////////////////////////////////////////////////////////Average BG Command START
				if(poloop==0){
	
					if(avgbackground==true){
	
					selection2=raw;
	
	
						if(selection2==imageoption[0] || selection2==imageoption[1]){
	
							if(converter==false){
								dir12=BGdest+"B"+File.separator;
								dir13=BGdest+"G"+File.separator;
								dir14=BGdest+"R"+File.separator;
	
								prelistB=getFileList(dir12);
								prelistG=getFileList(dir13);
								prelistR=getFileList(dir14);
								
								listblue=listrebuilder(prelistB,exten);
								listgreen=listrebuilder(prelistG,exten);
								listred=listrebuilder(prelistR,exten);
							}
	
							if(converter==true){
								if(background==false){
									dir12=BGdest+"B"+File.separator;
									dir13=BGdest+"G"+File.separator;
									dir14=BGdest+"R"+File.separator;
	
									prelistB=getFileList(dir12);
									prelistG=getFileList(dir13);
									prelistR=getFileList(dir14);
								
									listblue=listrebuilder(prelistB,exten);
									listgreen=listrebuilder(prelistG,exten);
									listred=listrebuilder(prelistR,exten);
								}
							}
	
							Dialog.create("Background Set Image Options");
							Dialog.addCheckbox("Include Merged Averaged Image?",false);
							Dialog.addCheckbox("Delete Individual Converted Files after Averaging?",false);
							Dialog.addNumber("Initial Image",1);
							if(converter==true){
								if(background==true){
									Dialog.addNumber("Number of Images", imagenumber2);
								}
							}
							if(converter==false){
								if(usrstyle==imstyle[0]){
									Dialog.addNumber("Number of Images", listblue.length);
								}
								if(usrstyle==imstyle[1]){
									Dialog.addNumber("Number of Image Stacks to open", listblue.length);
									Dialog.addNumber("Number of Slices to Average",3);
								}
							}
							if(converter==true){
								if(focuschoice==false){
									if(usrstyle==imstyle[0]){
										Dialog.addNumber("Number of Images", listblue.length);
									}
									if(usrstyle==imstyle[1]){
										Dialog.addNumber("Number of Image Stacks to open", listblue.length);
										Dialog.addNumber("Number of Slices to Average",3);
									}
								}
							}
							Dialog.show();
							merge2=Dialog.getCheckbox();
							condelete2=Dialog.getCheckbox();;
							imagestart13=Dialog.getNumber();
							imagenumber13=Dialog.getNumber();;
							if(usrstyle==imstyle[1]){
								slicecount2=Dialog.getNumber();;;
							}
							
							File.makeDirectory(BGdest+"Averages");
							solodir2=BGdest+"Averages"+File.separator;
						}
						else{
							if(selection2==imageoption[2]){
	
								if(converter==false){
									singledir2=BGsrc;
									prelist2=getFileList(singledir2);
									listsingle2=listrebuilder(prelist2,exten);
								}
	
								if(converter==true){
									if(background==false){
										singledir2=BGsrc;
										prelist2=getFileList(singledir2);
										listsingle2=listrebuilder(prelist2,exten);
									}
								}
	
								if(converter==true){
									if(background==true){
										singledir2=dir15;
										prelist2=getFileList(singledir2);
										listsingle2=listrebuilder(prelist2,exten);
									}
								}
	
								if(polarization==false){
									savedir2=BASEdirec+"Averages"+File.separator;
								}
								if(polarization==true){
									savedir2=BGdest+"Averages"+File.separator;
								}
	
								Dialog.create("Average BG Image Options");
								Dialog.addCheckbox("Delete Individual Converted Files after Averaging?",false);
								Dialog.addMessage("You have indicated that input images are stack files.\nPlease input below the initial file and number of stack files to open.\nThe default number of stack files to open is 1. However, you may choose to open\nseveral seperate stack files, which ImageJ will concatenate into a single larger stack.\nSo, finally, please indicate the total number of slices you wish to average over.");
								Dialog.addNumber("Select Initial Image",1);
								if(converter==true){
									if(background==true){
										Dialog.addNumber("Number of Images", imagenumber2);
									}
								}
								if(converter==false){
									if(usrstyle==imstyle[0]){
										Dialog.addNumber("Number of Images", listsingle2.length);
									}
									if(usrstyle==imstyle[1]){
										Dialog.addNumber("Number of Image Stacks to open", listsingle2.length);
										Dialog.addNumber("Number of Slices to Average",3);
									}
								}
								if(converter==true){
									if(focuschoice==false){
										if(usrstyle==imstyle[0]){
											Dialog.addNumber("Number of Images", listsingle2.length);
										}
										if(usrstyle==imstyle[1]){
											Dialog.addNumber("Number of Image Stacks to open", listsingle2.length);
											Dialog.addNumber("Number of Slices to Average",3);
										}
									}
								}
								Dialog.show();
								condelete2=Dialog.getCheckbox();
								preferredtitle2="AVG_background";
								imagestart13=Dialog.getNumber();
								imagenumber13=Dialog.getNumber();;
								if(usrstyle==imstyle[1]){
									slicecount2=Dialog.getNumber();;;
								}
							}
						}
					}
				}
	////////////////////////////////////Average BG Command END
	
	//////////////////////////////////////////////////////////////Average Darkfield Command START
				if(avgdarkchoice==true){
	
					selection3=raw;
	
	
					if(selection3==imageoption[0] || selection3==imageoption[1]){
	
						if(converter==false){
							dir17=BASEdirec+darkfolder+File.separator+"B"+File.separator;
							dir18=BASEdirec+darkfolder+File.separator+"G"+File.separator;
							dir19=BASEdirec+darkfolder+File.separator+"R"+File.separator;
	
							prelistB=getFileList(dir17);
							prelistG=getFileList(dir18);
							prelistR=getFileList(dir19);
							
							listblue=listrebuilder(prelistB,exten);
							listgreen=listrebuilder(prelistG,exten);
							listred=listrebuilder(prelistR,exten);
						}
	
						if(converter==true){
							if(background==false){
								dir17=BASEdirec+darkfolder+File.separator+"B"+File.separator;
								dir18=BASEdirec+darkfolder+File.separator+"G"+File.separator;
								dir19=BASEdirec+darkfolder+File.separator+"R"+File.separator;
	
								prelistB=getFileList(dir17);
								prelistG=getFileList(dir18);
								prelistR=getFileList(dir19);
								
								listblue=listrebuilder(prelistB,exten);
								listgreen=listrebuilder(prelistG,exten);
								listred=listrebuilder(prelistR,exten);
							}
						}
	
						if(poloop==0){
							Dialog.create("Darkfield Set Image Options");
							Dialog.addCheckbox("Include Merged Averaged Image?",false);
							Dialog.addCheckbox("Delete Individual Converted Files after Averaging?",false);
							Dialog.addNumber("Initial Image",1);
							if(converter==true){
								if(darkchoice==true){
									Dialog.addNumber("Number of Images", imagenumber3);
								}
							}
							if(converter==false){
								if(usrstyle==imstyle[0]){
									Dialog.addNumber("Number of Images", listblue.length);
								}
								if(usrstyle==imstyle[1]){
									Dialog.addNumber("Number of Image Stacks to open", listblue.length);
									Dialog.addNumber("Number of Slices to Average",3);
								}
							}
							if(converter==true){
								if(focuschoice==false){
									if(usrstyle==imstyle[0]){
										Dialog.addNumber("Number of Images", listblue.length);
									}
									if(usrstyle==imstyle[1]){
										Dialog.addNumber("Number of Image Stacks to open", listblue.length);
										Dialog.addNumber("Number of Slices to Average",3);
									}
								}
							}
							Dialog.show();
							merge3=Dialog.getCheckbox();
							condelete3=Dialog.getCheckbox();;
							imagestart14=Dialog.getNumber();
							imagenumber14=Dialog.getNumber();;
							if(usrstyle==imstyle[1]){
								slicecount3=Dialog.getNumber();;;
							}
						}
	
						File.makeDirectory(BASEdirec+darkfolder+File.separator+"Averages");
						solodir3=BASEdirec+darkfolder+File.separator+"Averages"+File.separator;
					}
					else{
						if(selection3==imageoption[2]){
	
							if(converter==false){
								singledir3=darsrc;
								prelist3=getFileList(singledir3);
								listsingle3=listrebuilder(prelist3,exten);
							}
	
							if(converter==true){
								if(darkchoice==false){
									singledir3=darsrc;
									prelist3=getFileList(singledir3);
									listsingle3=listrebuilder(prelist3,exten);
								}
							}
	
							if(converter==true){
								if(darkchoice==true){
									singledir3=dir20;
									prelist3=getFileList(singledir3);
									listsingle3=listrebuilder(prelist3,exten);
								}
							}
	
							savedir3=BASEdirec+"Averages"+File.separator;
	
							if(poloop==0){
								Dialog.create("Darkfield Image Number");
								Dialog.addCheckbox("Delete Individual Converted Files after Averaging?",false);
								Dialog.addMessage("You have indicated that input images are stack files.\nPlease input below the initial file and number of stack files to open.\nThe default number of stack files to open is 1. However, you may choose to open\nseveral seperate stack files, which ImageJ will concatenate into a single larger stack.\nSo, finally, please indicate the total number of slices you wish to average over.");
								Dialog.addNumber("Select Initial Image",1);
								if(converter==true){
									if(darkchoice==true){
										Dialog.addNumber("Number of Images", imagenumber3);
									}
								}
								if(converter==false){
									if(usrstyle==imstyle[0]){
										Dialog.addNumber("Number of Images", listsingle3.length);
									}
									if(usrstyle==imstyle[1]){
										Dialog.addNumber("Number of Image Stacks to open", listsingle3.length);
										Dialog.addNumber("Number of Slices to Average",3);
									}
								}
								if(converter==true){
									if(focuschoice==false){
										if(usrstyle==imstyle[0]){
											Dialog.addNumber("Number of Images", listsingle3.length);
										}
										if(usrstyle==imstyle[1]){
											Dialog.addNumber("Number of Image Stacks to open", listsingle3.length);
											Dialog.addNumber("Number of Slices to Average",3);
										}
									}
								}
								Dialog.show();
								condelete3=Dialog.getCheckbox();
								preferredtitle3="AVG_dark";
								imagestart14=Dialog.getNumber();
								imagenumber14=Dialog.getNumber();;
								if(usrstyle==imstyle[1]){
									slicecount3=Dialog.getNumber();;;
								}
							}
						}
					}
				}
	////////////////////////////////////Average Darkfield Command END
	
	//////////////////////////////////////////////////////////////Average DarkBG Command START
				if(poloop==0){
					if(avgdarkBGchoice==true){
	
						selection4=raw;
	
						if(selection4==imageoption[0] || selection4==imageoption[1]){
							
							if(converter==false){
								dir22=darkBGdest+"B"+File.separator;
								dir23=darkBGdest+"G"+File.separator;
								dir24=darkBGdest+"R"+File.separator;
	
								prelistB=getFileList(dir22);
								prelistG=getFileList(dir23);
								prelistR=getFileList(dir24);
								
								listblue=listrebuilder(prelistB,exten);
								listgreen=listrebuilder(prelistG,exten);
								listred=listrebuilder(prelistR,exten);
							}
	
							if(converter==true){
								if(darkBGchoice==false){
									dir22=darkBGdest+"B"+File.separator;
									dir23=darkBGdest+"G"+File.separator;
									dir24=darkBGdest+"R"+File.separator;
	
									prelistB=getFileList(dir22);
									prelistG=getFileList(dir23);
									prelistR=getFileList(dir24);
									
									listblue=listrebuilder(prelistB,exten);
									listgreen=listrebuilder(prelistG,exten);
									listred=listrebuilder(prelistR,exten);
								}
							}
	
							Dialog.create("Dark BG Set Image Options");
							Dialog.addCheckbox("Include Merged Averaged Image?",false);
							Dialog.addCheckbox("Delete Individual Converted Files after Averaging?",false);
							Dialog.addNumber("Initial Image",1);
							if(converter==true){
								if(darkBGchoice==true){
									Dialog.addNumber("Number of Images", imagenumber4);
								}
							}
							if(converter==false){
								if(usrstyle==imstyle[0]){
									Dialog.addNumber("Number of Images", listblue.length);
								}
								if(usrstyle==imstyle[1]){
									Dialog.addNumber("Number of Image Stacks to open", listblue.length);
									Dialog.addNumber("Number of Slices to Average",3);
								}
							}
							if(converter==true){
								if(focuschoice==false){
									if(usrstyle==imstyle[0]){
										Dialog.addNumber("Number of Images", listblue.length);
									}
									if(usrstyle==imstyle[1]){
										Dialog.addNumber("Number of Image Stacks to open", listblue.length);
										Dialog.addNumber("Number of Slices to Average",3);
									}
								}
							}
							Dialog.show();
							merge4=Dialog.getCheckbox();
							condelete4=Dialog.getCheckbox();;
							imagestart15=Dialog.getNumber();
							imagenumber15=Dialog.getNumber();;
							if(usrstyle==imstyle[1]){
								slicecount4=Dialog.getNumber();;;
							}
							File.makeDirectory(darkBGdest+"Averages");
							solodir4=darkBGdest+"Averages"+File.separator;
	
						}
						else{
							if(selection4==imageoption[2]){
	
								if(converter==false){
									singledir4=darkBGsrc;
									prelist4=getFileList(singledir4);
									listsingle4=listrebuilder(prelist4,exten);
								}
	
								if(converter==true){
									if(darkBGchoice==false){
										singledir4=darkBGsrc;
										prelist4=getFileList(singledir4);
										listsingle4=listrebuilder(prelist4,exten);
									}
								}
	
								if(converter==true){
									if(darkBGchoice==true){
										singledir4=dir25;
										prelist4=getFileList(singledir4);
										listsingle4=listrebuilder(prelist4,exten);
									}
								}
	
								if(polarization==false){
									savedir4=BASEdirec+"Averages"+File.separator;
								}
								if(polarization==true){
									savedir4=darkBGdest+"Averages"+File.separator;
								}
	
									Dialog.create("Average Dark BG Image Options");
									Dialog.addCheckbox("Delete Individual Converted Files after Averaging?",false);
									Dialog.addMessage("You have indicated that input images are stack files.\nPlease input below the initial file and number of stack files to open.\nThe default number of stack files to open is 1. However, you may choose to open\nseveral seperate stack files, which ImageJ will concatenate into a single larger stack.\nSo, finally, please indicate the total number of slices you wish to average over.");
									Dialog.addNumber("Select Initial Image",1);
								if(converter==true){
									if(darkBGchoice==true){
										Dialog.addNumber("Number of Images", imagenumber4);
									}
								}
								if(converter==false){
									if(usrstyle==imstyle[0]){
										Dialog.addNumber("Number of Images", listsingle4.length);
									}
									if(usrstyle==imstyle[1]){
										Dialog.addNumber("Number of Image Stacks to open", listsingle4.length);
										Dialog.addNumber("Number of Slices to Average",3);
									}
								}
								if(converter==true){
									if(focuschoice==false){
										if(usrstyle==imstyle[0]){
											Dialog.addNumber("Number of Images", listsingle4.length);
										}
										if(usrstyle==imstyle[1]){
											Dialog.addNumber("Number of Image Stacks to open", listsingle4.length);
											Dialog.addNumber("Number of Slices to Average",3);
										}
									}
								}
								Dialog.show();
								condelete4=Dialog.getCheckbox();
								preferredtitle4="AVG_darkBG";
								imagestart15=Dialog.getNumber();
								imagenumber15=Dialog.getNumber();;
								if(usrstyle==imstyle[1]){
									slicecount4=Dialog.getNumber();;;
								}
							}
						}
					}
				}
	////////////////////////////////////Average DarkBG Command END
			}
	////////////////////////////////////////////////////Averager Command END
	
	//////////
			for(sets=0;sets<5;sets++){
	//////////////////////////////////////////////// Converter Run START
				if(converter==true){
	/////////////////////////////////focus run start
					if(imagesets[sets]==true){
	
						if(setnames[sets]=="focuschoice"){
							dirA=dir1; if(raw!=imageoption[2]){dirB=dir2; dirC=dir3; dirD=dir4;}  if(raw==imageoption[2]){dirE=dir5;}
							LIST=list; Format=format; IMSTART=imagestart; IMNUM=imagenumber;
						}
						if(setnames[sets]=="defocuschoice"){
							dirA=dir6; if(raw!=imageoption[2]){dirB=dir7; dirC=dir8; dirD=dir9;} if(raw==imageoption[2]){dirE=dir10;}
							LIST=list1; Format=format1; IMSTART=imagestart1; IMNUM=imagenumber1;
						}
						if(setnames[sets]=="background" && poloop==0){
							dirA=dir11; if(raw!=imageoption[2]){dirB=dir12; dirC=dir13; dirD=dir14;} if(raw==imageoption[2]){dirE=dir15;}
							LIST=list2; Format=format2; IMSTART=imagestart2; IMNUM=imagenumber2;
						}
						if(setnames[sets]=="darkchoice"){
							dirA=dir16; if(raw!=imageoption[2]){dirB=dir17; dirC=dir18; dirD=dir19;} if(raw==imageoption[2]){dirE=dir20;}
							LIST=list3; Format=format3; IMSTART=imagestart3; IMNUM=imagenumber3;
						}
						if(setnames[sets]=="darkBGchoice" && poloop==0){
							dirA=dir21; if(raw!=imageoption[2]){dirB=dir22; dirC=dir23; dirD=dir24;} if(raw==imageoption[2]){dirE=dir25;}
							LIST=list4; Format=format4; IMSTART=imagestart4; IMNUM=imagenumber4;
						}
	
						setBatchMode(true);
						for (i=IMSTART-1; i<IMSTART-1+IMNUM; i++) {
							showProgress(i-IMSTART+1, IMNUM);
							pathFull=dirA+LIST[i];
							if (raw==imageoption[0]){
								run("DCRaw Reader...", "open=pathFull use_temporary_directory white=[Camera white balance] output_colorspace=raw document_mode_without_scaling read_as=[16-bit linear] interpolation=[High-speed, low-quality bilinear] half_size");
							}
							else{
								open(pathFull);
							}
							if(averager==false && (splitter==false || raw==imageoption[2])){
								if(usrstyle==imstyle[0]){
									Title4=getTitle();
							     	if (startsWith(Title4,"dcraw")){
							     		rawstring4 = indexOf(Title4,".CR2");
							     		rawtitle4 = substring(Title4,rawstring4-8,rawstring4);
							     		if (Format=="8-bit TIFF" || Format=="GIF"){
							        		convertTo8Bit();
							     		}
							        	saveAs(Format, dirE+rawtitle4);
							        	close();
							     	}
							     	else {
							     		if (Format=="8-bit TIFF" || Format=="GIF"){
							        		convertTo8Bit();
							     		}
							        	saveAs(Format, dirE+Title4);
							        	close();
							     	}
								}
								if(usrstyle==imstyle[1]){
									if (Format=="8-bit TIFF" || Format=="GIF"){
							        	convertTo8Bit();
							     	}
							     	run("Image Sequence... ", "format=&Format start=1 save=/Users/Lukas/Desktop/test");
							       	close();
								}
							}
	
							if(averager==true && raw==imageoption[2]){
								if(usrstyle==imstyle[0]){
									Title4=getTitle();
							     	if (startsWith(Title4,"dcraw")){
							     		rawstring4 = indexOf(Title4,".CR2");
							     		rawtitle4 = substring(Title4,rawstring4-8,rawstring4);
							     		if(Format=="8-bit TIFF" || Format=="GIF"){
							        		convertTo8Bit();
							     		}
							        		saveAs(Format, dirE+rawtitle4);
							        		close();
							     	}
							     	else{
							     		if(Format=="8-bit TIFF" || Format=="GIF"){
							        		convertTo8Bit();
							     		}
							        	saveAs(Format, dirE+Title4);
							        	close();
							     	}
								}
						     	if(usrstyle==imstyle[1]){
									if (Format=="8-bit TIFF" || Format=="GIF"){
							        	convertTo8Bit();
							     	}
							     	run("Image Sequence... ", "format=&Format start=1 save=/Users/Lukas/Desktop/test");
							       	close();
								}
							}
	
							if(splitter==true){
								run("Split Channels");
						     	if(raw!=imageoption[0]){
						        	if (Format=="8-bit TIFF" || Format=="GIF"){
						        		convertTo8Bit();
						        	}
						     	}
								Title1=getTitle();
						     	if (startsWith(Title1,"C3")){
						     			rawstring1 = indexOf(Title1,".CR2");
						     			rawtitle1 = substring(Title1,rawstring1-8,rawstring1);
						        		saveAs(Format,dirB+"blue_"+rawtitle1);
						        		close();
						        }
						        else {
						        	if (endsWith(Title1,"(blue)")){
						        		bluestring = indexOf(Title1,"(blue)");
						        		bluetitle = substring(Title1,0,bluestring);
						        		saveAs(Format,dirB+"blue_"+bluetitle);
						        		close();
						        	}
						    		else {
						     			if(startsWith(Title1,"C2")){
						     				rawstring1 = indexOf(Title1,".CR2");
						     				rawtitle1 = substring(Title1,rawstring1-8,rawstring1);
						     				saveAs(Format, dirC+"green_"+rawtitle1);
						     				close();
						     			}
						     			else{
						     				if (endsWith(Title1,"(green)")){
							     				greenstring = indexOf(Title1,"(green)");
							     				greentitle = substring(Title1,0,greenstring);
							     				saveAs(Format,dirC+"green_"+greentitle);
							     				close();
						     				}
						     				else {
						     					if (startsWith(Title1,"C1")){
						     						rawstring1 = indexOf(Title1,".CR2");
						     						rawtitle1 = substring(Title1,rawstring1-8,rawstring1);
						     						saveAs(Format, dirD+"red_"+rawtitle1);
						     						close();
						     					}
						     					else {
						     							if (endsWith(Title1,"(red)")){
						     								redstring = indexOf(Title1,"(red)");
						     								redtitle = substring(Title1,0,redstring);
						     								saveAs(Format,dirD+"red_"+redtitle);
						     								close();
						     							}
						     						}
						     					}
						     				}
						     			}
						        	}
								Title2=getTitle();
						     	if (startsWith(Title2,"C3")){
						     		rawstring2 = indexOf(Title2,".CR2");
						     		rawtitle2=substring(Title2,rawstring2-8,rawstring2);
						        	saveAs(Format, dirB+"blue_"+rawtitle2);
						        	close();
						        }
						        else {
						        	if (endsWith(Title2,"(blue)")){
						        		bluestring2 = indexOf(Title2,"(blue)");
						        		bluetitle2 = substring(Title2,0,bluestring2);
						        		saveAs(Format,dirB+"blue_"+bluetitle2);
						        		close();
						        	}
									else {
						     			if (startsWith(Title2,"C2")){
						     				rawstring2 = indexOf(Title2,".CR2");
						    				rawtitle2=substring(Title2,rawstring2-8,rawstring2);
						     				saveAs(Format, dirC+"green_"+rawtitle2);
						     				close();
										}
						     			else{
						     				if (endsWith(Title2,"(green)")){
						     					greenstring2 = indexOf(Title2,"(green)");
						     					greentitle2 = substring(Title2,0,greenstring2);
						     					saveAs(Format,dirC+"green_"+greentitle2);
						     					close();
						     				}
						     				else {
						     					if (startsWith(Title2,"C1")){
						     						rawstring2 = indexOf(Title2,".CR2");
						     						rawtitle2=substring(Title2,rawstring2-8,rawstring2);
						     						saveAs(Format, dirD+"red_"+rawtitle2);
						     						close();
						     					}
						     					else {
						     						if (endsWith(Title2,"(red)")){
						     							redstring2 = indexOf(Title2,"(red)");
						     							redtitle2 = substring(Title2,0,redstring2);
						     							saveAs(Format,dirD+"red_"+redtitle2);
						     							close();
						     						}
						     					}
						     				}
						     			}
						     		}
						        }
						        Title3=getTitle();
						     	if (startsWith(Title3,"C3")){
						     		rawstring3= indexOf(Title3,".CR2");
						     		rawtitle3 = substring(Title3,rawstring3-8,rawstring3);
						        	saveAs(Format, dirB+"blue_"+rawtitle3);
						        	close();
						        }
						        else {
						        	if (endsWith(Title3,"(blue)")){
						        		bluestring3 = indexOf(Title3,"(blue)");
						        		bluetitle3 = substring(Title3,0,bluestring3);
						        		saveAs(Format,dirB+"blue_"+bluetitle3);
						        		close();
						       		}
						    		else {
						     			if (startsWith(Title3,"C2")){
						     				rawstring3= indexOf(Title3,".CR2");
						     				rawtitle3 = substring(Title3,rawstring3-8,rawstring3);
						     				saveAs(Format, dirC+"green_"+rawtitle3);
						     				close();
						     			}
						     			else{
						     				if (endsWith(Title3,"(green)")){
						     					greenstring3 = indexOf(Title3,"(green)");
						     					greentitle3 = substring(Title3,0,greenstring3);
						     					saveAs(Format,dirC+"green_"+greentitle3);
						     					close();
						     				}
						     				else {
						     					if (startsWith(Title3,"C1")){
						     						rawstring3= indexOf(Title3,".CR2");
						     						rawtitle3 = substring(Title3,rawstring3-8,rawstring3);
						     						saveAs(Format, dirD+"red_"+rawtitle3);
						     						close();
						     					}
												else {
						     						if (endsWith(Title3,"(red)")){
						     							redstring3 = indexOf(Title3,"(red)");
						     							redtitle3 = substring(Title3,0,redstring3);
						     							saveAs(Format,dirD+"red_"+redtitle3);
						     							close();
						     						}
						     					}
						     				}
						     			}
						     		}
						        }
						    }
						}
					}
			//////////////////////////////// Focus Run End
				}
	///////////// CONVERTER RUN END
	
	
	//////////////////////////////////////////////////////////////////////Averaging Portion of Macro
	
				if(averager==true){
	
	/////////////////////////////// Averager Focus Run START
					if(avgsets[sets]==true){
	
						if(avgsetnames[sets]=="avgfocuschoice"){
							if(converter==true){dirA=dir1;} if(raw!=imageoption[2]){dirB=dir2; dirC=dir3; dirD=dir4; Merge=merge;SoloDir=solodir;}
							IMstart=imagestart11; IMnum=imagenumber11; if(usrstyle==imstyle[1]){SliceCount=slicecount;}
							if(raw==imageoption[2]){
								//dirE=dir5;
								SingleDir=singledir; ListSingle=listsingle; SaveDir=savedir; PTitle=preferredtitle;
							}
							mergename="merged_avg_focus"; deletion=condelete;
						}
						if(avgsetnames[sets]=="avgdefocuschoice"){
							if(converter==true){dirA=dir6;} if(raw!=imageoption[2]){dirB=dir7; dirC=dir8; dirD=dir9; Merge=merge1;SoloDir=solodir1;}
							IMstart=imagestart12; IMnum=imagenumber12; if(usrstyle==imstyle[1]){SliceCount=slicecount1;}
							if(raw==imageoption[2]){
								//dirE=dir10;
								SingleDir=singledir1; ListSingle=listsingle1; SaveDir=savedir1; PTitle=preferredtitle1;
							}
							mergename="merged_avg_defocus"; deletion=condelete1;
						}
						if(avgsetnames[sets]=="avgbackground" && poloop==0){
							if(converter==true){dirA=dir11;} if(raw!=imageoption[2]){dirB=dir12; dirC=dir13; dirD=dir14; Merge=merge2;SoloDir=solodir2;}
							IMstart=imagestart13; IMnum=imagenumber13; if(usrstyle==imstyle[1]){SliceCount=slicecount2;}
							if(raw==imageoption[2]){
								//dirE=dir15;
								SingleDir=singledir2; ListSingle=listsingle2; SaveDir=savedir2; PTitle=preferredtitle2;
							}
							mergename="merged_avg_BG"; deletion=condelete2;
						}
						if(avgsetnames[sets]=="avgdarkchoice"){
							if(converter==true){dirA=dir16;} if(raw!=imageoption[2]){dirB=dir17; dirC=dir18; dirD=dir19; Merge=merge3;SoloDir=solodir3;}
							IMstart=imagestart14; IMnum=imagenumber14; if(usrstyle==imstyle[1]){SliceCount=slicecount3;}
							if(raw==imageoption[2]){
								//dirE=dir20;
								SingleDir=singledir3; ListSingle=listsingle3; SaveDir=savedir3; PTitle=preferredtitle3;
							}
							mergename="merged_avg_dark"; deletion=condelete3;
						}
						if(avgsetnames[sets]=="avgdarkBGchoice" && poloop==0){
							if(converter==true){dirA=dir21;} if(raw!=imageoption[2]){dirB=dir22; dirC=dir23; dirD=dir24; Merge=merge4;SoloDir=solodir4;}
							IMstart=imagestart14; IMnum=imagenumber14; if(usrstyle==imstyle[1]){SliceCount=slicecount4;}
							if(raw==imageoption[2]){
								//dirE=dir25;
								SingleDir=singledir4; ListSingle=listsingle4; SaveDir=savedir4; PTitle=preferredtitle4;
							}
							mergename="merged_avg_darkBG"; deletion=condelete4;
						}
	
						if(raw==imageoption[0] || raw==imageoption[1]){
		
							setBatchMode(true);
							q=IMstart-1;
							imnum11=IMstart-1+IMnum;
		
							pathFullblue=dirB;
							pathFullgreen=dirC;
							pathFullred=dirD;
							
							run("Image Sequence...", "open=pathFullblue number=imnum11 starting=q increment=1 scale=100 file=[] or=[] sort");
							bluetitle=getTitle();
							selectWindow(bluetitle);
							rename("blue");
							if(usrstyle==imstyle[0]){
								run("Z Project...", "start=1 stop=imnum11 projection=[Average Intensity]");
							}
							if(usrstyle==imstyle[1]){
								run("Z Project...", "start=1 stop=SliceCount projection=[Average Intensity]");
							}
							selectWindow("AVG_blue");
							saveAs("TIFF",SoloDir+"AVG_blue");
							close();
							close();
		
		
							run("Image Sequence...", "open=pathFullgreen number=imnum11 starting=q increment=1 scale=100 file=[] or=[] sort");
							greentitle=getTitle();
							selectWindow(greentitle);
							rename("green");
							if(usrstyle==imstyle[0]){
								run("Z Project...", "start=1 stop=imnum11 projection=[Average Intensity]");
							}
							if(usrstyle==imstyle[1]){
								run("Z Project...", "start=1 stop=SliceCount projection=[Average Intensity]");
							}
							selectWindow("AVG_green");
		
							saveAs("TIFF",SoloDir+"AVG_green");
							close();
							close();
		
		
							run("Image Sequence...", "open=pathFullred number=imnum11 starting=q increment=1 scale=100 file=[] or=[] sort");
							redtitle=getTitle();
							selectWindow(redtitle);
							rename("red");
							if(usrstyle==imstyle[0]){
								run("Z Project...", "start=1 stop=imnum11 projection=[Average Intensity]");
							}
							if(usrstyle==imstyle[1]){
								run("Z Project...", "start=1 stop=SliceCount projection=[Average Intensity]");
							}
							selectWindow("AVG_red");
		
							saveAs("TIFF",SoloDir+"AVG_red");
							close();
							close();
		
							if (Merge==true){
								open(SoloDir+"AVG_red.tif");
								open(SoloDir+"AVG_green.tif");
								open(SoloDir+"AVG_blue.tif");
								run("Merge Channels...", "red=AVG_red.tif green=AVG_green.tif blue=AVG_blue.tif gray=*None* create");
								saveAs("TIFF",SoloDir+mergename);
								close();
							}
							if(deletion==true){
								delblue=getFileList(pathFullblue);
								delgreen=getFileList(pathFullgreen);
								delred=getFileList(pathFullred);
								for(del=0;del<imnum11;del++){
									File.delete(pathFullblue+delblue[del]);
									File.delete(pathFullgreen+delgreen[del]);
									File.delete(pathFullred+delred[del]);
								}
							}
						}
						else{
							if(selection==imageoption[2]){
								pathBW=BASEdirec+File.separator+SingleDir+File.separator;
								setBatchMode(true);
								q=IMstart-1;
								imnum11=IMstart-1+IMnum;
								run("Image Sequence...", "open=SingleDir number=imnum11 starting=q increment=1 scale=100 file=[] or=[] sort");
								singletitle=getTitle();
								selectWindow(singletitle);
								if(usrstyle==imstyle[0]){
									run("Z Project...", "start=1 stop=imnum11 projection=[Average Intensity]");
								}
								if(usrstyle==imstyle[1]){
									run("Z Project...", "start=1 stop=SliceCount projection=[Average Intensity]");
								}
								selectWindow("AVG_"+singletitle);
								saveAs("TIFF",SaveDir+PTitle);
								close();
								close();
								if(deletion==true){
									dellist=getFileList(SingleDir);
									for(del=0;del<imnum11;del++){
										File.delete(pathBW+dellist[del]);
									}
								}
							}
						}
					}
	//////////Averager END
				}
	///////////////////////////////////////////////////////////Loop End
			}
	
	////Extinction Analysis Portion of macro
	
			if(extanalysis==true){
	
				File.makeDirectory(BASEdirec+"Extinction");
	
				if(converter==true || averager==true){
					extchoices=raw;
				}
	
				setBatchMode(true);
	
				if(extchoices!=imageoption[2]){
	
					if(polarization==false){
						blueextname=newArray("blue_extinction.tif");
						greenextname=newArray("green_extinction.tif");
						redextname=newArray("red_extinction.tif");
	
						if(extdarkchoice!=darkBGsubopt[0]){
							bluedarkname=newArray("blue_darkfield.tif");
							greendarkname=newArray("green_darkfield.tif");
							reddarkname=newArray("red_darkfield.tif");
	
							bluedefname=newArray("blue_AvgDefocus_BGsub.tif");
							greendefname=newArray("green_AvgDefocus_BGsub.tif");
							reddefname=newArray("red_AvgDefocus_BGsub.tif");
						}
					}
	
					if(polarization==true){
						blueextname=newArray(polarnum);
						greenextname=newArray(polarnum);
						redextname=newArray(polarnum);
						if(extdarkchoice!=darkBGsubopt[0]){
							bluedarkname=newArray(polarnum);
							greendarkname=newArray(polarnum);
							reddarkname=newArray(polarnum);
	
							bluedefname=newArray(polarnum);
							greendefname=newArray(polarnum);
							reddefname=newArray(polarnum);
						}
	
							blueextname[poloop]="blue_extinction_"+polnames[poloop]+".tif";
							greenextname[poloop]="green_extinction_"+polnames[poloop]+".tif";
							redextname[poloop]="red_extinction_"+polnames[poloop]+".tif";
						if(extdarkchoice!=darkBGsubopt[0]){
							bluedarkname[poloop]="blue_darkfield_"+polnames[poloop]+".tif";
							greendarkname[poloop]="green_darkfield_"+polnames[poloop]+".tif";
							reddarkname[poloop]="red_darkfield_"+polnames[poloop]+".tif";
	
							bluedefname[poloop]="blue_AvgDefocus_BGsub_"+polnames[poloop]+".tif";
							greendefname[poloop]="green_AvgDefocus_BGsub_"+polnames[poloop]+".tif";
							reddefname[poloop]="red_AvgDefocus_BGsub_"+polnames[poloop]+".tif";
						}
					}
	
				////////////
				if(averager==true){
					//////////
					if(avgfocuschoice==true){
						bluefocuspath=solodir+"AVG_blue.tif";
						greenfocuspath=solodir+"AVG_green.tif";
						redfocuspath=solodir+"AVG_red.tif";
					}
					else{
						if(avgfocuschoice==false){
							multipleimage=File.exists(BASEdirec+focusfolder+File.separator+"Averages");
							if(multipleimage==true){
								bluefocuspath=BASEdirec+focusfolder+File.separator+"Averages"+File.separator+"AVG_blue.tif";
								greenfocuspath=BASEdirec+focusfolder+File.separator+"Averages"+File.separator+"AVG_green.tif";
								redfocuspath=BASEdirec+focusfolder+File.separator+"Averages"+File.separator+"AVG_red.tif";
							}
							if(multipleimage==false){
								extfoclistb=getFileList(BASEdirec+focusfolder+File.separator+"B");
								extfoclistg=getFileList(BASEdirec+focusfolder+File.separator+"G");
								extfoclistr=getFileList(BASEdirec+focusfolder+File.separator+"R");
								bluefocuspath=BASEdirec+focusfolder+File.separator+"B"+File.separator+extfoclistb[0];
								greenfocuspath=BASEdirec+focusfolder+File.separator+"G"+File.separator+extfoclistg[0];
								redfocuspath=BASEdirec+focusfolder+File.separator+"R"+File.separator+extfoclistr[0];
							}
						}
					}
					//////////
					if(avgdefocuschoice==true){
						bluedefocuspath=solodir1+"AVG_blue.tif";
						greendefocuspath=solodir1+"AVG_green.tif";
						reddefocuspath=solodir1+"AVG_red.tif";
					}
					else{
						if(avgdefocuschoice==false){
							multipleimage=File.exists(BASEdirec+defocusfolder+File.separator+"Averages");
							if(multipleimage==true){
								bluedefocuspath=BASEdirec+defocusfolder+File.separator+"Averages"+File.separator+"AVG_blue.tif";
								greendefocuspath=BASEdirec+defocusfolder+File.separator+"Averages"+File.separator+"AVG_green.tif";
								reddefocuspath=BASEdirec+defocusfolder+File.separator+"Averages"+File.separator+"AVG_red.tif";
							}
							if(multipleimage==false){
								extdefoclistb=getFileList(BASEdirec+defocusfolder+File.separator+"B");
								extdefoclistg=getFileList(BASEdirec+defocusfolder+File.separator+"G");
								extdefoclistr=getFileList(BASEdirec+defocusfolder+File.separator+"R");
								bluedefocuspath=BASEdirec+defocusfolder+File.separator+"B"+File.separator+extdefoclistb[0];
								greendefocuspath=BASEdirec+defocusfolder+File.separator+"G"+File.separator+extdefoclistg[0];
								reddefocuspath=BASEdirec+defocusfolder+File.separator+"R"+File.separator+extdefoclistr[0];
							}
						}
					}
					//////////
					if(avgbackground==true){
						bluebackpath=solodir2+"AVG_blue.tif";
						greenbackpath=solodir2+"AVG_green.tif";
						redbackpath=solodir2+"AVG_red.tif";
					}
					else{
						if(avgbackground==false){
							multipleimage=File.exists(BGdest+"Averages");
							if(multipleimage==true){
								bluebackpath=BGdest+"Averages"+File.separator+"AVG_blue.tif";
								greenbackpath=BGdest+"Averages"+File.separator+"AVG_green.tif";
								redbackpath=BGdest+"Averages"+File.separator+"AVG_red.tif";
							}
							if(multipleimage==false){
								extbglistb=getFileList(BGdest+"B");
								extbglistg=getFileList(BGdest+"G");
								extbglistr=getFileList(BGdest+"R");
								bluebackpath=BGdest+"B"+File.separator+extbglistb[0];
								greenbackpath=BGdest+"G"+File.separator+extbglistg[0];
								redbackpath=BGdest+"R"+File.separator+extbglistr[0];
							}
						}
					}
					//////////
					if(avgdarkchoice==true){
						blueDarkpath=solodir3+"AVG_blue.tif";
						greenDarkpath=solodir3+"AVG_green.tif";
						redDarkpath=solodir3+"AVG_red.tif";
					}
					else{
						if(avgdarkchoice==false){
							if(extdarkchoice!=darkBGsubopt[0]){
								multipleimage=File.exists(BASEdirec+darkfolder+File.separator+"Averages");
								if(multipleimage==true){
									blueDarkpath=BASEdirec+darkfolder+File.separator+"Averages"+File.separator+"AVG_blue.tif";
									greenDarkpath=BASEdirec+darkfolder+File.separator+"Averages"+File.separator+"AVG_green.tif";
									redDarkpath=BASEdirec+darkfolder+File.separator+"Averages"+File.separator+"AVG_red.tif";
								}
								if(multipleimage==false){
									extdarklistb=getFileList(BASEdirec+darkfolder+File.separator+"B");
									extdarklistg=getFileList(BASEdirec+darkfolder+File.separator+"G");
									extdarklistr=getFileList(BASEdirec+darkfolder+File.separator+"R");
									blueDarkpath=BASEdirec+darkfolder+File.separator+"B"+File.separator+extdarklistb[0];
									greenDarkpath=BASEdirec+darkfolder+File.separator+"G"+File.separator+extdarklistg[0];
									redDarkpath=BASEdirec+darkfolder+File.separator+"R"+File.separator+extdarklistr[0];
								}
							}
						}
					}
					//////////
					if(avgdarkBGchoice==true){
						bluedarkBGpath=solodir4+"AVG_blue.tif";
						greendarkBGpath=solodir4+"AVG_green.tif";
						reddarkBGpath=solodir4+"AVG_red.tif";
					}
					else{
						if(avgdarkBGchoice==false){
								if(extdarkchoice==darkBGsubopt[2]){
								multipleimage=File.exists(darkBGdest+"Averages");
								if(multipleimage==true){
									bluedarkBGpath=darkBGdest+"Averages"+File.separator+"AVG_blue.tif";
									greendarkBGpath=darkBGdest+"Averages"+File.separator+"AVG_green.tif";
									reddarkBGpath=darkBGdest+"Averages"+File.separator+"AVG_red.tif";
								}
								if(multipleimage==false){
									extdarkBGlistb=getFileList(darkBGdest+"B");
									extdarkBGlistg=getFileList(darkBGdest+"G");
									extdarkBGlistr=getFileList(darkBGdest+"R");
									bluedarkBGpath=darkBGdest+"B"+File.separator+extdarkBGlistb[0];
									greendarkBGpath=darkBGdest+"G"+File.separator+extdarkBGlistg[0];
									reddarkBGpath=darkBGdest+"R"+File.separator+extdarkBGlistr[0];
								}
							}
						}
					}
					//////////
				}
	
				else{
					if(converter==false){
						if(averager==false){
							multipleimage=File.exists(BASEdirec+focusfolder+File.separator+"Averages");
							if(multipleimage==true){
								bluefocuspath=BASEdirec+focusfolder+File.separator+"Averages"+File.separator+"AVG_blue.tif";
								greenfocuspath=BASEdirec+focusfolder+File.separator+"Averages"+File.separator+"AVG_green.tif";
								redfocuspath=BASEdirec+focusfolder+File.separator+"Averages"+File.separator+"AVG_red.tif";
							}
							if(multipleimage==false){
								extfoclistb=getFileList(BASEdirec+focusfolder+File.separator+"B");
								extfoclistg=getFileList(BASEdirec+focusfolder+File.separator+"G");
								extfoclistr=getFileList(BASEdirec+focusfolder+File.separator+"R");
								bluefocuspath=BASEdirec+focusfolder+File.separator+"B"+File.separator+extfoclistb[0];
								greenfocuspath=BASEdirec+focusfolder+File.separator+"G"+File.separator+extfoclistg[0];
								redfocuspath=BASEdirec+focusfolder+File.separator+"R"+File.separator+extfoclistr[0];
							}
	
							multipleimage=File.exists(BASEdirec+defocusfolder+File.separator+"Averages");
							if(multipleimage==true){
								bluedefocuspath=BASEdirec+defocusfolder+File.separator+"Averages"+File.separator+"AVG_blue.tif";
								greendefocuspath=BASEdirec+defocusfolder+File.separator+"Averages"+File.separator+"AVG_green.tif";
								reddefocuspath=BASEdirec+defocusfolder+File.separator+"Averages"+File.separator+"AVG_red.tif";
							}
							if(multipleimage==false){
								extbglistb=getFileList(BASEdirec+defocusfolder+File.separator+"B");
								extbglistg=getFileList(BASEdirec+defocusfolder+File.separator+"G");
								extbglistr=getFileList(BASEdirec+defocusfolder+File.separator+"R");
								bluebackpath=BASEdirec+defocusfolder+File.separator+"B"+File.separator+extbglistb[0];
								greenbackpath=BASEdirec+defocusfolder+File.separator+"G"+File.separator+extbglistg[0];
								redbackpath=BASEdirec+defocusfolder+File.separator+"R"+File.separator+extbglistr[0];
							}
	
							multipleimage=File.exists(BGdest+"Averages");
							if(multipleimage==true){
								bluebackpath=BGdest+"Averages"+File.separator+"AVG_blue.tif";
								greenbackpath=BGdest+"Averages"+File.separator+"AVG_green.tif";
								redbackpath=BGdest+"Averages"+File.separator+"AVG_red.tif";
							}
							if(multipleimage==false){
								extbglistb=getFileList(BGdest+"B");
								extbglistg=getFileList(BGdest+"G");
								extbglistr=getFileList(BGdest+"R");
								bluebackpath=BGdest+"B"+File.separator+extbglistb[0];
								greenbackpath=BGdest+"G"+File.separator+extbglistg[0];
								redbackpath=BGdest+"R"+File.separator+extbglistr[0];
							}
							if(extdarkchoice!=darkBGsubopt[0]){
								multipleimage=File.exists(BASEdirec+darkfolder+File.separator+"Averages");
								if(multipleimage==true){
									blueDarkpath=BASEdirec+darkfolder+File.separator+"Averages"+File.separator+"AVG_blue.tif";
									greenDarkpath=BASEdirec+darkfolder+File.separator+"Averages"+File.separator+"AVG_green.tif";
									redDarkpath=BASEdirec+darkfolder+File.separator+"Averages"+File.separator+"AVG_red.tif";
								}
								if(multipleimage==false){
									extdarklistb=getFileList(BASEdirec+darkfolder+File.separator+"B");
									extdarklistg=getFileList(BASEdirec+darkfolder+File.separator+"G");
									extdarklistr=getFileList(BASEdirec+darkfolder+File.separator+"R");
									blueDarkpath=BASEdirec+darkfolder+File.separator+"B"+File.separator+extdarklistb[0];
									greenDarkpath=BASEdirec+darkfolder+File.separator+"G"+File.separator+extdarklistg[0];
									redDarkpath=BASEdirec+darkfolder+File.separator+"R"+File.separator+extdarklistr[0];
								}
							}
							if(extdarkchoice==darkBGsubopt[2]){
								multipleimage=File.exists(darkBGdest+"Averages");
								if(multipleimage==true){
									bluedarkBGpath=darkBGdest+"Averages"+File.separator+"AVG_blue.tif";
									greendarkBGpath=darkBGdest+"Averages"+File.separator+"AVG_green.tif";
									reddarkBGpath=darkBGdest+"Averages"+File.separator+"AVG_red.tif";
								}
								if(multipleimage==false){
									extdarkBGlistb=getFileList(darkBGdest+"B");
									extdarkBGlistg=getFileList(darkBGdest+"G");
									extdarkBGlistr=getFileList(darkBGdest+"R");
									bluedarkBGpath=darkBGdest+"B"+File.separator+extdarkBGlistb[0];
									greendarkBGpath=darkBGdest+"G"+File.separator+extdarkBGlistg[0];
									reddarkBGpath=darkBGdest+"R"+File.separator+extdarkBGlistr[0];
								}
							}
						}
					}
					if(converter==true){
						if(averager==false){
							extfoclistb=getFileList(BASEdirec+focusfolder+File.separator+"B");
							extfoclistg=getFileList(BASEdirec+focusfolder+File.separator+"G");
							extfoclistr=getFileList(BASEdirec+focusfolder+File.separator+"R");
							bluefocuspath=BASEdirec+focusfolder+File.separator+"B"+File.separator+extfoclistb[0];
							greenfocuspath=BASEdirec+focusfolder+File.separator+"G"+File.separator+extfoclistg[0];
							redfocuspath=BASEdirec+focusfolder+File.separator+"R"+File.separator+extfoclistr[0];
	
							extdefoclistb=getFileList(BASEdirec+defocusfolder+File.separator+"B");
							extdefoclistg=getFileList(BASEdirec+defocusfolder+File.separator+"G");
							extdefoclistr=getFileList(BASEdirec+defocusfolder+File.separator+"R");
							bluedefocuspath=BASEdirec+defocusfolder+File.separator+"B"+File.separator+extdefoclistb[0];
							greendefocuspath=BASEdirec+defocusfolder+File.separator+"G"+File.separator+extdefoclistg[0];
							reddefocuspath=BASEdirec+defocusfolder+File.separator+"R"+File.separator+extdefoclistr[0];
	
							extbglistb=getFileList(BASEdirec+BGfolder+File.separator+"B");
							extbglistg=getFileList(BASEdirec+BGfolder+File.separator+"G");
							extbglistr=getFileList(BASEdirec+BGfolder+File.separator+"R");
							bluebackpath=BASEdirec+BGfolder+File.separator+"B"+File.separator+extbglistb[0];
							greenbackpath=BASEdirec+BGfolder+File.separator+"G"+File.separator+extbglistg[0];
							redbackpath=BASEdirec+BGfolder+File.separator+"R"+File.separator+extbglistr[0];
							if(extdarkchoice!=darkBGsubopt[0]){
								extdarklistb=getFileList(BASEdirec+darkfolder+File.separator+"B");
								extdarklistg=getFileList(BASEdirec+darkfolder+File.separator+"G");
								extdarklistr=getFileList(BASEdirec+darkfolder+File.separator+"R");
								blueDarkpath=BASEdirec+darkfolder+File.separator+"B"+File.separator+extdarklistb[0];
								greenDarkpath=BASEdirec+darkfolder+File.separator+"G"+File.separator+extdarklistg[0];
								redDarkpath=BASEdirec+darkfolder+File.separator+"R"+File.separator+extdarklistr[0];
								if(extdarkchoice==darkBGsubopt[2]){
									extdarkBGlistb=getFileList(BASEdirec+darkBGfolder+File.separator+"B");
									extdarkBGlistg=getFileList(BASEdirec+darkBGfolder+File.separator+"G");
									extdarkBGlistr=getFileList(BASEdirec+darkBGfolder+File.separator+"R");
									bluedarkBGpath=BASEdirec+darkBGfolder+File.separator+"B"+File.separator+extdarkBGlistb[0];
									greendarkBGpath=BASEdirec+darkBGfolder+File.separator+"G"+File.separator+extdarkBGlistg[0];
									reddarkBGpath=BASEdirec+darkBGfolder+File.separator+"R"+File.separator+extdarkBGlistr[0];
								}
							}
						}
					}
				}
				//////////////////
	
				savepath=BASEdirec+"Extinction"+File.separator;
	
				// Open focus & BG images and subtract BG
				open(bluefocuspath);
				rename("bluefocus");
				open(bluebackpath);
				rename("blueBG");
				imageCalculator("Subtract create 32-bit", "bluefocus","blueBG");
				selectWindow("bluefocus");
				close();
	
	
				open(greenfocuspath);
				rename("greenfocus");
				open(greenbackpath);
				rename("greenBG");
				imageCalculator("Subtract create 32-bit", "greenfocus","greenBG");
				selectWindow("greenfocus");
				close();
	
				open(redfocuspath);
				rename("redfocus");
				open(redbackpath);
				rename("redBG");
				imageCalculator("Subtract create 32-bit", "redfocus","redBG");
				selectWindow("redfocus");
				close();
	
				//Open defocus images and subtract BG
				open(bluedefocuspath);
				rename("blueDefocus");
				imageCalculator("Subtract create 32-bit", "blueDefocus", "blueBG");
				selectWindow("blueDefocus");
				close();
				selectWindow("blueBG");
				close();
				if(extdarkchoice!=darkBGsubopt[0]){
					saveAs("Tiff", BASEdirec+defocusfolder+File.separator+bluedefname[poloop]);
				}
				newbluedef=getTitle();
				
	
				open(greendefocuspath);
				rename("greenDefocus");
				imageCalculator("Subtract create 32-bit", "greenDefocus", "greenBG");
				selectWindow("greenDefocus");
				close();
				selectWindow("greenBG");
				close();
				if(extdarkchoice!=darkBGsubopt[0]){
					saveAs("Tiff", BASEdirec+defocusfolder+File.separator+greendefname[poloop]);
				}
				newgreendef=getTitle();
	
				open(reddefocuspath);
				rename("redDefocus");
				imageCalculator("Subtract create 32-bit", "redDefocus", "redBG");
				selectWindow("redDefocus");
				close();
				selectWindow("redBG");
				close();
				if(extdarkchoice!=darkBGsubopt[0]){
					saveAs("Tiff", BASEdirec+defocusfolder+File.separator+reddefname[poloop]);
				}
				newreddef=getTitle();
	
				//Open darkfield images, convert to 32-bit, save images (if no BG correction requested)
				if(extdarkchoice==darkBGsubopt[1]){
					open(blueDarkpath);
					rename("blueDark");
					run("32-bit");
					selectWindow("blueDark");
					saveAs("Tiff", savepath+bluedarkname[poloop]);
					close();
	
					open(greenDarkpath);
					rename("greenDark");
					run("32-bit");
					selectWindow("greenDark");
					saveAs("Tiff", savepath+greendarkname[poloop]);
					close();
	
					open(redDarkpath);
					rename("redDark");
					run("32-bit");
					selectWindow("redDark");
					saveAs("Tiff", savepath+reddarkname[poloop]);
					close();
				}
	
				//Open darkfield images and subtract darkBG image (if requested)
				if(extdarkchoice==darkBGsubopt[2]){
					open(blueDarkpath);
					rename("blueDark");
					open(bluedarkBGpath);
					rename("bluedarkBG");
					imageCalculator("Subtract create 32-bit", "blueDark", "bluedarkBG");
					selectWindow("Result of blueDark");
					saveAs("Tiff", savepath+bluedarkname[poloop]);
					close();
					selectWindow("blueDark");
					close();
					selectWindow("bluedarkBG");
					close();
	
					open(greenDarkpath);
					rename("greenDark");
					open(greendarkBGpath);
					rename("greendarkBG");
					imageCalculator("Subtract create 32-bit", "greenDark", "greendarkBG");
					selectWindow("Result of greenDark");
					saveAs("Tiff", savepath+greendarkname[poloop]);
					close();
					selectWindow("greenDark");
					close();
					selectWindow("greendarkBG");
					close();
	
					open(redDarkpath);
					rename("redDark");
					open(reddarkBGpath);
					rename("reddarkBG");
					imageCalculator("Subtract create 32-bit", "redDark", "reddarkBG");
					selectWindow("Result of redDark");
					saveAs("Tiff", savepath+reddarkname[poloop]);
					close();
					selectWindow("redDark");
					close();
					selectWindow("reddarkBG");
					close();
				}
	
				//Open darkfield images, convert to 32-bit, and subtract numerical offset (if requested)
				if(extdarkchoice==darkBGsubopt[3]){
					open(blueDarkpath);
					rename("blueDark");
					run("32-bit");
					run("Subtract...", "value=DBGoffset");
					selectWindow("blueDark");
					saveAs("Tiff", savepath+bluedarkname[poloop]);
					close();
	
					open(greenDarkpath);
					rename("greenDark");
					run("32-bit");
					selectWindow("greenDark");
					run("Subtract...", "value=DBGoffset");
					saveAs("Tiff", savepath+greendarkname[poloop]);
					close();
	
					open(redDarkpath);
					rename("redDark");
					run("32-bit");
					selectWindow("redDark");
					run("Subtract...", "value=DBGoffset");
					saveAs("Tiff", savepath+reddarkname[poloop]);
					close();
				}
	
				// Run extinction analysis
	
				imageCalculator("Divide create 32-bit", "Result of bluefocus", newbluedef);
				selectWindow("Result of Result of bluefocus");
				rename("blue_transmission");
				run("Multiply...", "value=-1");
				run("Add...", "value=1");
				rename("blue_extinction");
				selectWindow("Result of bluefocus");
				close();
				selectWindow(newbluedef);
				close();
				selectWindow("blue_extinction");
				saveAs("Tiff", savepath+blueextname[poloop]);
				close();
	
				imageCalculator("Divide create 32-bit", "Result of greenfocus", newgreendef);
				selectWindow("Result of Result of greenfocus");
				rename("green_transmission");
				run("Multiply...", "value=-1");
				run("Add...", "value=1");
				rename("green_extinction");
				selectWindow("Result of greenfocus");
				close();
				selectWindow(newgreendef);
				close();
				selectWindow("green_extinction");
				saveAs("Tiff", savepath+greenextname[poloop]);
				close();
	
				imageCalculator("Divide create 32-bit", "Result of redfocus", newreddef);
				selectWindow("Result of Result of redfocus");
				rename("red_transmission");
				run("Multiply...", "value=-1");
				run("Add...", "value=1");
				rename("red_extinction");
				selectWindow("Result of redfocus");
				close();
				selectWindow(newreddef);
				close();
				selectWindow("red_extinction");
				saveAs("Tiff", savepath+redextname[poloop]);
				close();
	
				}
	
				else{
					if(extchoices==imageoption[2]){
	
						if(polarization==false){
							BWextname=newArray("Extinction.tif");
							if(extdarkchoice!=darkBGsubopt[0]){
								BWdarkname=newArray("BWdarkfield.tif");
								BWdefname=newArray("AvgDefocus_BGsub.tif");
							}
						}
	
						if(polarization==true){
							BWextname=newArray(polarnum);
							if(extdarkchoice!=darkBGsubopt[0]){
								BWdarkname=newArray(polarnum);
								BWdefname=newArray(polarnum);
							}
	
							BWextname[poloop]="Extinction_"+polnames[poloop]+".tif";
							if(extdarkchoice!=darkBGsubopt[0]){
								BWdarkname[poloop]="BWdarkfield_"+polnames[poloop]+".tif";
								BWdefname[poloop]="AvgDefocus_BGsub_"+polnames[poloop]+".tif";
							}
						}
				//////////////
						if(averager==false){
							if(polarization==false){
								showMessage("Select Focus Image");
						 		singlefocuspath = File.openDialog("Select Focus Image");
						 		if(singlefocuspath==""){
						 			exit();
						 		}
						 		showMessage("Select Defocus Image");
								singledefocuspath = File.openDialog("Select Defocus Image");
							 	if(singledefocuspath==""){
								 	exit();
						 		}
						 		showMessage("Select Background Image");
								singlebackpath = File.openDialog("Select Background Image");
							 	if(singlebackpath==""){
								 	exit();
							 	}
							 	if(extdarkchoice!=darkBGsubopt[0]){
									showMessage("Select Darkfield Image");
									singledarkpath = File.openDialog("Select Darkfield Image");
									if(singledarkpath==""){
									 	exit();
								 	}
								 	if(extdarkchoice==darkBGsubopt[2]){
								 		showMessage("Select Darkfield-BG Image");
										singledarkBGpath = File.openDialog("Select Background Image");
							 			if(singledarkBGpath==""){
								 			exit();
							 			}
								 	}
							 	}
							}
							if(polarization==true){
								singlefocuspath=BASEdirec+"Averages"+File.separator+"AVG_focus.tif";
								singledefocuspath=BASEdirec+"Averages"+File.separator+"AVG_defocus.tif";
								singlebackpath=BGdest+"Averages"+File.separator+"AVG_background.tif";
								if(extdarkchoice!=darkBGsubopt[0]){
									singledarkpath=BASEdirec+"Averages"+File.separator+"AVG_dark.tif";
									darkaverage=File.exists(singledarkpath);
									if(darkaverage==true){
										singledarkpath=singledarkpath;
									}
									if(darkaverage==false){
										singledarklist=getFileList(BASEdirec+darkfolder+"converted");
										singledarkpath=BASEdirec+darkfolder+File.separator+"converted"+File.separator+singledarklist[0];
									}
	
									if(extdarkchoice==darkBGsubopt[2]){
										singledarkBGpath=darkBGdest+"Averages"+File.separator+"AVG_darkBG.tif";
										darkBGaverage=File.exists(singledarkBGpath);
										if(darkBGaverage==true){
											singledarkBGpath=singledarkBGpath;
										}
										if(darkBGaverage==false){
											singledarkBGlist=getFileList(BASEdirec+darkBGfolder+"converted");
											singledarkBGpath=BASEdirec+darkBGfolder+File.separator+"converted"+File.separator+singledarkBGlist[0];
										}
									}
								}
							}
						}
						if(averager==true){
							if(polarization==false){
								if(avgfocuschoice==false){
									showMessage("Select Focus Image");
						 			singlefocuspath = File.openDialog("Select Focus Image");
						 			if(singlefocuspath==""){
						 				exit();
						 			}
								}
								if(avgfocuschoice==true){
									singlefocuspath=savedir+preferredtitle+".tif";
								}
								//////////////////////////////////////////////////////////////////
								if(avgdefocuschoice==false){
									showMessage("Select Defocus Image");
									singledefocuspath = File.openDialog("SelectDefocus Image");
									if(singledefocuspath==""){
									 	exit();
								 	}
								}
								if(avgdefocuschoice==true){
									singledefocuspath = savedir1+preferredtitle1+".tif";
								}
								//////////////////////////////////////////////////////////////////
								if(avgbackground==false){
									showMessage("Select Background Image");
									singlebackpath = File.openDialog("Select Background Image");
									if(singlebackpath==""){
								 		exit();
								 	}
								}
								if(avgbackground==true){
									singlebackpath = savedir2+preferredtitle2+".tif";
								}
								//////////////////////////////////////////////////////////////////
								if(avgdarkchoice==false && darkdefault==true){
									showMessage("Select Darkfield Image");
									singledarkpath = File.openDialog("Select Darkfield Image");
									if(singledarkpath==""){
								 		exit();
								 	}
								}
								if(avgdarkchoice==true){
									singledarkpath = savedir3+preferredtitle3+".tif";
								}
								//////////////////////////////////////////////////////////////////
								if(avgdarkBGchoice==false && extdarkchoice==darkBGsubopt[2]){
									showMessage("Select Darkfield-BG Image");
									singledarkBGpath = File.openDialog("Select Darkfield-BG Image");
									if(singledarkBGpath==""){
								 		exit();
								 	}
								}
								if(avgdarkBGchoice==true && extdarkchoice==darkBGsubopt[2]){
									singledarkBGpath = savedir4+preferredtitle4+".tif";
								}
								//////////////////////////////////////////////////////////////////
							}
	
							if(polarization==true){
								singlefocuspath=BASEdirec+"Averages"+File.separator+"AVG_focus.tif";
								singledefocuspath=BASEdirec+"Averages"+File.separator+"AVG_defocus.tif";
								singlebackpath=BGdest+"Averages"+File.separator+"AVG_background.tif";
								if(extdarkchoice!=darkBGsubopt[0]){
									singledarkpath=BASEdirec+"Averages"+File.separator+"AVG_dark.tif";
									darkaverage=File.exists(singledarkpath);
									if(darkaverage==true){
										singledarkpath=singledarkpath;
									}
									if(darkaverage==false){
										singledarklist=getFileList(BASEdirec+darkfolder+"converted");
										singledarkpath=BASEdirec+darkfolder+File.separator+"converted"+File.separator+singledarklist[0];
									}
									if(extdarkchoice==darkBGsubopt[2]){
										singledarkpath=darkBGdest+"Averages"+File.separator+"AVG_darkBG.tif";
										darkBGaverage=File.exists(singledarkpath);
										if(darkBGaverage==true){
											singledarkBGpath=singledarkBGpath;
										}
										if(darkBGaverage==false){
											singledarkBGlist=getFileList(BASEdirec+darkBGfolder+"converted");
											singledarkBGpath=BASEdirec+darkBGfolder+File.separator+"converted"+File.separator+singledarkBGlist[0];
										}
									}
								}
							}
						}
				///////////////
						singlesavepath=BASEdirec+"Extinction"+File.separator;
	
					 	//Open focus image and subtract BG
					 	open(singlefocuspath);
					 	rename("focus");
						open(singlebackpath);
						rename("BG");
						imageCalculator("Subtract create 32-bit","focus","BG");
						selectWindow("focus");
						close();
						//Open defocus image and subtract BG
						open(singledefocuspath);
						rename("defocus");
						imageCalculator("Subtract create 32-bit","defocus","BG");
						selectWindow("defocus");
						close();
						selectWindow("BG");
						close();
						if(extdarkchoice!=darkBGsubopt[0]){
							saveAs("Tiff", BASEdirec+defocusfolder+File.separator+BWdefname[poloop]);
						}
						newdef=getTitle();
						//Open darkfield image without BG correction (if requested), convert to 32-bit, save image
						if(extdarkchoice==darkBGsubopt[1]){
							open(singledarkpath);
							rename("darkfield");
							run("32-bit");
							selectWindow("darkfield");
							saveAs("Tiff", singlesavepath+BWdarkname[poloop]);
							close();
						}
						//Open darkfield image, and subtract darkBG image (if requested)
						if(extdarkchoice==darkBGsubopt[2]){
							open(singledarkpath);
							rename("darkfield");
							open(singledarkBGpath);
							rename("darkBG");
							imageCalculator("Subtract create 32-bit","darkfield","darkBG");
							selectWindow("Result of darkfield");
							saveAs("Tiff", singlesavepath+BWdarkname[poloop]);
							close();
							selectWindow("darkfield");
							close();
							selectWindow("darkBG");
							close();
						}
						//Open darkfield, convert to 32-bit, subtract numeric offset (if requested), save image
						if(extdarkchoice==darkBGsubopt[3]){
							open(singledarkpath);
							rename("darkfield");
							run("32-bit");
							run("Subtract...", "value=DBGoffset");
							selectWindow("darkfield");
							saveAs("Tiff", singlesavepath+BWdarkname[poloop]);
							close();
						}
	
						//Extinction Analysis
						imageCalculator("Divide create 32-bit", "Result of focus", newdef);
						selectWindow("Result of focus");
						close();
						selectWindow(newdef);
						close();
						selectWindow("Result of Result of focus");
						run("Multiply...", "value=-1");
						run("Add...", "value=1");
						rename(selectedtitle);
						selectWindow(selectedtitle);
						saveAs("Tiff", singlesavepath+BWextname[poloop]);
						close();
	
					}
				}
			}
	///////////////////End of Converter/Averager/Extinction Polarization Loop
		}
	}
	
	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////// Particle Analysis portion of Macro
	////////////////////////////////////////////////////////////////////////////////////////////////////////////// NEW FUNCTIONS
	
	//////////////////////////////////////////////////////////////////// Array functions
	function arrayvalue(array,value){
		numoccur=0;
		for (i=0;i<array.length;i++){
			if(array[i]==value){
				numoccur++;
			}
		}
		return numoccur;
	}
	
	function arrayvalueindices(array,value){
		numoccur=0;
		for (i=0;i<array.length;i++){
			if(array[i]==value){
				numoccur++;
			}
		}
		if (numoccur==0){
			valueposition=newArray(1);
			valueposition[0]=0;
			return valueposition;
		}
		if (numoccur>0){
			valueposition=newArray(numoccur);
			numoccur=0;
			for(i=0;i<array.length;i++){
				if(array[i]==value){
					valueposition[numoccur]=i;
					numoccur++;
				}
			}
			return valueposition;
		}
	}
	
	function arraytrimmer(array,value){
		numoccur=0;
		for (i=0;i<array.length;i++){
			if(array[i]==value){
				numoccur++;
			}
		}
		if (numoccur==0){
			return array;
		}
		if (numoccur>0){
			valueantiposition=newArray(array.length-numoccur);
			numoccur1=0;
			for(i=0;i<array.length;i++){
				if(array[i]!=value){
					valueantiposition[numoccur1]=i;
					numoccur1++;
				}
			}
			trimarray=newArray(array.length-numoccur);
			for(i=0;i<array.length-numoccur;i++){
				trimarray[i]=array[valueantiposition[i]];
			}
			return trimarray;
		}
	}
	
	function countskipper(array,value){
		numoccur=0;
		for(i=0;i<array.length;i++){
			if(array[i]==value){
				numoccur++;
			}
		}
		count=newArray(array.length);
		for(i=0;i<array.length;i++){
			count[i]=i;
		}
		if(numoccur==0){
			return count;
		}
		if(numoccur>0){
			valueposition=newArray(numoccur);
			numoccur=0;
			for(i=0;i<array.length;i++){
				if(array[i]==value){
					valueposition[numoccur]=i;
					numoccur++;
				}
			}
			for(i=0;i<numoccur;i++){
				count=arraytrimmer(count,valueposition[i]);
			}
			return count;
		}
	}
	
	function fronttrim (array,number){
		R=Array.reverse(array);
		S=Array.trim(R,R.length-number);
		T=Array.reverse(S);
		return T;
	}
	
	function trimposition(array,position){
		trimarray=newArray(array.length-1);
		frontarray=newArray(position);
		backarray=newArray(array.length-position-1);
		for (i=0;i<position;i++){
			frontarray[i]=array[i];
		}
		for (i=0;i<array.length-position-1;i++){
			backarray[i]=array[i+position+1];
		}
		trimarray=Array.concat(frontarray,backarray);
		return trimarray;
	}
	
	function arraySum (array){
		sum=array[0];
		for(i=0;i<array.length-1;i++){
			sum=array[i+1]+sum;
		}
		return sum;
	}
	
	function arrayrebuild(datarray,refarray,refvalue){
		numoccur=0;
		for (i=0;i<refarray.length;i++){
			if(refarray[i]==refvalue){
				numoccur++;
			}
		}
		if (numoccur==0){
			rebuilt=newArray(1);
			rebuilt[0]=0;
			return rebuilt;
		}
		if (numoccur>0){
			rebuilt=newArray(numoccur);
			numoccur=0;
			for(i=0;i<refarray.length;i++){
				if(refarray[i]==refvalue){
					rebuilt[numoccur]=datarray[i];
					numoccur++;
				}
			}
			return rebuilt;
		}
	}
	//////////////////////////////////////// Numeric functions
	function isOdd(value){
		return value&1;
	}
	function isInt(value){
		if(value%1==0){return true;}
		else{return false;}
	}
	///////////////////////////////////////////////////////// Gaussian Coordinate Fitting
	function gaussCoord(initialx,initialy,rad,imagename){
		radint=isInt(rad);
		if(radint==true){
			num=rad*2+1;
		}
		if(radint==false){
			radint=isInt(2*rad);
			if(radint==true){
				radodd=isOdd(2*rad);
				if(radodd==true){
					num=rad*2;
				}
				else{
					num=rad*2+1;
				}
			}
			if(radint==false){
				radodd=isOdd(round(2*rad));
				if(radodd==true){
					num=round(rad*2);
				}
				if(radodd==false){
					num=round(rad*2)+1;
				}
			}
		}
		centerR=sqrt(pow(initialx,2)+pow(initialy,2));
		x=newArray(num);
		y=newArray(num);
		xI=newArray(num);
		yI=newArray(num);
		
		selectWindow(imagename);
				
		for(i=0;i<num;i++){
			x[i]=initialx-rad+i;
			y[i]=initialy-rad+i;
			xI[i]=getPixel(initialx-rad+i,initialy);
			yI[i]=getPixel(initialx,initialy-rad+i);
		}
				
		Fit.doFit("Gaussian",x,xI);
		xoffset=Fit.p(0);
		xamp=Fit.p(1);
		xcent=Fit.p(2);
		xwidth=Fit.p(3);
		rxfit=Fit.rSquared;
						
		Fit.doFit("Gaussian",y,yI);
		yoffset=Fit.p(0);
		yamp=Fit.p(1);
		ycent=Fit.p(2);
		ywidth=Fit.p(3);
		ryfit=Fit.rSquared;
		
		selectWindow(imagename);
		
		cent=newArray(2);
		centx=newArray(2);
		centy=newArray(2);
		for(j=0;j<2;j++){
			if(j==0){a=1;b=1;}
			if(j==1){a=-1;b=-1;}
			pixcoord=newArray(num);
			pixI=newArray(num);
			for(i=0;i<num;i++){
				x[i]=initialx-rad+i;
				y[i]=initialy-a*rad+b*i;
				pixcoord[i]=sqrt(pow(x[i],2)+pow(y[i],2));
				pixI[i]=getPixel(x[i],y[i]);
			}
					
			Fit.doFit("Gaussian",pixcoord,pixI);
			cent[j]=Fit.p(2);
			centx[j]=initialx-((cent[j]-centerR)/sqrt(2));
			centy[j]=initialy-((cent[j]-centerR)/sqrt(2));
		}
		centerx=(xcent+centx[0]+centx[1])/3;
		centery=(ycent+centy[0]+centy[1])/3;
		xy=newArray(centerx,centery);
		return xy;
	}
	//////////////////////////////////////////////// ROI Coordinate Function
	function ROIcoord(inrad,rad){
		radint=isInt(rad);
		if(radint==true){
			length=rad*2+1;
		}
		if(radint==false){
			radint=isInt(2*rad);
			if(radint==true){
				radodd=isOdd(2*rad);
				if(radodd==true){
					length=rad*2;
				}
				else{
					length=rad*2+1;
				}
			}
			if(radint==false){
				radodd=isOdd(round(2*rad));
				if(radodd==true){
					length=round(rad*2);
				}
				if(radodd==false){
					length=round(rad*2)+1;
				}
			}
		}
		r=newArray(pow(length,2));
		s=newArray(pow(length,2));
		goodpoints=0;
		for(l=0;l<length;l++){
			sum=l*(length);
			for(k=0;k<length;k++){
				r[k+sum]=round(k-rad);
				s[k+sum]=round(l-rad);
				if(inrad==0){
					if(sqrt(pow(r[k+sum],2)+pow(s[k+sum],2))>=inrad && sqrt(pow(r[k+sum],2)+pow(s[k+sum],2))<=rad){
						goodpoints++;
					}
				}
				else{
					if(sqrt(pow(r[k+sum],2)+pow(s[k+sum],2))>inrad && sqrt(pow(r[k+sum],2)+pow(s[k+sum],2))<=rad){
						goodpoints++;
					}
				}
			}
		}
		
		newarrx=newArray(goodpoints);
		newarry=newArray(goodpoints);
		goodpoints=0;
		for(l=0;l<pow(length,2);l++){
			if(inrad==0){
				if(sqrt(pow(r[l],2)+pow(s[l],2))>=inrad && sqrt(pow(r[l],2)+pow(s[l],2))<=rad){
					newarrx[goodpoints]=r[l];
					newarry[goodpoints]=s[l];
					goodpoints++;
				}
			}
			else{
				if(sqrt(pow(r[l],2)+pow(s[l],2))>inrad && sqrt(pow(r[l],2)+pow(s[l],2))<=rad){
					newarrx[goodpoints]=r[l];
					newarry[goodpoints]=s[l];
					goodpoints++;
				}
			}
		}
		r=newarrx;
		s=newarry;
		fullpix=Array.concat(r,s);
		return fullpix;
	}
	//////////////////////////////////////////////// ROI Extinction Measurement Function
	function ROIext(x0,y0,posarray,name){
		selectWindow(name);
		ROIx=Array.slice(posarray,0,posarray.length/2);
		ROIy=Array.slice(posarray,posarray.length/2,posarray.length);
		pixval=newArray(ROIx.length);
		for(l=0;l<ROIx.length;l++){
			pixval[l]=getPixel(x0+ROIx[l],y0+ROIy[l]);
		}
		Array.getStatistics(pixval,min,max,mean,stdDev);
		ROIdata=newArray(mean,ROIx.length);
		return ROIdata;
	}
	//////////////////////////////////////////////// Sigma-loop Extinction Measurement Function
	function sigmaloopext(x0,y0,posarray,name,Swidth){
		selectWindow(name);
		ROIx=Array.slice(posarray,0,posarray.length/2);
		ROIy=Array.slice(posarray,posarray.length/2,posarray.length);
		pixelvalues=newArray(ROIx.length);
		for(l=0;l<ROIx.length;l++){
			pixelvalues[l]=getPixel(x0+ROIx[l],y0+ROIy[l]);
		}
		Array.getStatistics(pixelvalues,min,max,mean,sigma);
		numpix=pixelvalues.length;
		goodcounts=0;
		for(l=0;l<numpix;l++){
			if(pixelvalues[l]>=mean-(Swidth*sigma) && pixelvalues[l]<=mean+(Swidth*sigma)){
				goodcounts++;
			}
		}
		pixpos=newArray(numpix);
		sigma1=1;
		sigma2=2;
		iteration=0;
		while(sigma2!=sigma1){
			pixvalues=newArray(goodcounts);
			goodcounts=0;
			for(l=0;l<numpix;l++){
				if(pixelvalues[l]>=mean-(Swidth*sigma) && pixelvalues[l]<=mean+(Swidth*sigma)){
					pixvalues[goodcounts]=pixelvalues[l];
					pixpos[l]=l;
					goodcounts++;
				}
				else{
					pixpos[l]=-1;
				}
			}
			sigma1=sigma;
			Array.getStatistics(pixvalues,min,max,mean,sigma);
			goodcounts=0;
			for(l=0;l<numpix;l++){
				if(pixelvalues[l]>=mean-(Swidth*sigma) && pixelvalues[l]<=mean+(Swidth*sigma)){
					goodcounts++;
				}
			}
			sigma2=sigma;
			iteration++;
		}
		finalmean=mean;
		quality=100*goodcounts/numpix;
		datarray=newArray(finalmean,quality);
		return datarray;
	}
	//////////////////////////////////////////////// Double Radius with sigma-loop Extinction Measurement Function
	function DRext(X,Y,posarray1,posarray2,IMAGE,areacalib,Nsigma){
		DATA1=ROIext(X,Y,posarray1,IMAGE);
		M1=DATA1[0];
		A1=DATA1[1];
			
		DATA2=sigmaloopext(X,Y,posarray2,IMAGE,Nsigma);
		M2=DATA2[0];
		
		extinction=areacalib*A1*(M1-M2);
		return extinction;
	}
	//////////////////////////////////////////////// Sigma Loop Outer Radius Quality Check Function
	function sigcheck(X,Y,posarray,IMAGE,Nsigma){	
		DATA2=sigmaloopext(X,Y,posarray,IMAGE,Nsigma);
		if(DATA2[1]>=50){
			qfac=true;
		}
		if(DATA2[1]<50){
			qfac=false;
		}
		quality=qfac;
		return quality;
	}
	//////////////////////////////////////////////// Single Radius (calibrated) Extinction Measurement Function
	function sinext(X,Y,RADIUS1,posarray1,IMAGE,areacalib){
		DATA1=ROIext(X,Y,posarray1,IMAGE);
		M1=DATA1[0];
		A1=DATA1[1];
		ext=areacalib*M1*A1;
		return ext;
	}
	///////////////////////////////////////////////////////////////////////Analysis Main Options and Folder Names
	if (partanalysis==true){
		if(extanalysis==true){
			ImageType=newArray("Color","B/W");
			normMethod=newArray("Defocus","Shifted Sample");
			Dialog.create("Analysis Options");
			Dialog.addCheckbox("Run Darkfield Registration and Correlation?",true);
			Dialog.addCheckbox("Include Single Radius (Mean) Method?",false);
			Dialog.addCheckbox("Include Single Radius (Regional) Method?",false);
			Dialog.addCheckbox("Include Double Radius (Local) Method?",true);
			Dialog.addCheckbox("Single NP size?",true);
			Dialog.addChoice("Select Normalization Method",normMethod,normMethod[0]);
			mat=newArray("Silver","Gold","Non-Metallic");
			Dialog.addChoice("Choose Material to Analyze",mat,mat[1]);
			if(polarization==false){
				Dialog.setInsets(30,20,10);
				Dialog.addMessage("One-time Only Calibration Schemes:");
				Dialog.addCheckbox("Calibrate the particle measurement radius (Ri)?",false);
				Dialog.addCheckbox("Calibrate the Eta factor for Dark-Field?",false);
			}
			Dialog.show();
			if(extchoices==imageoption[0] || extchoices==imageoption[1]){
				type=ImageType[0];
			}
			if(extchoices==imageoption[2]){
				type=ImageType[1];
			}
			dark=Dialog.getCheckbox();
			singleradius=Dialog.getCheckbox();;
			regional=Dialog.getCheckbox();;;
			doubleradius=Dialog.getCheckbox();;;;
			NPsize=Dialog.getCheckbox;;;;;
			normchoice=Dialog.getChoice();
			if(type==ImageType[0]){
			matchoice=Dialog.getChoice();;
			}
			else{
				if(type==ImageType[1]){
					matchoice="Non-Metallic";
				}
			}
			if(polarization==false){
				radiuscalibration=Dialog.getCheckbox();;;;;;
				Etacalibration=Dialog.getCheckbox();;;;;;;
			}
		}
	if(polarization==true){
		polarizedfilter=matchoice;
		radiuscalibration=false;
		Etacalibration=false;
		File.makeDirectory(polbase+"Polarization_data");
		File.makeDirectory(polbase+"Polarization_data"+File.separator+"Pol_Processing");
		polproc=polbase+"Polarization_data"+File.separator+"Pol_Processing"+File.separator;
	}
	
	if(normchoice=="Defocus"){
		referenceshift=false;
	}
	if(normchoice=="Shifted Sample"){
		referenceshift=true;
	}
	
	if(type==ImageType[0]){
		if(polarization==false){
			blueextname=newArray("blue_extinction.tif");
			greenextname=newArray("green_extinction.tif");
			redextname=newArray("red_extinction.tif");
	
			if(dark==true){
				bluedarkname=newArray("blue_darkfield.tif");
				greendarkname=newArray("green_darkfield.tif");
				reddarkname=newArray("red_darkfield.tif");
	
				bluedefname=newArray("blue_AvgDefocus_BGsub.tif");
				greendefname=newArray("green_AvgDefocus_BGsub.tif");
				reddefname=newArray("red_AvgDefocus_BGsub.tif");
			}
		}
	
		if(polarization==true){
			blueextname=newArray(polarnum);
			greenextname=newArray(polarnum);
			redextname=newArray(polarnum);
	
			if(dark==true){
				bluedarkname=newArray(polarnum);
				greendarkname=newArray(polarnum);
				reddarkname=newArray(polarnum);
	
				bluedefname=newArray(polarnum);
				greendefname=newArray(polarnum);
				reddefname=newArray(polarnum);
			}
	
			for(i=0;i<polarnum;i++){
				blueextname[i]="blue_extinction_"+polnames[i]+".tif";
				greenextname[i]="green_extinction_"+polnames[i]+".tif";
				redextname[i]="red_extinction_"+polnames[i]+".tif";
				if(dark==true){
					bluedarkname[i]="blue_darkfield_"+polnames[i]+".tif";
					greendarkname[i]="green_darkfield_"+polnames[i]+".tif";
					reddarkname[i]="red_darkfield_"+polnames[i]+".tif";
	
					bluedefname[i]="blue_AvgDefocus_BGsub_"+polnames[i]+".tif";
					greendefname[i]="green_AvgDefocus_BGsub_"+polnames[i]+".tif";
					reddefname[i]="red_AvgDefocus_BGsub_"+polnames[i]+".tif";
				}
			}
		}
		blueparticle=newArray(polarnum);
		greenparticle=newArray(polarnum);
		redparticle=newArray(polarnum);
		if(dark==true){
			darkblue=newArray(polarnum);
			darkgreen=newArray(polarnum);
			darkred=newArray(polarnum);
	
			defblue=newArray(polarnum);
			defgreen=newArray(polarnum);
			defred=newArray(polarnum);
		}
	}
	
	if(type==ImageType[1]){
		if(polarization==false){
			BWextname=newArray("Extinction.tif");
			if(dark==true){
				BWdarkname=newArray("BWdarkfield.tif");
				defBWname=newArray("AvgDefocus_BGsub.tif");
			}
		}
	
		if(polarization==true){
			BWextname=newArray(polarnum);
			if(dark==true){
				BWdarkname=newArray(polarnum);
				defBWname=newArray(polarnum);
			}
	
			for(i=0;i<polarnum;i++){
				BWextname[i]="Extinction_"+polnames[i]+".tif";
				if(dark==true){
					BWdarkname[i]="BWdarkfield_"+polnames[i]+".tif";
					defBWname[i]="AvgDefocus_BGsub_"+polnames[i]+".tif";
				}
			}
		}
		BWparticle=newArray(polarnum);
		if(dark==true){
			darkBW=newArray(polarnum);
			defBW=newArray(polarnum);
		}
	}
	
	for(poloop=0;poloop<polarnum;poloop++){
		if(polarization==true){
			BASEdirec=polbase+polnames[poloop]+File.separator;
		}
	
		if(extanalysis==false){
			partsave=BASEdirec+"Extinction"+File.separator;
		}
	
		File.makeDirectory(BASEdirec+"Processing");
		processing=BASEdirec+"Processing"+File.separator;
	
		if(type==ImageType[0]){
			blueparticle[poloop]=BASEdirec+"Extinction"+File.separator+blueextname[poloop];
			greenparticle[poloop]=BASEdirec+"Extinction"+File.separator+greenextname[poloop];
			redparticle[poloop]=BASEdirec+"Extinction"+File.separator+redextname[poloop];
			if(dark==true){
				darkblue[poloop]=BASEdirec+"Extinction"+File.separator+bluedarkname[poloop];
				darkgreen[poloop]=BASEdirec+"Extinction"+File.separator+greendarkname[poloop];
				darkred[poloop]=BASEdirec+"Extinction"+File.separator+reddarkname[poloop];
	
				defblue[poloop]=BASEdirec+defocusfolder+File.separator+bluedefname[poloop];
				defgreen[poloop]=BASEdirec+defocusfolder+File.separator+greendefname[poloop];
				defred[poloop]=BASEdirec+defocusfolder+File.separator+reddefname[poloop];
			}
		}
	
		if(type==ImageType[1]){
			BWparticle[poloop]=BASEdirec+"Extinction"+File.separator+BWextname[poloop];
			if(dark==true){
				darkBW[poloop]=BASEdirec+"Extinction"+File.separator+BWdarkname[poloop];
				defBW[poloop]=BASEdirec+defocusfolder+File.separator+defBWname[poloop];
			}
		}
	}
	
	if(matchoice=="Silver" && type!=ImageType[1]){
		mainimage=blueparticle[0];
		regfiles=blueextname;
		mainparticle=blueparticle;
	}
	if((matchoice=="Gold" || matchoice=="Non-metallic") && type!=ImageType[1]){
		mainimage=greenparticle[0];
		regfiles=greenextname;
		mainparticle=greenparticle;
	}
	if(type==ImageType[1]){
		mainimage=BWparticle[0];
		regfiles=BWextname;
		mainparticle=BWparticle;
	}
	
	
	if(dark==true){
		if(matchoice=="Silver" && type!=ImageType[1]){
			maindarkimage=darkblue[0];
			regdarkfiles=bluedarkname;
		}
		if((matchoice=="Gold" || matchoice=="Non-metallic") && type!=ImageType[1]){
			maindarkimage=darkgreen[0];
			regdarkfiles=greendarkname;
		}
		if(type==ImageType[1]){
			maindarkimage=darkBW[0];
			regdarkfiles=BWdarkname;
		}
	}
	
	open(mainimage);
	maintitle=getTitle();
	close();
	if(dark==true){
		open(maindarkimage);
		maindarktitle=getTitle();
		close();
	}
	
	setBatchMode(false);
	
	if(radiuscalibration==true){
		open(mainimage);
		maintitle=getTitle();
		run("Set Scale...", "distance=0 known=0 pixel=1 unit=pixel global");
		waitForUser("Scan image for a reasonably intense nanoparticle.\nPlace cursor over center of particle and note X and Y coordinates of the brightest pixel\n(appearing in the ImageJ toolbar).  The particle should be well isolated (at least 60 pixels away\nfrom any other peaks).");
		Dialog.create("Ri Optimization Operation");
		Dialog.addNumber("Enter X Coordinate of particle",0);
		Dialog.addNumber("Enter Y Coordinate of particle",0);
		//Dialog.addNumber("Enter X Coordinate of background",0);
		//Dialog.addNumber("Enter Y Coordinate of background",0);
		Dialog.show();
	
		Xopt=Dialog.getNumber();
		Yopt=Dialog.getNumber();
		//Xback=Dialog.getNumber();
		//Yback=Dialog.getNumber();
		fullcount=48;
		opt=newArray(fullcount);
		opt2=newArray(fullcount);
		extopt=newArray(fullcount);
		optradius=newArray(fullcount);
		selectWindow(maintitle);
		run("Select None");
		run("Clear Results");
		for(i=0;i<fullcount;i++){
			optrad=i/2;
			makeOval(Xopt-optrad,Yopt-optrad,2*optrad,2*optrad);
			List.setMeasurements;
			opt[i]=List.getValue("Mean");
			//area=List.getValue("Area");
			makeOval(Xopt-2*optrad,Yopt-2*optrad,4*optrad,4*optrad);
			List.setMeasurements;
			opt2[i]=List.getValue("Mean");
			//area2=List.getValue("Area");
			extopt[i]=((PI*pow(optrad,2)*opt[i])-((PI*pow(optrad,2))*((pow(2*optrad,2)*opt2[i]-pow(optrad,2)*opt[i])/(pow(2*optrad,2)-pow(optrad,2)))));
			optradius[i]=optrad;
		}
		selectWindow(maintitle);
		close();
		Array.getStatistics(extopt,min,max,mean,stdDev);
		maxext=max;
		Plot.create("Ext Vs. Measurement Radius", "Radius (pixels)", "Extinction (unscaled)");
	    Plot.setLimits(0, fullcount/2, 0,maxext+0.1*maxext);
	    Plot.setColor("red");
	    Plot.add("circles",optradius,extopt);
	    Plot.setColor("black");
	    setJustification("center");
	    Plot.addText("Extinction Cross-Section vs. Measurement Radius",0.5, 0.0);
	    Plot.show();
	}
	///////////////////////////////////////////////////Registration Section
	
	//////////////////Single Brightfield and Darkfield (unpolarized or zeroth angle of polarized)
	if(dark==true){
	repeat=true;
	while(repeat==true){
	
	open(mainimage);
	
	run("Set Scale...", "distance=0 known=0 pixel=1 unit=pixel global");
	waitForUser("Use 'Find Maxima' to PREVIEW selection and \ncalibrate 'Extinction Image' noise tolerance. Do NOT run.\n \nFor 'Image Registration' purposes choose a noise tolerance\ngiving a low number of points.\n \nPRESS 'Ok' ONLY AFTER PREVIEW.");
	
	Dialog.create("Registration Options");
	Dialog.addNumber("Noise Tolerance",0.01);
	Dialog.addNumber("Pattern Recognition Tolerance",3);
	Dialog.show();
	
	noisetol=Dialog.getNumber();
	shiftol=Dialog.getNumber();;
	
	selectWindow(maintitle);
	makeOval(178,178,156,156);
	waitForUser("Reformat Selection Oval");
	getSelectionBounds(t,r,width,height);
	rectlocx=t;
	rectlocy=r;
	ovalwi=width;
	ovalhe=height;
	
	run("Find Maxima...", "noise=noisetol output=List exclude");
	run("Find Maxima...", "noise=noisetol output=[Point Selection]");
	numresults=nResults;
	
	xreg=newArray(numresults);
	yreg=newArray(numresults);
	
	for (i=0;i<numresults;i++){
		xreg[i]=getResult("X",i);
		yreg[i]=getResult("Y",i);
	}
	
	open(maindarkimage);
	
	run("Set Scale...", "distance=0 known=0 pixel=1 unit=pixel global");
	
	run("Clear Results");
	
	selectWindow(maindarktitle);
	
	makeOval(rectlocx,rectlocy,ovalwi,ovalhe);
	
	wait(2000);
	waitForUser("Use 'Find Maxima' to PREVIEW selection and \ncalibrate 'Darkfield Image' noise tolerance. Do NOT run.\n \nAttempt choose a noise tolerance giving a similar number\nof points as in the 'Extinction' calibration performed.\n \nAgain, ONLY PRESS 'OK' AFTER FINISHING PREVIEW");
	
	Dialog.create("Darkfield Maxima");
	Dialog.addNumber("Specify Noise Tolerance for Dark-field Image",500);
	Dialog.show();
	
	darknoise=Dialog.getNumber();
	
	run("Find Maxima...", "noise=darknoise output=List exclude");
	run("Find Maxima...", "noise=darknoise output=[Point Selection]");
	numresults1=nResults;
	run("Select None");
	
	x1=newArray(numresults1);
	y1=newArray(numresults1);
	
	for (i=0;i<numresults1;i++){
	x1[i]=getResult("X",i);
	y1[i]=getResult("Y",i);
	}
	
	if(numresults>=numresults1){
		check=numresults1;
	}
	else{
		if(numresults1>numresults){
			check=numresults;
		}
	}
	
	
	ref=newArray(check);
	for(i=0;i<check;i++){
		if((x1[i]-shiftol<xreg[0] && xreg[0]<x1[i]+shiftol)&&(y1[i]-shiftol<yreg[0] && yreg[0]<y1[i]+shiftol)){
			ref[i]=1;
		}
		else{
			ref[i]=0;
		}
	}
	
	refcheck=arrayvalueindices(ref,1);
	x1trim=trimposition(x1,refcheck[0]);
	y1trim=trimposition(y1,refcheck[0]);
	
	c=newArray(numresults1-1);
	e=newArray(numresults-1);
	xdif=newArray(numresults-1);
	ydif=newArray(numresults-1);
	x1dif=newArray(numresults1);
	y1dif=newArray(numresults1);
	for(i=0;i<numresults-1;i++){
		xdif[i]=abs(xreg[0]-xreg[i+1]);
		ydif[i]=abs(yreg[0]-yreg[i+1]);
	}
	for(i=0;i<numresults1;i++){
		x1dif[i]=abs(x1[refcheck[0]]-x1[i]);
		y1dif[i]=abs(y1[refcheck[0]]-y1[i]);
	}
	
	x1dif=trimposition(x1dif,refcheck[0]);
	y1dif=trimposition(y1dif,refcheck[0]);
	
	for(i=0;i<numresults-1;i++){
		for(j=0;j<numresults1-1;j++){
			if(((x1dif[j]-shiftol)<xdif[i] && xdif[i]<(x1dif[j]+shiftol)) && ((y1dif[j]-shiftol)<ydif[i] && ydif[i]<(y1dif[j]+shiftol))){
				c[j]=1;
			}
			else {
				c[j]=0;
			}
		}
		ccheck=arrayvalue(c,1);
		d=arrayvalueindices(c,1);
		if(ccheck==0){
			d=newArray(1);
			d[0]=0.1;
		}
		e[i]=d[0];
	}
	etrim=arraytrimmer(e,0.1);
	xshift=newArray(etrim.length);
	yshift=newArray(etrim.length);
	indreg=countskipper(e,0.1);
	xnewbase=fronttrim(xreg,1);
	ynewbase=fronttrim(yreg,1);
	newx=newArray(etrim.length);
	newy=newArray(etrim.length);
	newx1=newArray(etrim.length);
	newy1=newArray(etrim.length);
	for(i=0;i<etrim.length;i++){
		newx[i]=xnewbase[indreg[i]];
		newy[i]=ynewbase[indreg[i]];
	}
	for(i=0;i<etrim.length;i++){
			xshift[i]=newx[i]-x1trim[etrim[i]];
			yshift[i]=newy[i]-y1trim[etrim[i]];
	}
	
	Array.getStatistics(xshift,min,max,mean,stdDev);
	darkx=mean;
	Array.getStatistics(yshift,min,max,mean,stdDev);
	darky=mean;
	
	
	selectWindow(maintitle);
	close();
	selectWindow(maindarktitle);
	close();
	
	run("Clear Results");
	
	print("Bright-Dark x shift ="+darkx);
	print("Bright-Dark y shift ="+darky);
	
	waitForUser("Check Log of Registration Shifts for Abnormalities");
	
	Dialog.create("Registration Check");
	Dialog.addCheckbox("Repeat Registration",false);
	Dialog.show();
	
	repeat=Dialog.getCheckbox();
		}
	}
	//////////////////////////////////////// Polarization Registration
	if(polarization==true){
	
	repeat=true;
	defaultnoisetol=0.01;
	defaultshiftol=3;
	if(dark==true){
		defaultnoisetol=noisetol;
		defaultshifttol=shiftol;
	}
	while(repeat==true){
		setBatchMode(false);
		open(mainimage);
		run("Set Scale...", "distance=0 known=0 pixel=1 unit=pixel global");
		waitForUser("Use 'Find Maxima' to PREVIEW selection and \ncalibrate 'Extinction Image' noise tolerance. Do NOT run.\n \nFor 'Image Registration' purposes choose a noise tolerance\ngiving a low number of points.\n \nAgain, ONLY PRESS 'OK' AFTER FINISHING PREVIEW");
	
		Dialog.create("Registration Options");
		Dialog.addNumber("Noise Tolerance",defaultnoisetol);
		Dialog.addNumber("Pattern Recognition Tolerance",defaultshiftol);
		Dialog.show();
	
	noisetol=Dialog.getNumber();
	shiftol=Dialog.getNumber();;
	
	shiftExtx=newArray(polarnum);
	shiftExty=newArray(polarnum);
	
	for(a=1;a<polarnum;a++){
	
	selectWindow(maintitle);
	if(a==1){
		if(dark==true){
			makeOval(rectlocx,rectlocy,ovalwi,ovalhe);
		}
		if(dark==false){
			makeOval(178,178,156,156);
		}
	waitForUser("Reformat Selection Oval");
	getSelectionBounds(t,r,width,height);
	rectlocx=t;
	rectlocy=r;
	ovalwi=width;
	ovalhe=height;
	selectWindow(maintitle);
	close();
	setBatchMode(true);
	open(mainimage);
	makeOval(rectlocx,rectlocy,ovalwi,ovalhe);
	}
	if(a>1){
		selectWindow(maintitle);
		makeOval(rectlocx,rectlocy,ovalwi,ovalhe);
	}
	run("Find Maxima...", "noise=noisetol output=List exclude");
	run("Find Maxima...", "noise=noisetol output=[Point Selection]");
	numresults=nResults;
	
	x=newArray(numresults);
	y=newArray(numresults);
	
	for (i=0;i<numresults;i++){
		x[i]=getResult("X",i);
		y[i]=getResult("Y",i);
	}
	
	BASEdirec=polbase+polnames[a]+File.separator;
	open(BASEdirec+"Extinction"+File.separator+regfiles[a]);
	run("Set Scale...", "distance=0 known=0 pixel=1 unit=pixel global");
	
	run("Clear Results");
	selectWindow(regfiles[a]);
	makeOval(rectlocx,rectlocy,ovalwi,ovalhe);
	
	run("Find Maxima...", "noise=noisetol output=List exclude");
	run("Find Maxima...", "noise=noisetol output=[Point Selection]");
	
	wait(2000);
	numresults1=nResults;
	run("Select None");
	
	x1=newArray(numresults1);
	y1=newArray(numresults1);
	
	for (i=0;i<numresults1;i++){
	x1[i]=getResult("X",i);
	y1[i]=getResult("Y",i);
	}
	
	if(numresults>=numresults1){
		check=numresults1;
	}
	else{
		if(numresults1>numresults){
			check=numresults;
		}
	}
	
	
	ref=newArray(check);
	for(i=0;i<check;i++){
		if((x1[i]-shiftol<x[0] && x[0]<x1[i]+shiftol)&&(y1[i]-shiftol<y[0] && y[0]<y1[i]+shiftol)){
			ref[i]=1;
		}
		else{
			ref[i]=0;
		}
	}
	
	refcheck=arrayvalueindices(ref,1);
	x1trim=trimposition(x1,refcheck[0]);
	y1trim=trimposition(y1,refcheck[0]);
	
	c=newArray(numresults1-1);
	e=newArray(numresults-1);
	xdif=newArray(numresults-1);
	ydif=newArray(numresults-1);
	x1dif=newArray(numresults1);
	y1dif=newArray(numresults1);
	for(i=0;i<numresults-1;i++){
		xdif[i]=abs(x[0]-x[i+1]);
		ydif[i]=abs(y[0]-y[i+1]);
	}
	for(i=0;i<numresults1;i++){
		x1dif[i]=abs(x1[refcheck[0]]-x1[i]);
		y1dif[i]=abs(y1[refcheck[0]]-y1[i]);
	}
	
	x1dif=trimposition(x1dif,refcheck[0]);
	y1dif=trimposition(y1dif,refcheck[0]);
	
	for(i=0;i<numresults-1;i++){
		for(j=0;j<numresults1-1;j++){
		if(((x1dif[j]-shiftol)<xdif[i] && xdif[i]<(x1dif[j]+shiftol)) && ((y1dif[j]-shiftol)<ydif[i] && ydif[i]<(y1dif[j]+shiftol))){
			c[j]=1;
				}
			else {
				c[j]=0;
				}
		}
		ccheck=arrayvalue(c,1);
		d=arrayvalueindices(c,1);
		if(ccheck==0){
			d=newArray(1);
			d[0]=0.1;
		}
		e[i]=d[0];
	}
	etrim=arraytrimmer(e,0.1);
	xshift=newArray(etrim.length);
	yshift=newArray(etrim.length);
	indreg=countskipper(e,0.1);
	xnewbase=fronttrim(x,1);
	ynewbase=fronttrim(y,1);
	newx=newArray(etrim.length);
	newy=newArray(etrim.length);
	newx1=newArray(etrim.length);
	newy1=newArray(etrim.length);
	for(i=0;i<etrim.length;i++){
		newx[i]=xnewbase[indreg[i]];
		newy[i]=ynewbase[indreg[i]];
	}
	for(i=0;i<etrim.length;i++){
			xshift[i]=newx[i]-x1trim[etrim[i]];
			yshift[i]=newy[i]-y1trim[etrim[i]];
	}
	
	Array.getStatistics(xshift,min,max,mean,stdDev);
	shiftExtx[a]=mean;
	Array.getStatistics(yshift,min,max,mean,stdDev);
	shiftExty[a]=mean;
	
	selectWindow(regfiles[a]);
	close();
	
	run("Clear Results");
	}
	print("Extinction Polarization Shifts");
	Array.print(shiftExtx);
	Array.print(shiftExty);
	
	waitForUser("Check Log of Registration Shifts for Abnormalities");
	
	Dialog.create("Registration Check");
	Dialog.addCheckbox("Repeat Registration",false);
	Dialog.show();
	
	repeat=Dialog.getCheckbox();
	selectWindow(maintitle);
	close();
		}
	
	/////////////////////////////Darkfield (Polarized) Registration
	if(dark==true){
	repeat=true;
	
	defaultnoisetol=darknoise;
	defaultshiftol=shiftol;
	
	while(repeat==true){
		setBatchMode(false);
		open(maindarkimage);
		waitForUser("Use 'Find Maxima' to PREVIEW selection and \ncalibrate 'Darkfield Image' noise tolerance. Do NOT run.\n \nFor 'Image Registration' purposes choose a noise tolerance\ngiving a low number of points.\n \nAgain, ONLY PRESS 'OK' AFTER FINISHING PREVIEW");
	
		run("Set Scale...", "distance=0 known=0 pixel=1 unit=pixel global");
		Dialog.create("Registration Options");
		Dialog.addNumber("Noise Tolerance",defaultnoisetol);
		Dialog.addNumber("Pattern Recognition Tolerance",defaultshiftol);
		Dialog.show();
	
	noisetol=Dialog.getNumber();
	shiftol=Dialog.getNumber();;
	
	shiftDarkx=newArray(polarnum);
	shiftDarky=newArray(polarnum);
	
	for(a=1;a<polarnum;a++){
	
	if(a==1){
		selectWindow(maindarktitle);
		makeOval(rectlocx,rectlocy,ovalwi,ovalhe);
		waitForUser("Reformat Selection Oval");
		getSelectionBounds(t,r,width,height);
		rectlocx=t;
		rectlocy=r;
		ovalwi=width;
		ovalhe=height;
		selectWindow(maindarktitle);
		close();
		setBatchMode(true);
		open(maindarkimage);
		makeOval(rectlocx,rectlocy,ovalwi,ovalhe);
	}
	if(a>1){
		selectWindow(maindarktitle);
	}
	makeOval(rectlocx,rectlocy,ovalwi,ovalhe);
	run("Find Maxima...", "noise=noisetol output=List exclude");
	run("Find Maxima...", "noise=noisetol output=[Point Selection]");
	numresults=nResults;
	
	x=newArray(numresults);
	y=newArray(numresults);
	
	for (i=0;i<numresults;i++){
	x[i]=getResult("X",i);
	y[i]=getResult("Y",i);
	}
	
	BASEdirec=polbase+polnames[a]+File.separator;
	open(BASEdirec+"Extinction"+File.separator+regdarkfiles[a]);
	run("Set Scale...", "distance=0 known=0 pixel=1 unit=pixel global");
	
	run("Clear Results");
	selectWindow(regdarkfiles[a]);
	makeOval(rectlocx,rectlocy,ovalwi,ovalhe);
	
	run("Find Maxima...", "noise=noisetol output=List exclude");
	run("Find Maxima...", "noise=noisetol output=[Point Selection]");
	numresults1=nResults;
	run("Select None");
	
	x1=newArray(numresults1);
	y1=newArray(numresults1);
	
	for (i=0;i<numresults1;i++){
	x1[i]=getResult("X",i);
	y1[i]=getResult("Y",i);
	}
	
	if(numresults>=numresults1){
		check=numresults1;
	}
	else{
		if(numresults1>numresults){
			check=numresults;
		}
	}
	
	
	ref=newArray(check);
	for(i=0;i<check;i++){
		if((x1[i]-shiftol<x[0] && x[0]<x1[i]+shiftol)&&(y1[i]-shiftol<y[0] && y[0]<y1[i]+shiftol)){
			ref[i]=1;
		}
		else{
			ref[i]=0;
		}
	}
	
	refcheck=arrayvalueindices(ref,1);
	x1trim=trimposition(x1,refcheck[0]);
	y1trim=trimposition(y1,refcheck[0]);
	
	c=newArray(numresults1-1);
	e=newArray(numresults-1);
	xdif=newArray(numresults-1);
	ydif=newArray(numresults-1);
	x1dif=newArray(numresults1);
	y1dif=newArray(numresults1);
	for(i=0;i<numresults-1;i++){
		xdif[i]=abs(x[0]-x[i+1]);
		ydif[i]=abs(y[0]-y[i+1]);
	}
	for(i=0;i<numresults1;i++){
		x1dif[i]=abs(x1[refcheck[0]]-x1[i]);
		y1dif[i]=abs(y1[refcheck[0]]-y1[i]);
	}
	
	x1dif=trimposition(x1dif,refcheck[0]);
	y1dif=trimposition(y1dif,refcheck[0]);
	
	for(i=0;i<numresults-1;i++){
		for(j=0;j<numresults1-1;j++){
		if(((x1dif[j]-shiftol)<xdif[i] && xdif[i]<(x1dif[j]+shiftol)) && ((y1dif[j]-shiftol)<ydif[i] && ydif[i]<(y1dif[j]+shiftol))){
			c[j]=1;
				}
			else {
				c[j]=0;
				}
		}
		ccheck=arrayvalue(c,1);
		d=arrayvalueindices(c,1);
		if(ccheck==0){
			d=newArray(1);
			d[0]=0.1;
		}
		e[i]=d[0];
	}
	etrim=arraytrimmer(e,0.1);
	xshift=newArray(etrim.length);
	yshift=newArray(etrim.length);
	indreg=countskipper(e,0.1);
	xnewbase=fronttrim(x,1);
	ynewbase=fronttrim(y,1);
	newx=newArray(etrim.length);
	newy=newArray(etrim.length);
	newx1=newArray(etrim.length);
	newy1=newArray(etrim.length);
	for(i=0;i<etrim.length;i++){
		newx[i]=xnewbase[indreg[i]];
		newy[i]=ynewbase[indreg[i]];
	}
	for(i=0;i<etrim.length;i++){
			xshift[i]=newx[i]-x1trim[etrim[i]];
			yshift[i]=newy[i]-y1trim[etrim[i]];
	}
	
	Array.getStatistics(xshift,min,max,mean,stdDev);
	shiftDarkx[a]=mean;
	Array.getStatistics(yshift,min,max,mean,stdDev);
	shiftDarky[a]=mean;
	
	selectWindow(regdarkfiles[a]);
	close();
	
	run("Clear Results");
	}
	print("Darkfield Polarization Shifts");
	Array.print(shiftDarkx);
	Array.print(shiftDarky);
	
	waitForUser("Check Log of Registration Shifts for Abnormalities");
	
	Dialog.create("Registration Check");
	Dialog.addCheckbox("Repeat Registration",false);
	Dialog.show();
	
	repeat=Dialog.getCheckbox();
	selectWindow(maindarktitle);
	close();
		}
	
	}
	/////////////////////////////////////End Registration
	}
	/////////////////////////////////////////////////Begin Calibration
	if(polarization==true){
		if(type==ImageType[0]){
			matchoice="Non-Metallic";
		}
	}
	microm=getInfo("micrometer.abbreviation");
	setBatchMode(false);
	open(mainimage);
	run("Set Scale...", "distance=0 known=0 pixel=1 unit=pixel");
	
	Dialog.create("Radius Input");
	Dialog.addNumber("Radius of Region of Interest",12);
	Dialog.addNumber("Radius of 2nd ROI (if applicable)",24);
	Dialog.show();
	
	radius=Dialog.getNumber();
	radius2=Dialog.getNumber();;
	diameter=2*radius;
	diameter2=2*radius2;
	
	BGradius=radius;
	
	ROI1=ROIcoord(0,radius);
	ROI2=ROIcoord(radius,radius2);
	
	print("radius of ROI 1="+radius);
	print("radius2 of ROI 2 (if applicable)="+radius2);
	
	waitForUser("Use 'Find Maxima' ONLY TO PREVIEW selection and calibrate\nnoise tolerance, but do NOT run.\n \nRecord appropriate noise tolerance number such that all\nparticles are marked in the 'Preview' including any debris\nor small scatterers, then hit CANCEL.\n \nDo NOT PRESS 'OK' on this window until finishing those steps.");
	
	leftButton=16;
	newImage("Test","8bit",1,1,1);
	waitForUser("Also use this time to measure (manually) the\nmeans of the brightest and dimmest nanoparticles to calibrate\nthe upper and lower particle sizes.\nPress 'OK' to close this dialog. Then hold 'Shift' and left\nclick a particle at the center to measure the mean.\nRelease shift to move about image without measuring.\nYou may zoom in for more accurate centering.\nPlease be careful to Exchane the zoom tool for the hand tool\nin order to measure. Close the blank Image called 'Test' to\nend 'Point Selection Measurement'");
	 xl2=-1; yl2=-1; zl2=-1; flags2=-1;
	while(isOpen("Test")){
		while(isKeyDown("shift")){
			getCursorLoc(xLoc,yLoc,zLoc,flags);
			if (xLoc!=xl2 || yLoc!=yl2 || zLoc!=zl2 || flags!=flags2) {
				if (flags&leftButton!=0){
					xl2=xLoc; yl2=yLoc; zl2=zLoc; flags2=flags;
					makeOval(xl2-radius,yl2-radius,diameter,diameter);
					run("Measure");
					wait(100);
				}
			}
		}
	}
	
	
	Dialog.create("Automation Data");
	if(singleradius|doubleradius==true){
		Dialog.addNumber("Desired Number of BG Data Points in Main Region",2000);
	}
	if(regional==true){
		Dialog.addNumber("Desired Number of 'Regional' BG Data Points",15);
	}
	Dialog.addNumber("Scale value in pixels/"+microm,24.34);
	Dialog.addNumber("Specify Noise tolerance",0.006);
	if(regional==true){
		Dialog.addNumber("BG Regional-Reduction Radius?",(3*radius)+radius/3);
	}
	if(NPsize==true){
		Dialog.addNumber("Specify Upper Limit of Mean Intensity",0.026);
		Dialog.addNumber("Specify Lower Limit of Mean Intensity",0.008);
		Dialog.addNumber("Enter Particle Diameter (nm)",60);
	}
	if(NPsize==false){
		Dialog.addNumber("Specify Upper Limit of Mean Intensity",0.025);
		Dialog.addNumber("Specify Upper Boundary Intensity",0.2);
		Dialog.addNumber("Specify Lower Boundary Intensity",0.015);
		Dialog.addNumber("Specify Lower Limit of Mean Intensity",0.01);
	}
	if(NPsize==false){
		Dialog.addNumber("Enter Size 1 Particle Diameter (nm)",40);
		Dialog.addNumber("Enter Size 2 Particle Diameter (nm)",100);
	}
	if(dark==true){
		Dialog.addNumber("Darkfield Exposure Time (s)",1);
		Dialog.addNumber("Brightfield Exposure Time (s)",1/20);
		if(type==ImageType[0]){
			Dialog.addNumber("Eta (B)",1);
			Dialog.addNumber("Eta (G)",1);
			Dialog.addNumber("Eta (R)",1);
		}
		if(type==ImageType[1]){
			Dialog.addNumber("Eta factor",1);
		}
	}
	if(polarization==true){
		Dialog.addNumber("Specify Number of Polarization Fit Noise Simulations",100);
	}
	if(referenceshift==true){
		Dialog.addNumber("For Normalization by Shifted Reference enter x shift",-15);
		Dialog.addNumber("For Normalization by Shifted Reference enter y shift",2);
	}
	Dialog.setInsets(15,0,10);
	Dialog.addCheckbox("Perform Gaussian fit to particle intensity\nprofile for improved coordinate centering?",true);
	if(doubleradius==true && referenceshift==false){
		Dialog.setInsets(10,0,10);
		Dialog.addMessage("Below, input N for sigma exclusion. For the\ndouble radius method, pixels in the outer radius\nwill be excluded from measurement if they\nare outside of the (Mean +/- N*sigma).");
		Dialog.setInsets(10,0,0);
		Dialog.addNumber("N ",2);
	}  
	Dialog.show();
	
	
	if(singleradius|doubleradius==true){
		datapoints=Dialog.getNumber();
		print("Number of BG Datapoints (single region)="+datapoints);
	}
	if(regional==true){
		datapoints1=Dialog.getNumber();;
		print("Number of BG Datapoints (multi region)="+datapoints1);
	}
	scale=Dialog.getNumber();;;
	noisetol=Dialog.getNumber();;;;
	print("Scale="+scale+" pixels/"+microm);
	print("Noise Tolerance="+noisetol);
	if(regional==true){
		regRad=Dialog.getNumber();;;;;
		print("Radius of BG Region (regional measurement)="+regRad);
	}
	if(NPsize==true){
		uplim=Dialog.getNumber();;;;;;
		lowlim=Dialog.getNumber();;;;;;;
		size1=Dialog.getNumber();;;;;;;;
		print("Upper Mean Intensity Limit="+uplim);
		print("Lower Mean Intensity Limit="+lowlim);
		print("Particle Size="+size1);
	}
	if(NPsize==false){
		uplim=Dialog.getNumber();;;;;;
		upboundlim=Dialog.getNumber();;;;;;;
		lowboundlim=Dialog.getNumber();;;;;;;;
		lowlim=Dialog.getNumber();;;;;;;;;
		size1=Dialog.getNumber();;;;;;;;;;
		size2=Dialog.getNumber();;;;;;;;;;;
		print("Maximum Mean Intensity Limit (Larger Particles)="+uplim);
		print("Lower Bound for Larger particle Mean Intensity="+upboundlim);
		print("Upper Bound for Smaller particle Mean Intensity="+lowboundlim);
		print("Minimum Mean Intensity Limit (Smaller Particles)="+lowlim);
		print("Smaller Particle Size="+size1);
		print("Larger Particle Size="+size2);
	}
	if(dark==true){
		darkexp=Dialog.getNumber();;;;;;;;;;;;
		brightexp=Dialog.getNumber();;;;;;;;;;;;;
		print("Darkfield Exposure Time="+darkexp);
		print("Brightfield Exposure Time="+brightexp);
		if(type==ImageType[0]){
			EtaB=Dialog.getNumber();;;;;;;;;;;;;;
			EtaG=Dialog.getNumber();;;;;;;;;;;;;;;
			EtaR=Dialog.getNumber();;;;;;;;;;;;;;;;
			Eta=newArray(EtaB,EtaG,EtaR);
			print("Blue Channel Eta="+EtaB);
			print("Green Channel Eta="+EtaG);
			print("Red Channel Eta="+EtaR);
		}
		if(type==ImageType[1]){
			EtaBW=Dialog.getNumber();;;;;;;;;;;;;;
			Eta=newArray(EtaBW,0);
			print("Eta="+EtaBW);
		}
		exprel=darkexp/brightexp;
	}
	if(polarization==true){
		noisefits=Dialog.getNumber();;;;;;;;;;;;;;;
		print("Number of fits for polarization noise measurements="+noisefits);
	}
	if(referenceshift==true){
		refshiftx=Dialog.getNumber();;;;;;;;;;;;;;;;
		refshifty=Dialog.getNumber();;;;;;;;;;;;;;;;;
		magrefshift=sqrt(pow(refshiftx,2)+pow(refshifty,2));
		print("X coordinate reference shift="+refshiftx);
		print("Y coordinate reference shift="+refshifty);
		print("Magnitude of reference shift="+magrefshift);
	}
	gaussfit=Dialog.getCheckbox();
	if(doubleradius==true && referenceshift==false){
		Nsig=Dialog.getNumber();;;;;;;;;;;;;;;;;;
	}
	
	scalecal=1000/scale;
	areacal=pow(scalecal,2);
	
	/////////////////////////////////////////////////////////////////// Particle/BG Position Filtering Section
	selectWindow(maintitle);
	
	run("Select None");
	run("Clear Results");
	run("Set Scale...", "distance=scale known=1000 pixel=1 unit=nm");
	imageH=getHeight();
	imageW=getWidth();
	if(regional==true){
		makeOval((regRad+radius)-1, (regRad+radius)-1, imageW-2*(regRad+radius), imageH-2*(regRad+radius));
	}
	else{
		makeOval(0.125*imageW,0.125*imageH,0.75*imageW,0.75*imageH);
	}
	repeat=true;
	while(repeat==true){
		waitForUser("Reformat Selection Oval");
		getSelectionBounds(t,r,width,height);
		rectlocx=t;
		rectlocy=r;
		ovalwi=width;
		ovalhe=height;
	
		if(regional==true){
			if(rectlocx<(regRad+radius)-1 || rectlocy<(regRad+radius)-1 || rectlocx+ovalwi>imageW-(regRad+radius) || rectlocy+ovalhe>imageH-(regRad+radius)){
				repeat=true;
				showMessage("Based on the size of the 'regional radius' you have chosen,\n your desired selection oval may contain points, whose\n'regional radius' extends beyond the image limits.\nPlease reformat the selection oval again and reduce\nthe size or position it further from the edge.");
			}
			else{
				repeat=false;
			}
		}
		if(regional==false){
			repeat=false;
		}
	}
	
	print("X Coordinate of Selection Oval="+rectlocx);
	print("Y Coordinate of Selection Oval="+rectlocy);
	print("Width of Selection Oval="+ovalwi);
	print("Height of Selection Oval="+ovalhe);
	run("Find Maxima...", "noise=noisetol output=List exclude");
	numresults=nResults;
	
	selectWindow(maintitle);
	run("Select None");
	
	xprime=newArray(numresults);
	yprime=newArray(numresults);
	if(referenceshift==true){
		xprimeref=newArray(numresults);
		yprimeref=newArray(numresults);
	}
	
	for(i=0;i<numresults;i++){
		xprime[i]=getResult("X",i);
		yprime[i]=getResult("Y",i);
		if(referenceshift==true){
			xprimeref[i]=xprime[i]+refshiftx;
			yprimeref[i]=yprime[i]+refshifty;
		}
	}
	if(referenceshift==true){
		xprime=Array.concat(xprime,xprimeref);
		yprime=Array.concat(yprime,yprimeref);
		dx=newArray(2*numresults);
	}
	
	if(referenceshift==false){
		dx=newArray(numresults);
	}
	
	dSR=newArray(numresults);
	dDR=newArray(numresults);
	if(singleradius==true || regional==true){
		singlerad=true;
	}
	if(singleradius==false && regional==false){
		singlerad=false;
	}
	
	radsets=newArray(singlerad,doubleradius);
	radnames=newArray("singleradius/Regional","doubleradius");
	
	for(radloop=0;radloop<2;radloop++){
		if(radsets[radloop]==true){
			if(radnames[radloop]=="singleradius/Regional"){
				spacing=2*radius;
				ex=dSR;
			}
			if(radnames[radloop]=="doubleradius"){
				spacing=2*radius;
				if(referenceshift==true){
					spacing=2*radius;
				}
				ex=dDR;
			}
			if(referenceshift==false){
				cx=newArray(numresults);
				totresults=numresults;
			}
			if(referenceshift==true){
				cx=newArray(numresults*2);
				totresults=numresults*2;
			}
	
			for(i=0;i<totresults;i++){
				for(j=0;j<totresults;j++){
					if(xprime[i]==xprime[j] && yprime[i]==yprime[j]){
								cx[j]=1;
					}
					else{
						proximity=sqrt(pow(xprime[j]-xprime[i],2)+pow(yprime[j]-yprime[i],2));
						if(proximity<spacing){
							cx[j]=0;
						}
						else{
							cx[j]=1;
						}
					}
				}
				Array.getStatistics(cx,min,max,mean,stdDev);
				dx[i]=mean;
			}
			if(referenceshift==true){
				for(i=0;i<numresults;i++){
					if(dx[i]+dx[i+numresults]==2){
						ex[i]=1;
					}
					else{
						ex[i]=0;
					}
				}
			}
			if(referenceshift==false){
				for(i=0;i<numresults;i++){
					ex[i]=dx[i];
				}
			}
		}
	}
	
	if(referenceshift==true){
		xprime=Array.slice(xprime,0,numresults);
		yprime=Array.slice(yprime,0,numresults);
	}
	
	if(singleradius==true || regional==true){
		proxfiltSR=arrayvalueindices(dSR,1);
		xSR=newArray(proxfiltSR.length);
		ySR=newArray(proxfiltSR.length);
		intcheckSR=newArray(proxfiltSR.length);
		meanfiltSR=newArray(proxfiltSR.length);
	}
	if(doubleradius==true){
		proxfiltDR=arrayvalueindices(dDR,1);
		xDR=newArray(proxfiltDR.length);
		yDR=newArray(proxfiltDR.length);
		intcheckDR=newArray(proxfiltDR.length);
		meanfiltDR=newArray(proxfiltDR.length);
	}
	
	selectWindow(maintitle);
	
	run("Select None");
	run("Clear Results");
	run("Set Scale...", "distance=scale known=1000 pixel=1 unit=nm");
	for(radloop=0;radloop<2;radloop++){
		if(radsets[radloop]==true){
			if(radnames[radloop]=="singleradius/Regional"){
				xRAD=xSR;
				yRAD=ySR;
				Proxfilter=proxfiltSR;
				Intcheck=intcheckSR;
				Meanfilt=meanfiltSR;
			}
			if(radnames[radloop]=="doubleradius"){
				xRAD=xDR;
				yRAD=yDR;
				Proxfilter=proxfiltDR;
				Intcheck=intcheckDR;
				Meanfilt=meanfiltDR;
				sigmaquality=newArray(Proxfilter.length);
			}
			for(i=0;i<Proxfilter.length;i++){
				xRAD[i]=xprime[Proxfilter[i]];
				yRAD[i]=yprime[Proxfilter[i]];
				makeOval(xRAD[i]-radius,yRAD[i]-radius,diameter,diameter);
				List.setMeasurements;
				Meanfilt[i]=List.getValue("Mean");
				area=List.getValue("Area");
				if(doubleradius==true && referenceshift==false){
					sigmaquality[i]=sigcheck(xRAD[i],yRAD[i],ROI2,maintitle,Nsig);
				}
			}
			if(doubleradius==true && referenceshift==false){
				xRAD=arrayrebuild(xRAD,sigmaquality,true);
				yRAD=arrayrebuild(yRAD,sigmaquality,true);
				Meanfilt=arrayrebuild(Meanfilt,sigmaquality,true);
				Proxfilter=arrayrebuild(Proxfilter,sigmaquality,true);
			}
			if(NPsize==true){
				for(i=0;i<Proxfilter.length;i++){
					if(lowlim<Meanfilt[i] && Meanfilt[i]<uplim){
						Intcheck[i]=size1;
					}
					else{
						Intcheck[i]=0;
					}
				}
			}
			if(NPsize==false){
				for(i=0;i<Proxfilter.length;i++){
					if(lowlim<Meanfilt[i] && Meanfilt[i]<lowboundlim){
						Intcheck[i]=size1;
					}
					if(upboundlim<Meanfilt[i] && Meanfilt[i]<uplim){
						Intcheck[i]=size2;
					}
					if(Meanfilt[i]<lowlim || Meanfilt[i]>uplim){
						Intcheck[i]=0;
					}
				}
			}
			if(radnames[radloop]=="singleradius/Regional"){
				size1posSR=arrayvalueindices(Intcheck,size1);
				xs1SR=newArray(size1posSR.length);
				ys1SR=newArray(size1posSR.length);
				for(i=0;i<size1posSR.length;i++){
					xs1SR[i]=xRAD[size1posSR[i]];
					ys1SR[i]=yRAD[size1posSR[i]];
					if(gaussfit==true){
						xygauss=gaussCoord(xs1SR[i],ys1SR[i],radius,maintitle);
						xs1SR[i]=xygauss[0];
						ys1SR[i]=xygauss[1];
					}
				}
				if(NPsize==false){
					size2posSR=arrayvalueindices(Intcheck,size2);
					xs2SR=newArray(size2posSR.length);
					ys2SR=newArray(size2posSR.length);
					for(i=0;i<size2posSR.length;i++){
						xs2SR[i]=xRAD[size2posSR[i]];
						ys2SR[i]=yRAD[size2posSR[i]];
						if(gaussfit==true){
							xygauss=gaussCoord(xs2SR[i],ys2SR[i],radius,maintitle);
							xs2SR[i]=xygauss[0];
							ys2SR[i]=xygauss[1];
						}
					}
				}
			}
			if(radnames[radloop]=="doubleradius"){
				size1posDR=arrayvalueindices(Intcheck,size1);
				xs1DR=newArray(size1posDR.length);
				ys1DR=newArray(size1posDR.length);
				for(i=0;i<size1posDR.length;i++){
					xs1DR[i]=xRAD[size1posDR[i]];
					ys1DR[i]=yRAD[size1posDR[i]];
					if(gaussfit==true){
						xygauss=gaussCoord(xs1DR[i],ys1DR[i],radius,maintitle);
						xs1DR[i]=xygauss[0];
						ys1DR[i]=xygauss[1];
					}
				}
				if(NPsize==false){
					size2posDR=arrayvalueindices(Intcheck,size2);
					xs2DR=newArray(size2posDR.length);
					ys2DR=newArray(size2posDR.length);
					for(i=0;i<size2posDR.length;i++){
						xs2DR[i]=xRAD[size2posDR[i]];
						ys2DR[i]=yRAD[size2posDR[i]];
						if(gaussfit==true){
							xygauss=gaussCoord(xs2DR[i],ys2DR[i],radius,maintitle);
							xs2DR[i]=xygauss[0];
							ys2DR[i]=xygauss[1];
						}
					}
				}
			}
		}
	}
	
	if(polarization==false){
		processname=processing;
	}
	if(polarization==true){
		processname=polproc;
	}
	if(singleradius==true || doubleradius==true){
		if(referenceshift==false){
			newImage("Single/Double","32-bit",datapoints,2,1);
			saveAs("Tiff",processname+"S_D");
			close();
		}
		if(referenceshift==true){
			newImage("Single/Double","32-bit",datapoints,2*2,1);
			saveAs("Tiff",processname+"S_D");
			close();
		}
		singleregion=true;
	}
	if(singleradius==false && doubleradius==false){
		singleregion=false;
	}
	
	
	
	if(regional==true){
		if(NPsize==true){
			regionalS1=true;
			regionalS2=false;
			if(referenceshift==false){
				newImage("Regional_size1","32-bit",datapoints1,size1posSR.length*2,1);
				saveAs("Tiff",processname+"Reg_s1");
				close();
			}
			if(referenceshift==true){
				newImage("Regional_size1","32-bit",datapoints1,size1posSR.length*2*2,1);
				saveAs("Tiff",processname+"Reg_s1");
				close();
			}
		}
		if(NPsize==false){
			regionalS1=true;
			regionalS2=true;
			if(referenceshift==false){
				newImage("Regional_size1","32-bit",datapoints1,size1posSR.length*2,1);
				saveAs("Tiff",processname+"Reg_s1");
				close();
				newImage("Regional_size2","32-bit",datapoints1,size2posSR.length*2,1);
				saveAs("Tiff",processname+"Reg_s2");
				close();
			}
			if(referenceshift==true){
				newImage("Regional_size1","32-bit",datapoints1,size1posSR.length*2*2,1);
				saveAs("Tiff",processname+"Reg_s1");
				close();
				newImage("Regional_size2","32-bit",datapoints1,size2posSR.length*2*2,1);
				saveAs("Tiff",processname+"Reg_s2");
				close();
			}
		}
	}
	if(regional==false){
		regionalS1=false;
		regionalS2=false;
	}
	
	selectWindow(maintitle);
	close();
	
	radsets=newArray(singleregion,regionalS1,regionalS2);
	radname=newArray("singleregion","regional_size1","regional_size2");
	centerx=((ovalwi/2)+rectlocx);
	centery=((ovalhe/2)+rectlocy);
	
	setBatchMode(true);
	
	if(regional==true){
		if(referenceshift==false){
			reflist1=size1posSR.length;
			if(NPsize==false){
				reflist2=size2posSR.length;
			}
		}
		if(referenceshift==true){
			reflist1=2*size1posSR.length;
			xref1=newArray(size1posSR.length);
			yref1=newArray(size1posSR.length);
			for(i=0;i<size1posSR.length;i++){
				xref1[i]=xs1SR[i]+refshiftx;
				yref1[i]=ys1SR[i]+refshifty;
			}
			xs1SR=Array.concat(xs1SR,xref1);
			ys1SR=Array.concat(ys1SR,yref1);
			if(NPsize==false){
				reflist2=2*size2posSR.length;
				xref2=newArray(size2posSR.length);
				yref2=newArray(size2posSR.length);
				for(i=0;i<size1posSR.length;i++){
					xref2[i]=xs2SR[i]+refshiftx;
					yref2[i]=ys2SR[i]+refshifty;
				}
				xs2SR=Array.concat(xs2SR,xref2);
				ys2SR=Array.concat(ys2SR,yref2);
			}
		}
	}
	
	for(radloop=0;radloop<3;radloop++){
		if(radsets[radloop]==true){
			if(radname[radloop]=="singleregion"){
				bgimagename="S_D.tif";
				bgregion=1;
				bRadlimit=sqrt(pow((sqrt(2)/2)*((ovalwi/2)-radius2),2)+pow((sqrt(2)/2)*((ovalhe/2)-radius2),2));
				kRadlimit=2*radius;
				xrad=xprime;
				yrad=yprime;
				c=newArray(numresults);
				AllResults=numresults;
				if(referenceshift==true){
					for(t=0;t<numresults;t++){
						xprimeref[t]=xprime[t]+refshiftx;
						yprimeref[t]=yprime[t]+refshifty;
					}
					xBGref=Array.concat(xprime,xprimeref);
					yBGref=Array.concat(yprime,yprimeref);
					xrad=xBGref;
					yrad=yBGref;
					AllResults=2*numresults;
					c=newArray(2*numresults);
					kRadlimit=2*radius+magrefshift;
				}
				counter1=newArray(1);
				countBG=counter1;
				Datapoints=datapoints;
				open(processname+"S_D.tif");
			}
			if(radname[radloop]=="regional_size1"){
				bgimagename="Reg_s1.tif";
				bgregion=reflist1;
				counter2=newArray(reflist1);
				bRadlimit=regRad-BGradius;
				kRadlimit=2*radius;
				countBG=counter2;
				Datapoints=datapoints1;
				xsize=xs1SR;
				ysize=ys1SR;
				open(processname+"Reg_s1.tif");
				open(mainimage);
				run("Set Scale...", "distance=scale known=1000 pixel=1 unit=nm");
			}
			if(radname[radloop]=="regional_size2"){
				bgimagename="Reg_s2.tif";
				bgregion=reflist2;
				counter3=newArray(reflist2);
				bRadlimit=regRad-BGradius;
				kRadlimit=2*radius;
				countBG=counter3;
				Datapoints=datapoints1;
				xsize=xs2SR;
				ysize=ys2SR;
				open(processname+"Reg_s2.tif");
				selectWindow(maintitle);
			}
	
			for(i=0;i<bgregion;i++){
				if(radname[radloop]=="regional_size1" || radname[radloop]=="regional_size2"){
					selectWindow(maintitle);
					run("Select None");
					run("Clear Results");
					makeOval(xsize[i]-(regRad+radius),ysize[i]-(regRad+radius),(regRad+radius)*2,(regRad+radius)*2);
					run("Find Maxima...", "noise=noisetol output=List exclude");
					RegResults=nResults;
					xOut=newArray(RegResults);
					yOut=newArray(RegResults);
					for(t=0;t<RegResults;t++){
						xOut[t]=getResult("X",t);
						yOut[t]=getResult("Y",t);
					}
					xrad=xOut;
					yrad=yOut;
					AllResults=RegResults;
					if(referenceshift==true){
						xOutref=newArray(RegResults);
						yOutref=newArray(RegResults);
						for(t=0;t<RegResults;t++){
							xOutref[t]=xOut[t]+refshiftx;
							yOutref[t]=yOut[t]+refshifty;
						}
						xBGref=Array.concat(xOut,xOutref);
						yBGref=Array.concat(yOut,yOutref);
						xrad=xBGref;
						yrad=yBGref;
						AllResults=2*RegResults;
					}
				}
				j=0;
				BGdatacount=0;
				while(BGdatacount<Datapoints){
					if(radname[radloop]=="singleregion"){
						bx=rectlocx+random()*ovalwi;
						by=rectlocy+random()*ovalhe;
						bRad=sqrt(pow(bx-centerx,2)+pow(by-centery,2));
					}
					if(radname[radloop]=="regional_size1" || radname[radloop]=="regional_size2"){
						if(referenceshift==false){
							c=newArray(RegResults);
						}
						if(referenceshift==true){
							c=newArray(2*RegResults);
						}
						if(radname[radloop]=="regional_size1"){
							bx=(xsize[i]-regRad)+(random()*2*regRad);
							by=(ysize[i]-regRad)+(random()*2*regRad);
							bRad=sqrt(pow(bx-xsize[i],2)+pow(by-ysize[i],2));
						}
						if(radname[radloop]=="regional_size2"){
								bx=(xsize[i]-regRad)+(random()*2*regRad);
								by=(ysize[i]-regRad)+(random()*2*regRad);
								bRad=sqrt(pow(bx-xsize[i],2)+pow(by-ysize[i],2));
						}
	
					}
					for(k=0;k<AllResults;k++){
						kRad=sqrt(pow(xrad[k]-bx,2)+pow(yrad[k]-by,2));
							if(kRad<kRadlimit){
								c[k]=0;
							}
							else{
								c[k]=1;
							}
					}
					Array.getStatistics(c,min,max,mean,stdDev);
					d=mean;
					if(radnames[radloop]=="singleregion" && referenceshift==false){
						sigmaquality=sigcheck(xrad[k],yrad[k],ROI2,maintitle,Nsig);
						if(bRad>bRadlimit || sigmaquality==false){
							d=d;
						}
						else{
							d++;
						}
					}
					else{
						if(bRad>bRadlimit){
							d=d;
						}
						else{
							d++;
						}
					}
					
					selectWindow(bgimagename);
	
					if(d==2){
						setPixel(j,2*i,bx);
						setPixel(j,2*i+1,by);
						BGdatacount++;
						j++;
					}
					if(d!=2){
						BGdatacount=BGdatacount;
						j=j;
					}
				}
				countBG[i]=Datapoints;
			}
			if(radname[radloop]=="singleregion"){
				selectWindow("S_D.tif");
				saveAs("Tiff",processname+"S_D");
			}
			if(radname[radloop]=="regional_size1"){
				selectWindow("Reg_s1.tif");
				saveAs("Tiff",processname+"Reg_s1");
			}
			if(radname[radloop]=="regional_size2"){
				selectWindow("Reg_s2.tif");
				saveAs("Tiff",processname+"Reg_s2");
			}
			selectWindow(bgimagename);
			close();
		}
	}
	if(regional==true){
		selectWindow(maintitle);
		close();
	
		if(referenceshift==true){
			xs1SR=Array.slice(xs1SR,0,size1posSR.length);
			ys1SR=Array.slice(ys1SR,0,size1posSR.length);
			if(NPsize==false){
				xs2SR=Array.slice(xs2SR,0,size2posSR.length);
				ys2SR=Array.slice(ys2SR,0,size2posSR.length);
			}
		}
	}
	
	///////////////////////////////////////////////////////////////////////////////Particle/BG Measurement Section
	for(poloop=0;poloop<polarnum;poloop++){
	if(polarization==true){
		BASEdirec=polbase+polnames[poloop]+File.separator;
		processing=BASEdirec+"Processing"+File.separator;
	}
	if(NPsize==true){
		SIZE1=true;
		SIZE2=false;
	}
	if(NPsize==false){
		SIZE1=true;
		SIZE2=true;
	}
	extinctionSet=true;
	if(dark==true){
		darkfield=true;
		defocusSet=true;
		darkfieldSet=true;
	}
	else{
		if(dark==false){
			darkfield=false;
			defocusSet=false;
			darkfieldSet=false;
		}
	}
	if(type==ImageType[0]){color=true; BW=false;}
	
	if(type==ImageType[1]){BW=true; color=false;}
	
	
		if(color==true){
			colornames=newArray("B","G","R");
			colorsets=newArray(true,true,true);
			colorlength=3;
			signalcolor=newArray("blue","green","red");
			BGcolor="gray";
			ecSR=newArray("SR - Blue Extinction (nm^2)","SR - Green Extinction (nm^2)","SR - Red Extinction (nm^2)");
			ecRR=newArray("RR - Blue Extinction (nm^2)","RR - Green Extinction (nm^2)","RR - Red Extinction (nm^2)");
			ecDR=newArray("DR - Blue Extinction (nm^2)","DR - Green Extinction (nm^2)","DR - Red Extinction (nm^2)");
			scSR=newArray("SR - Blue Scattering (nm^2)","SR - Green Scattering (nm^2)","SR - Red Scattering (nm^2)");
			scRR=newArray("RR - Blue Scattering (nm^2)","RR - Green Scattering (nm^2)","RR - Red Scattering (nm^2)");
			scDR=newArray("DR - Blue Scattering (nm^2)","DR - Green Scattering (nm^2)","DR - Red Scattering (nm^2)");
		}
		if(BW==true){
			color1=true;
			colornames=newArray("BW",0);
			colorsets=newArray(true,0);
			colorlength=1;
			signalcolor=newArray("black");
			BGcolor="gray";
			ecSR=newArray("SR - BG Extinction (nm^2)",0);
			ecRR=newArray("RR - BG Extinction (nm^2)",0);
			ecDR=newArray("DR - BG Extinction (nm^2)",0);
			scSR=newArray("SR - BG Scattering (nm^2)",0);
			scRR=newArray("RR - BG Scattering (nm^2)",0);
			scDR=newArray("DR - BG Scattering (nm^2)",0);
		}
	imagesets=newArray(extinctionSet,defocusSet,darkfieldSet);
	imagenames=newArray("ext","defocus","dark");
	sizesets=newArray(SIZE1,SIZE2);
	sizenames=newArray("size1","size2");
	
	radsets=newArray(singleradius,doubleradius,regionalS1,regionalS2);
	radnames=newArray("singleradius","doubleradius","regional_size1","regional_size2");
	
	datatype=newArray("SP","Mean","Standard Deviation","SP/Mean(RR)","SP/StdDev(RR)","SP/Break(RR)");
	if(polarization==false){
		savedata=newArray("BG_Data.xls","Extinction_Data.xls");
	}
	if(polarization==true){
		savedata=newArray("BG_Data_"+polnames[poloop]+".xls","Extinction_Data_"+polnames[poloop]+".xls");
	}
	
	run("Clear Results");
	
	brightsaveimages=newArray("SR_BG_ext","DR_BG_ext","region_BG_ext");
	defsaveimages=newArray("SRDR_BG_defocus","SRDR_BG_defocus","region_BG_defocus");
	darksaveimages=newArray("SR_BG_dark","DR_BG_dark","region_BG_dark");
	
	extDistimages=newArray("SR_BG_ext_dist","DR_BG_ext_dist","RR_BG_ext_dist");
	darkDistimages=newArray("SR_BG_dark_dist","DR_BG_dark_dist","RR_BG_dark_dist");
	
	if(singleradius==true){
		newImage("SR_BG_ext","32-bit",colorlength,datapoints,1);
		newImage("SR_BG_ext_dist","32-bit",colorlength,2,1);
		if(darkfield==true){
			newImage("SR_BG_dark","32-bit",colorlength,datapoints,1);
			newImage("SR_BG_dark_dist","32-bit",colorlength,2,1);
			newImage("SRDR_BG_defocus","32-bit",colorlength,datapoints,1);
		}
	}
	
	if(doubleradius==true){
		newImage("DR_BG_ext","32-bit",colorlength,datapoints,1);
		newImage("DR_BG_ext_dist","32-bit",colorlength,2,1);
		if(darkfield==true){
			newImage("DR_BG_dark","32-bit",colorlength,datapoints,1);
			newImage("DR_BG_dark_dist","32-bit",colorlength,2,1);
			if(singleradius==false){
				newImage("SRDR_BG_defocus","32-bit",colorlength,datapoints,1);
			}
		}
	}
	
	if(regional==true){
		if(NPsize==true){
			sum=arraySum(counter2);
			imagelength=datapoints1*reflist1;
			distlength=2*reflist1;
		}
		if(NPsize==false){
			sum1=arraySum(counter2); sum2=arraySum(counter3);
			sum=sum1+sum2;
			imagelength=datapoints1*(reflist1+reflist2);
			distlength=2*(reflist1+reflist2);
		}
		newImage("region_BG_ext","32-bit",colorlength,imagelength,1);
		newImage("RR_BG_ext_dist","32-bit",colorlength,distlength,1);
		if(darkfield==true){
			newImage("region_BG_dark","32-bit",colorlength,imagelength,1);
			newImage("RR_BG_dark_dist","32-bit",colorlength,distlength,1);
			newImage("region_BG_defocus","32-bit",colorlength,imagelength,1);
		}
	}
	
	if((singleradius==true || doubleradius==true) && regional==true){
		if(NPsize==true){
				if(counter1[0]>=(sum+(4*reflist1))){
					meanrow=counter1[0];
				}
				if((sum+(4*size1posSR.length))>counter1[0]){
					meanrow=sum+(4*reflist1);
				}
		}
		if(NPsize==false){
			if(counter1[0]>=(sum+(4*(reflist2+reflist1)))){
				meanrow=counter1[0];
			}
			if((sum+(4*(reflist2+reflist1)))>counter1[0]){
				meanrow=sum+(4*(reflist2+reflist1));
			}
		}
	}
	if(singleradius==false && doubleradius==false && regional==true){
		if(NPsize==true){
			meanrow=sum+(4*reflist1);
		}
		if(NPsize==false){
			meanrow=sum+(4*(reflist2+reflist1));
		}
	}
	if((singleradius==true || doubleradius==true) && regional==false){
		meanrow=counter1[0];
	}
	
	for(images=0;images<3;images++){
		if(imagesets[images]==true){
			if(imagenames[images]=="ext"){
				if(BW==true){
					colorchannel=newArray(BWextname[poloop],0);
				}
				if(color==true){
					colorchannel=newArray(blueextname[poloop],greenextname[poloop],redextname[poloop]);
				}
				saveimages=brightsaveimages;
				DistImages=extDistimages;
			}
			if(imagenames[images]=="dark"){
				if(BW==true){
					colorchannel=newArray(BWdarkname[poloop],0);
				}
				if(color==true){
					colorchannel=newArray(bluedarkname[poloop],greendarkname[poloop],reddarkname[poloop]);
				}
				saveimages=darksaveimages;
				DistImages=darkDistimages;
			}
			if(imagenames[images]=="defocus"){
				if(BW==true){
					colorchannel=newArray(defBWname[poloop],0);
				}
				if(color==true){
					colorchannel=newArray(bluedefname[poloop],greendefname[poloop],reddefname[poloop]);
				}
				saveimages=defsaveimages;
			}
			for(coloop=0;coloop<colorlength;coloop++){
				if(colorsets[coloop]==true){
					if(imagenames[images]=="ext" || imagenames[images]=="dark"){
						DIR=BASEdirec+"Extinction"+File.separator;
					}
					if(imagenames[images]=="defocus"){
						DIR=BASEdirec+defocusfolder+File.separator;
					}
					open(DIR+colorchannel[coloop]);
					run("Set Scale...", "distance=scale known=1000 pixel=1 unit=nm");
					for(radloop=0;radloop<4;radloop++){
						if(radsets[radloop]==true){
							if(radnames[radloop]=="singleradius"){
								xname="X"; yname="Y"; bgregion=1; row=0; bgimagename2="S_D.tif"; Datapoints=datapoints;
							}
							if(radnames[radloop]=="doubleradius"){
								xname="X"; yname="Y"; bgregion=1; row=0; bgimagename2="S_D.tif"; Datapoints=datapoints;
							}
							if(radnames[radloop]=="regional_size1"){
								xname="Xr"; yname="Yr"; bgregion=reflist1; row=0; bgimagename2="Reg_s1.tif"; Datapoints=datapoints1;
							}
							if(radnames[radloop]=="regional_size2"){
								xname="Xr"; yname="Yr"; bgregion=reflist2; bgimagename2="Reg_s2.tif"; Datapoints=datapoints1;
								sumcount=datapoints1*reflist1; row=sumcount+(4*reflist1);
							}
							open(processname+bgimagename2);
							for(i=0;i<meanrow;i++){
								setResult("Label",i,datatype[0]);
							}
							for(i=0;i<bgregion;i++){
								for(j=0;j<Datapoints;j++){
										selectWindow(bgimagename2);
										xBG=getPixel(j,2*i);
										yBG=getPixel(j,2*i+1);
									if(imagenames[images]=="ext"){
										setResult(xname,row+j,xBG);
										setResult(yname,row+j,yBG);
									}
								}
								if(radnames[radloop]=="singleradius" || radnames[radloop]=="doubleradius"){
									row=row+Datapoints;
								}
								if(radnames[radloop]=="regional_size1" || radnames[radloop]=="regional_size2"){
									row=row+Datapoints+4;
								}
							}
							setResult("Label",0,"Single Particle (SP)");
							selectWindow(bgimagename2);
							close();
						}
					}
					for(radloop=0;radloop<4;radloop++){
						if(radsets[radloop]==true){
							if(radnames[radloop]=="singleradius"){
								xname="X"; yname="Y"; extcol=ecSR; scatcol=scSR; bgregion=1; row=0; bgimagename2="S_D.tif";
							}
							if(radnames[radloop]=="doubleradius"){
								xname="X"; yname="Y"; extcol=ecDR; scatcol=scDR; bgregion=1; row=0; bgimagename2="S_D.tif";
							}
							if(radnames[radloop]=="regional_size1"){
								xname="Xr"; yname="Yr"; extcol=ecRR; scatcol=scRR; bgregion=reflist1; row=0; bgimagename2="Reg_s1.tif";
								regionmean1=newArray(reflist1); RegionMean1=newArray(reflist1);
								regionstd1=newArray(reflist1); RegionStd1=newArray(reflist1);
								RegionMean=regionmean1; RegionMeanFinal=RegionMean1;
								RegionStd=regionstd1; RegionStdFinal=RegionStd1;
								imagerow=0;
							}
							if(radnames[radloop]=="regional_size2"){
								xname="Xr"; yname="Yr"; extcol=ecRR; scatcol=scRR; bgregion=reflist2; bgimagename2="Reg_s2.tif";
								regionmean2=newArray(reflist2); RegionMean2=newArray(reflist2);
								regionstd2=newArray(reflist2);	RegionStd2=newArray(reflist2);
								RegionMean=regionmean2; RegionMeanFinal=RegionMean2;
								RegionStd=regionstd2; RegionStdFinal=RegionStd2;
								sumcount=arraySum(counter2);
								row=sumcount+(4*reflist1);
								imagerow=reflist1*datapoints1;
							}
							if(imagenames[images]=="ext"){
								col=extcol;
								if(polarization==false){shift1X=0; shift1Y=0;}
								if(polarization==true){shift1X=shiftExtx[poloop]; shift1Y=shiftExty[poloop];}
							}
							if(imagenames[images]=="dark"){
								col=scatcol;
								if(polarization==false){shift1X=0; shift1Y=0;}
								if(polarization==true){shift1X=darkx+shiftDarkx; shift1Y=darky+shiftDarky;}
							}
							open(processname+bgimagename2);
							for(i=0;i<bgregion;i++){
								if(radnames[radloop]=="singleradius"){
									countBG=counter1;
									mean1=newArray(countBG[i]);
									ext=newArray(countBG[i]);
									mean0=mean1;
								}
								if(radnames[radloop]=="doubleradius"){
									countBG=counter1;
									mean2=newArray(countBG[i]);
									if(referenceshift==true){
										mean3=newArray(countBG[i]);
									}
									ext2=newArray(countBG[i]);
								}
								if(radnames[radloop]=="regional_size1"){
									countBG=counter2;
									mean4=newArray(countBG[i]);
									mean0=mean4;
								}
								if(radnames[radloop]=="regional_size2"){
									countBG=counter3;
									mean5=newArray(countBG[i]);
									mean0=mean5;
								}
								for(j=0;j<countBG[i];j++){
									selectWindow(bgimagename2);
									xBG=getPixel(j,2*i);
									yBG=getPixel(j,2*i+1);
									selectWindow(colorchannel[coloop]);
									if(radnames[radloop]=="singleradius" || radnames[radloop]=="regional_size1" || radnames[radloop]=="regional_size2"){
										makeOval(xBG-BGradius-shift1X,yBG-BGradius-shift1Y,BGradius*2,BGradius*2);
										List.setMeasurements;
										area1=List.getValue("Area");
										mean0[j]=List.getValue("Mean");
									}
									if(radnames[radloop]=="doubleradius"){
										if(referenceshift==false){
											ext2[j]=DRext(xBG,yBG,ROI1,ROI2,colorchannel[coloop],areacal,Nsig);
											makeOval(xBG-radius-shift1X,yBG-radius-shift1Y,diameter,diameter);
											List.setMeasurements;
											mean2[j]=List.getValue("Mean");
										}
										if(referenceshift==true){
											if(magrefshift>(2*radius+2)){
												makeOval(xBG-radius-shift1X,yBG-radius-shift1Y,diameter,diameter);
												List.setMeasurements;
												//area2=List.getValue("Area");
												mean2[j]=List.getValue("Mean");
												makeOval(xBG-radius2-shift1X,yBG-radius2-shift1Y,diameter2,diameter2);
												List.setMeasurements;
												mean3[j]=List.getValue("Mean");
												ext2[j]=areacal*((PI*pow(radius,2)*mean2[j])-((PI*pow(radius,2))*((pow(radius2,2)*mean3[j]-pow(radius,2)*mean2[j])/(pow(radius2,2)-pow(radius,2)))));
												
												makeOval(xBG-radius-shift1X+refshiftx,yBG-radius-shift1Y+refshifty,diameter,diameter);
												List.setMeasurements;
												//area2=List.getValue("Area");
												mean2[j]=List.getValue("Mean");
												makeOval(xBG-radius2-shift1X+refshiftx,yBG-radius2-shift1Y+refshifty,diameter2,diameter2);
												List.setMeasurements;
												mean3[j]=List.getValue("Mean");
												ext2[j]=(ext2[j]-(areacal*((PI*pow(radius,2)*mean2[j])-((PI*pow(radius,2))*((pow(radius2,2)*mean3[j]-pow(radius,2)*mean2[j])/(pow(radius2,2)-pow(radius,2)))))))/2;
											}
											if(magrefshift<=(2*radius+2)){
												makeOval(xBG-radius-shift1X,yBG-radius-shift1Y,diameter,diameter);
												List.setMeasurements;
												//area2=List.getValue("Area");
												mean2[j]=List.getValue("Mean");
												makeOval(xBG-radius-shift1X+refshiftx,yBG-radius-shift1Y+refshifty,diameter,diameter);
												List.setMeasurements;
												mean3[j]=List.getValue("Mean");
												ext2[j]=areacal*PI*pow(radius,2)*(mean2[j]-mean3[j])/2;
											}
										}
									}
								}
								if(imagenames[images]=="ext" || imagenames[images]=="dark"){
									if(radnames[radloop]=="singleradius"){
										Array.getStatistics(mean1,min,max,mean,stdDev);
										MAINBG=mean*area1;
										MAINstd=stdDev*area1;
										selectWindow(DistImages[0]); setPixel(coloop,0,MAINBG);
									}
									if(radnames[radloop]=="doubleradius"){
										Array.getStatistics(ext2,min,max,mean,stdDev);
										ext2BG=mean;
										ext2std=stdDev;
										selectWindow(DistImages[1]); setPixel(coloop,0,ext2BG); setPixel(coloop,1,ext2std);
									}
									if(radnames[radloop]=="regional_size1"){
										Array.getStatistics(mean4,min,max,mean,stdDev);
										RegionMean[i]=mean*area1;
										RegionStd[i]=stdDev*area1;
										selectWindow(DistImages[2]); setPixel(coloop,2*i,RegionMean[i]);
									}
									if(radnames[radloop]=="regional_size2"){
										Array.getStatistics(mean5,min,max,mean,stdDev);
										RegionMean[i]=mean*area1;
										RegionStd[i]=stdDev*area1;
										selectWindow(DistImages[2]); setPixel(coloop,(2*i)+(2*size1posSR.length),RegionMean[i]);
									}
								}
								for(j=0;j<countBG[i];j++){
									if(imagenames[images]=="ext"){
										if(radnames[radloop]=="singleradius"){
											mean1[j]=(mean1[j]*area1)-MAINBG;
											setResult(col[coloop],row+j,mean1[j]);
											selectWindow(saveimages[0]);
											setPixel(coloop,j,mean1[j]);
										}
										if(radnames[radloop]=="doubleradius"){
											setResult(col[coloop],row+j,ext2[j]);
											selectWindow(saveimages[1]);
											setPixel(coloop,j,ext2[j]);
										}
										if(radnames[radloop]=="regional_size1"){
											mean4[j]=(mean4[j]*area1)-regionmean1[i];
											setResult(col[coloop],row+j,mean4[j]);
											selectWindow(saveimages[2]);
											setPixel(coloop,imagerow+j,mean4[j]);
										}
										if(radnames[radloop]=="regional_size2"){
											mean5[j]=(mean5[j]*area1)-regionmean2[i];
											setResult(col[coloop],row+j,mean5[j]);
											selectWindow(saveimages[2]);
											setPixel(coloop,imagerow+j,mean5[j]);
										}
									}
									if(imagenames[images]=="defocus"){
										if(radnames[radloop]=="singleradius"){
											selectWindow(saveimages[0]);
											setPixel(coloop,j,mean1[j]);
										}
										if(radnames[radloop]=="doubleradius" && singleradius==false){
											selectWindow(saveimages[1]);
											setPixel(coloop,j,mean2[j]);
										}
										if(radnames[radloop]=="regional_size1"){
											selectWindow(saveimages[2]);
											setPixel(coloop,imagerow+j,mean4[j]);
										}
										if(radnames[radloop]=="regional_size2"){
											selectWindow(saveimages[2]);
											setPixel(coloop,imagerow+j,mean5[j]);
										}
									}
									if(imagenames[images]=="dark"){
										if(radnames[radloop]=="singleradius"){
											selectWindow("SRDR_BG_defocus.tif");
											defvalue=getPixel(coloop,row+j);
											mean1[j]=Eta[coloop]*((mean1[j]*area1)-MAINBG)/(defvalue*exprel);
											setResult(col[coloop],row+j,mean1[j]);
											selectWindow(saveimages[0]);
											setPixel(coloop,j,mean1[j]);
										}
										if(radnames[radloop]=="doubleradius"){
											selectWindow("SRDR_BG_defocus.tif");
											defvalue=getPixel(coloop,row+j);
											ext2[j]=Eta[coloop]*ext2[j]/(defvalue*exprel);
											setResult(col[coloop],row+j,ext2[j]);
											selectWindow(saveimages[1]);
											setPixel(coloop,j,ext2[j]);
										}
										if(radnames[radloop]=="regional_size1"){
											selectWindow("region_BG_defocus.tif");
											defvalue=getPixel(coloop,imagerow+j);
											mean4[j]=Eta[coloop]*((mean4[j]*area1)-regionmean1[i])/(defvalue*exprel);
											setResult(col[coloop],row+j,mean4[j]);
											selectWindow(saveimages[2]);
											setPixel(coloop,imagerow+j,mean4[j]);
										}
										if(radnames[radloop]=="regional_size2"){
											selectWindow("region_BG_defocus.tif");
											defvalue=getPixel(coloop,imagerow+j);
											mean5[j]=Eta[coloop]*((mean5[j]*area1)-regionmean2[i])/(defvalue*exprel);
											setResult(col[coloop],row+j,mean5[j]);
											selectWindow(saveimages[2]);
											setPixel(coloop,imagerow+j,mean5[j]);
										}
									}
								}
								if(imagenames[images]!="defocus"){
									if(radnames[radloop]=="singleradius" || radnames[radloop]=="doubleradius"){
										row=row+Datapoints;
									}
									if(radnames[radloop]=="regional_size1" || radnames[radloop]=="regional_size2"){
										row=row+Datapoints+4;
										imagerow=imagerow+Datapoints;
	
										if(radnames[radloop]=="regional_size1"){
											Array.getStatistics(mean4,min,max,mean,stdDev);
											RegionMean1[i]=mean;
											RegionStd1[i]=stdDev;
											selectWindow(DistImages[2]);
											setPixel(coloop,(2*i)+1,RegionStd1[i]);
										}
										if(radnames[radloop]=="regional_size2"){
											Array.getStatistics(mean5,min,max,mean,stdDev);
											RegionMean2[i]=mean;
											RegionStd2[i]=stdDev;
											selectWindow(DistImages[2]);
											setPixel(coloop,((2*i)+1)+(2*size1posSR.length),RegionStd2[i]);
										}
										setResult(col[coloop],row-4,0);
										setResult(col[coloop],row-3,RegionMeanFinal[i]);
										setResult(col[coloop],row-2,RegionStdFinal[i]);
										setResult(col[coloop],row-1,0);
										if(singleradius==true || doubleradius==true){
											setResult("Label",row-4,datatype[5]);
											setResult("Label",row-3,datatype[3]);
											setResult("Label",row-2,datatype[4]);
											setResult("Label",row-1,datatype[5]);
										}
										if(singleradius==false && doubleradius==false){
											setResult("Label",row-4,"Break");
											setResult("Label",row-3,datatype[1]);
											setResult("Label",row-2,datatype[2]);
											setResult("Label",row-4,"Break");
										}
									}
								}
								if(imagenames[images]=="defocus"){
									imagerow=imagerow+Datapoints;
								}
							}
							if(imagenames[images]!="defocus"){
								if(radnames[radloop]=="singleradius"){
									Array.getStatistics(mean1,min,max,mean,stdDev);
									fullmean=mean; fullstd=stdDev;
									selectWindow(DistImages[0]); setPixel(coloop,1,fullstd);
								}
								if(imagenames[images]=="dark" && radnames[radloop]=="doubleradius"){
									Array.getStatistics(ext2,min,max,mean,stdDev);
									ext2BG=mean; ext2std=stdDev;
									selectWindow(DistImages[1]); setPixel(coloop,0,ext2BG); setPixel(coloop,1,ext2std);
								}
								if(radnames[radloop]=="regional_size1"){
									selectWindow(saveimages[2]);
									SIheight=getHeight();
									makeRectangle(coloop,0,1,SIheight);
									getRawStatistics(nPixels, mean, min, max, std, histogram);
									fullmean1=mean;
									fullstd1=std;
								}
								if(radnames[radloop]=="regional_size2"){
									selectWindow(saveimages[2]);
									SIheight=getHeight();
									makeRectangle(coloop,0,1,SIheight);
									getRawStatistics(nPixels, mean, min, max, std, histogram);
									fullmean2=mean;
									fullstd2=std;
								}
							}
							selectWindow(bgimagename2);
							close();
							updateResults();
						}
					}
					if(imagenames[images]!="defocus"){
						setResult("Label",meanrow,datatype[1]);
						setResult("Label",meanrow+1,datatype[2]);
						for(radloop=0;radloop<4;radloop++){
							if(radsets[radloop]==true){
								if(radnames[radloop]=="singleradius"){
									extcol=ecSR; scatcol=scSR;
								}
								if(radnames[radloop]=="doubleradius"){
									extcol=ecDR; scatcol=scDR;
								}
								if(radnames[radloop]=="regional_size1"){
									extcol=ecRR; scatcol=scRR;
								}
								if(radnames[radloop]=="regional_size2"){
									extcol=ecRR; scatcol=scRR;
								}
								if(imagenames[images]=="ext"){
									col=extcol;
								}
								if(imagenames[images]=="dark"){
									col=scatcol;
								}
								if(radnames[radloop]=="singleradius"){
									setResult(col[coloop],meanrow,fullmean);
									setResult(col[coloop],meanrow+1,fullstd);
								}
								if(radnames[radloop]=="doubleradius"){
									setResult(col[coloop],meanrow,ext2BG);
									setResult(col[coloop],meanrow+1,ext2std);
								}
								if(regionalS1==true && regionalS2==false){
									if(radnames[radloop]=="regional_size1"){
										setResult(col[coloop],meanrow,fullmean1);
										setResult(col[coloop],meanrow+1,fullstd1);
									}
								}
								if(regionalS1==true && regionalS2==true){
									if(radnames[radloop]=="regional_size2"){
										setResult(col[coloop],meanrow,fullmean2);
										setResult(col[coloop],meanrow+1,fullstd2);
									}
								}
							}
						}
					}
					selectWindow(colorchannel[coloop]);
					close();
				}
			}
			for(radloop=0;radloop<4;radloop++){
				if(radsets[radloop]==true){
					if(radnames[radloop]=="singleradius"){
						selectWindow(saveimages[0]);
						saveAs("Tiff",processing+saveimages[0]);
					}
					if(radnames[radloop]=="doubleradius"){
						if(singleradius==true){
							if(imagenames[images]!="defocus"){
								selectWindow(saveimages[1]);
								saveAs("Tiff",processing+saveimages[1]);
							}
						}
						if(singleradius==false){
							selectWindow(saveimages[1]);
							saveAs("Tiff",processing+saveimages[1]);
						}
					}
					if(regionalS1==true && regionalS2==false){
						if(radnames[radloop]=="regional_size1"){
							selectWindow(saveimages[2]);
							saveAs("Tiff",processing+saveimages[2]);
						}
					}
					if(regionalS1==true && regionalS2==true){
						if(radnames[radloop]=="regional_size2"){
							selectWindow(saveimages[2]);
							saveAs("Tiff",processing+saveimages[2]);
						}
					}
				}
			}
		}
	}
	
	saveAs("Results",BASEdirec+"Extinction"+File.separator+savedata[0]);
	
	for(Q=0;Q<3;Q++){
		if(imagesets[Q]==true){
			if(Q==0){saveimages=brightsaveimages; DistImages=extDistimages;}
			if(Q==1){saveimages=defsaveimages;}
			if(Q==2){saveimages=darksaveimages; DistImages=darkDistimages;}
			if(singleradius==true){
				name=saveimages[0]+".tif";
				selectWindow(name);
				close();
				if(Q==0 || Q==2){
					selectWindow(DistImages[0]);
					saveAs("Tiff",processing+DistImages[0]);
					close();
				}
			}
			if(doubleradius==true){
				name=saveimages[1]+".tif";
				if(singleradius==true && Q!=1){
					selectWindow(name);
					close();
				}
				if(singleradius==false){
					selectWindow(name);
					close();
				}
				if(Q==0 || Q==2){
					selectWindow(DistImages[1]);
					saveAs("Tiff",processing+DistImages[1]);
					close();
				}
			}
			if(regional==true){
				name=saveimages[2]+".tif";
				selectWindow(name);
				close();
				if(Q==0 || Q==2){
					selectWindow(DistImages[2]);
					saveAs("Tiff",processing+DistImages[2]);
					close();
				}
			}
		}
	}
	
	
	////////////////////////////////////////////////////////Signal Data
	if((singleradius==true && doubleradius==true && regional==true) || (singleradius==true && doubleradius==true) || (doubleradius==true && regional==true)){
		if(xs1SR.length==xs1DR.length){
			SRDR1=true;
		}
		if(xs1SR.length>xs1DR.length){
			SRDR1=false;
		}
		if(xs1SR.length<xs1DR.length){
			SRDR1=false;
		}
	
		if(NPsize==false){print("Blue Channel Eta="+EtaB)
			if(xs2SR.length==xs2DR.length){
				SRDR2=true;
			}
			if(xs2SR.length>xs2DR.length){
				SRDR2=false;
			}
			if(xs2SR.length<xs2DR.length){
				SRDR2=false;
			}
		}
	}
	
	brightsaveimages=newArray("SR_ext","DR_ext","region_ext");
	defsaveimages=newArray("SR_defocus","DR_defocus","SR_defocus");
	darksaveimages=newArray("SR_dark","DR_dark","region_dark");
	etasaveimages=newArray("SR_eta","DR_eta","region_eta");
	
	if(singleradius|regional==true && NPsize==true){
		imagelengthSR=xs1SR.length;
	}
	if(singleradius|regional==true && NPsize==false){
		imagelengthSR=xs1SR.length+xs2SR.length;
	}
	if(doubleradius==true && NPsize==true){
		imagelengthDR=xs1DR.length;
	}
	if(doubleradius==true && NPsize==false){
		imagelengthDR=xs1DR.length+xs2DR.length;
	}
	
	setBatchMode(true);
	
	if(singleradius==true){
		newImage("SR_ext","32-bit",colorlength,imagelengthSR,1);
		if(darkfield==true){
			newImage("SR_dark","32-bit",colorlength,imagelengthSR,1);
			newImage("SR_defocus","32-bit",colorlength,imagelengthSR,1);
			if(Etacalibration==true){
				newImage("SR_eta","32-bit",colorlength,imagelengthSR,1);
				SReta=newArray(colorlength);
				SRetaStd=newArray(colorlength);
			}
		}
	}
	
	if(doubleradius==true){
		newImage("DR_ext","32-bit",colorlength,imagelengthDR,1);
		if(darkfield==true){
			newImage("DR_dark","32-bit",colorlength,imagelengthDR,1);
			newImage("DR_defocus","32-bit",colorlength,imagelengthDR,1);
			if(Etacalibration==true){
				newImage("DR_eta","32-bit",colorlength,imagelengthDR,1);
				DReta=newArray(colorlength);
				DRetaStd=newArray(colorlength);
			}
		}
	}
	
	if(regional==true){
		newImage("region_ext","32-bit",colorlength,imagelengthSR,1);
		if(darkfield==true){
			newImage("region_dark","32-bit",colorlength,imagelengthSR,1);
			if(singleradius==false){
				newImage("SR_defocus","32-bit",colorlength,imagelengthSR,1);
			}
			if(Etacalibration==true){
				newImage("region_eta","32-bit",colorlength,imagelengthSR,1);
				RReta=newArray(colorlength);
				RRetaStd=newArray(colorlength);
			}
		}
	}
	
	for(sizeloop=0;sizeloop<2;sizeloop++){
		if(sizesets[sizeloop]==true){
			run("Clear Results");
			for(images=0;images<3;images++){
				if(imagesets[images]==true){
					if(imagenames[images]=="ext"){
						if(polarization==false){xfinalshift=0; yfinalshift=0;}
						if(polarization==true){
							xfinalshift=shiftExtx[poloop];
							yfinalshift=shiftExty[poloop];
						}
						if(BW==true){
							colorchannel=newArray(BWextname[poloop],0);
						}
						if(color==true){
							colorchannel=newArray(blueextname[poloop],greenextname[poloop],redextname[poloop]);
						}
					}
					if(imagenames[images]=="dark"){
						if(polarization==false){xfinalshift=0; yfinalshift=0;}
						if(polarization==true){
							xfinalshift=darkx+shiftDarkx[poloop];
							yfinalshift=darky+shiftDarky[poloop];
						}
						if(BW==true){
							colorchannel=newArray(BWdarkname[poloop],0);
						}
						if(color==true){
							colorchannel=newArray(bluedarkname[poloop],greendarkname[poloop],reddarkname[poloop]);
						}
					}
					if(imagenames[images]=="defocus"){
						if(polarization==false){xfinalshift=0; yfinalshift=0;}
						if(polarization==true){
							xfinalshift=shiftExtx[poloop];
							yfinalshift=shiftExty[poloop];
						}
						if(BW==true){
							colorchannel=newArray(defBWname[poloop],0);
						}
						if(color==true){
							colorchannel=newArray(bluedefname[poloop],greendefname[poloop],reddefname[poloop]);
						}
					}
					for(coloop=0;coloop<colorlength;coloop++){
						if(colorsets[coloop]==true){
							if(imagenames[images]=="ext" || imagenames[images]=="dark"){
								DIR=BASEdirec+"Extinction"+File.separator;
							}
							if(imagenames[images]=="defocus"){
								DIR=BASEdirec+defocusfolder+File.separator;
							}
							open(DIR+colorchannel[coloop]);
							run("Set Scale...", "distance=scale known=1000 pixel=1 unit=nm");
							for(radloop=0;radloop<4;radloop++){
								if(radsets[radloop]==true){
									if(radnames[radloop]=="singleradius"){
										if(sizenames[sizeloop]=="size1"){
											xname="X-SR"; yname="Y-SR"; xpart=xs1SR; ypart=ys1SR; numpart=size1posSR.length; extcol=ecSR; scatcol=scSR; sizerow=0;
										}
										if(sizenames[sizeloop]=="size2"){
											xname="X-SR"; yname="Y-SR"; xpart=xs2SR; ypart=ys2SR; numpart=size2posSR.length; extcol=ecSR; scatcol=scSR; sizerow=xs1SR.length;
										}
										mean1=newArray(numpart); mean0=mean1;
									}
									if(radnames[radloop]=="doubleradius"){
										if(sizenames[sizeloop]=="size1"){
											xname="X-DR"; yname="Y-DR"; xpart=xs1DR; ypart=ys1DR; numpart=size1posDR.length; extcol=ecDR; scatcol=scDR; sizerow=0;
											if(singleradius|regional==true){
												if(SRDR1==true){xname="X-SR"; yname="Y-SR";}
											}
											if(singleradius==false && regional==false){
												xname="X"; yname="Y";
											}
										}
										if(sizenames[sizeloop]=="size2"){
											xname="X-DR"; yname="Y-DR"; xpart=xs2DR; ypart=ys2DR; numpart=size2posDR.length; extcol=ecDR; scatcol=scDR; sizerow=xs1DR.length;
											if(singleradius|regional==true){
												if(SRDR2==true){xname="X-SR"; yname="Y-SR";}
											}
											if(singleradius==false && regional==false){
												xname="X"; yname="Y";
											}
										}
										mean2=newArray(numpart); if(referenceshift==true){mean3=newArray(numpart);} ext=newArray(numpart);
									}
									if(radnames[radloop]=="regional_size1" && sizenames[sizeloop]=="size1"){
										xname="X-SR"; yname="Y-SR"; xpart=xs1SR; ypart=ys1SR; numpart=size1posSR.length; extcol=ecRR; scatcol=scRR; sizerow=0;
										mean4=newArray(numpart); mean0=mean4;
									}
									if(radnames[radloop]=="regional_size2" && sizenames[sizeloop]=="size2"){
										xname="X-SR"; yname="Y-SR"; xpart=xs2SR; ypart=ys2SR; numpart=size2posSR.length; extcol=ecRR; scatcol=scRR; sizerow=xs1SR.length;
										mean5=newArray(numpart); mean0=mean5;
									}
									if(imagenames[images]=="ext"){
										col=extcol;
									}
									if(imagenames[images]=="dark"){
										col=scatcol;
									}
									for(i=0;i<numpart;i++){
										if(radnames[radloop]=="singleradius"){
											if(imagenames[images]=="ext" || imagenames[images]=="dark"){
												if(i==0){open(processing+"SR_BG_"+imagenames[images]+"_dist.tif");} BGMEAN=getPixel(coloop,0);
												if(i==numpart-1){selectWindow("SR_BG_"+imagenames[images]+"_dist.tif"); close();}
											}
										}
										if(radnames[radloop]=="regional_size1" && sizenames[sizeloop]=="size1"){
											if(imagenames[images]=="ext" || imagenames[images]=="dark"){
												if(i==0){open(processing+"RR_BG_"+imagenames[images]+"_dist.tif");}
												selectWindow("RR_BG_"+imagenames[images]+"_dist.tif"); BGMEAN=getPixel(coloop,2*i);
												if(referenceshift==true){BGMEANref=getPixel(coloop,2*(i+size1posSR.length));}
												if(i==numpart-1){selectWindow("RR_BG_"+imagenames[images]+"_dist.tif"); close();}
											}
										}
										if(radnames[radloop]=="regional_size2" && sizenames[sizeloop]=="size2"){
											if(imagenames[images]=="ext" || imagenames[images]=="dark"){
												if(i==0){open(processing+"RR_BG_"+imagenames[images]+"_dist.tif");}
												selectWindow("RR_BG_"+imagenames[images]+"_dist.tif"); BGMEAN=getPixel(coloop,(2*i)+(2*reflist1));
												if(referenceshift==true){BGMEANref=getPixel(coloop,2*(i+size2posSR.length)+(2*reflist1));}
												if(i==numpart-1){selectWindow("RR_BG_"+imagenames[images]+"_dist.tif"); close();}
											}
										}
										if(imagenames[images]=="ext"){
											saveimages=brightsaveimages;
											darkmultiplier=1;
										}
										if(imagenames[images]=="defocus"){
											saveimages=defsaveimages;
										}
										if(imagenames[images]=="dark"){
											saveimages=darksaveimages;
											if(radnames[radloop]=="singleradius"){DEFIMAGE=defsaveimages[0];}
											if(radnames[radloop]=="doubleradius"){DEFIMAGE=defsaveimages[1];}
											if(radnames[radloop]=="regional_size1" || radnames[radloop]=="regional_size2"){DEFIMAGE=defsaveimages[2];}
											selectWindow(DEFIMAGE);
											defvalue=getPixel(coloop,i+sizerow);
											darkmultiplier=Eta[coloop]/(defvalue*exprel);
										}
										if(radnames[radloop]=="singleradius" || (radnames[radloop]=="regional_size1" && sizenames[sizeloop]=="size1") || (radnames[radloop]=="regional_size2" && sizenames[sizeloop]=="size2")){
											selectWindow(colorchannel[coloop]);
											makeOval(xpart[i]-radius-xfinalshift,ypart[i]-radius-yfinalshift,radius*2,radius*2);
											List.setMeasurements;
											area1=List.getValue("Area");
											meanref1=List.getValue("Mean");
											if(referenceshift==false){
												meanref2=0;
												denominator=1;
												BGMEANref=0;
											}
											if(referenceshift==true){
												makeOval(xpart[i]-radius-xfinalshift+refshiftx,ypart[i]-radius-yfinalshift+refshifty,radius*2,radius*2);
												List.setMeasurements;
												meanref2=List.getValue("Mean");
												denominator=2;
											}
											if(imagenames[images]=="ext" || imagenames[images]=="dark"){
												mean0[i]=(((meanref1*area1)-BGMEAN)-((meanref2*area1)-BGMEANref))*darkmultiplier/denominator;
												if(radnames[radloop]=="singleradius"){IMAGE=saveimages[0];}
												if(radnames[radloop]=="regional_size1" || radnames[radloop]=="regional_size2"){IMAGE=saveimages[2];}
												selectWindow(IMAGE); setPixel(coloop,i+sizerow,mean0[i]);
												if(darkfield==true && Etacalibration==true){
													if(radnames[radloop]=="singleradius"){etaimage=etasaveimages[0]; extimage=brightsaveimages[0];}
													if(radnames[radloop]=="regional_size1" || radnames[radloop]=="regional_size2"){etaimage=etasaveimages[2]; extimage=brightsaveimages[2];}
													selectWindow(extimage);
													extvalue=getPixel(coloop,i+sizerow);
													etacal=extvalue*(1.170767/(mean0[i]));
													selectWindow(etaimage);
													setPixel(coloop,i+sizerow,etacal);
												}
											}
										}
										if(radnames[radloop]=="doubleradius"){
											selectWindow(colorchannel[coloop]);
											if(referenceshift==false){
												makeOval(xpart[i]-radius-xfinalshift,ypart[i]-radius-yfinalshift,diameter,diameter);
												List.setMeasurements;
												mean2[i]=List.getValue("Mean");
												ext1=DRext(xpart[i],ypart[i],ROI1,ROI2,colorchannel[coloop],areacal,Nsig);
											}
											if(referenceshift==true){
												if(magrefshift>(2*radius+2)){
													makeOval(xpart[i]-radius-xfinalshift,ypart[i]-radius-yfinalshift,diameter,diameter);
													List.setMeasurements;
													mean2[i]=List.getValue("Mean");
													selectWindow(colorchannel[coloop]);
													makeOval(xpart[i]-radius2-xfinalshift,ypart[i]-radius2-yfinalshift,diameter2,diameter2);
													List.setMeasurements;
													mean3[i]=List.getValue("Mean");
													ext1=areacal*((PI*pow(radius,2)*mean2[j])-((PI*pow(radius,2))*((pow(radius2,2)*mean3[j]-pow(radius,2)*mean2[j])/(pow(radius2,2)-pow(radius,2)))));
													
													makeOval(xpart[i]-radius-xfinalshift+refshiftx,ypart[i]-radius-yfinalshift+refshifty,diameter,diameter);
													List.setMeasurements;
													mean2[i]=List.getValue("Mean");
													selectWindow(colorchannel[coloop]);
													makeOval(xpart[i]-radius2-xfinalshift+refshiftx,ypart[i]-radius2-yfinalshift+refshifty,diameter2,diameter2);
													List.setMeasurements;
													mean3[i]=List.getValue("Mean");
													ext1=(ext1-(areacal*((PI*pow(radius,2)*mean2[j])-((PI*pow(radius,2))*((pow(radius2,2)*mean3[j]-pow(radius,2)*mean2[j])/(pow(radius2,2)-pow(radius,2)))))))/2;
												}
												if(magrefshift<=(2*radius+2)){
													makeOval(xpart[i]-radius-xfinalshift,ypart[i]-radius-yfinalshift,diameter,diameter);
													List.setMeasurements;
													mean2[i]=List.getValue("Mean");
													makeOval(xpart[i]-radius-xfinalshift+refshiftx,ypart[i]-radius-yfinalshift+refshifty,diameter,diameter);
													List.setMeasurements;
													mean3[i]=List.getValue("Mean");
													ext1=areacal*PI*pow(radius,2)*(mean2[i]-mean3[i])/2;
												}
											}
											if(imagenames[images]=="ext" || imagenames[images]=="dark"){
												ext[i]=ext1*darkmultiplier;
												selectWindow(saveimages[1]); setPixel(coloop,i+sizerow,ext[i]);
											}
											if(darkfield==true && Etacalibration==true){
													etaimage=etasaveimages[1]; extimage=brightsaveimages[1];
													selectWindow(extimage);
													extvalue=getPixel(coloop,i+sizerow);
													etacal=extvalue*(1.170767/(ext[i]));
													selectWindow(etaimage);
													setPixel(coloop,i+sizerow,etacal);
											}
										}
										if(imagenames[images]=="defocus"){
											if(radnames[radloop]=="singleradius"){
												selectWindow(saveimages[0]);
												setPixel(coloop,i+sizerow,meanref1);
											}
											if(radnames[radloop]=="doubleradius"){
												selectWindow(saveimages[1]);
												setPixel(coloop,i+sizerow,mean2[i]);
											}
											if(singleradius==false){
												if(radnames[radloop]=="regional_size1" && sizenames[sizeloop]=="size1"){
													selectWindow(saveimages[2]);
													setPixel(coloop,i,meanref1);
												}
												if(radnames[radloop]=="regional_size2" && sizenames[sizeloop]=="size2"){
													selectWindow(saveimages[2]);
													setPixel(coloop,i+sizerow,meanref1);
												}
											}
										}
									}
								}
							}
							selectWindow(colorchannel[coloop]);
							close();
						}
					}
				}
			}
			if((NPsize==true && sizenames[sizeloop]=="size1") || (NPsize==false && sizenames[sizeloop]=="size2")){
				for(images=0;images<3;images++){
					if(imagesets[images]==true){
						if(imagenames[images]=="ext"){saveimages=brightsaveimages;}
						if(imagenames[images]=="defocus"){saveimages=defsaveimages;}
						if(imagenames[images]=="dark"){saveimages=darksaveimages;}
						for(radloop=0;radloop<4;radloop++){
							if(radsets[radloop]==true){
								if(radnames[radloop]=="singleradius"){
									selectWindow(saveimages[0]);
									saveAs("Tiff",processing+saveimages[0]);
									close();
									if(imagenames[images]=="dark" && Etacalibration==true){
										selectWindow(etasaveimages[0]);
										for(coloop=0;coloop<colorlength;coloop++){
											makeRectangle(coloop,0,1,imagelengthSR);
											getRawStatistics(nPixels, mean, min, max, std, histogram);
											SReta[coloop]=mean;
											SRetaStd[coloop]=std;
											print("SR eta "+coloop+"= "+SReta[coloop]);
											print("SR eta StdDev "+coloop+"= "+SRetaStd[coloop]);
										}
										selectWindow(etasaveimages[0]);
										saveAs("Tiff",processing+etasaveimages[0]);
										close();
									}
								}
								if(radnames[radloop]=="doubleradius"){
									selectWindow(saveimages[1]);
									saveAs("Tiff",processing+saveimages[1]);
									close();
									if(imagenames[images]=="dark" && Etacalibration==true){
										selectWindow(etasaveimages[1]);
										for(coloop=0;coloop<colorlength;coloop++){
											makeRectangle(coloop,0,1,imagelengthDR);
											getRawStatistics(nPixels, mean, min, max, std, histogram);
											DReta[coloop]=mean;
											DRetaStd[coloop]=std;
											print("DR eta "+coloop+"= "+DReta[coloop]);
											print("DR eta StdDev "+coloop+"= "+DRetaStd[coloop]);
										}
										selectWindow(etasaveimages[1]);
										saveAs("Tiff",processing+etasaveimages[1]);
										close();
									}
								}
								if(regionalS1==true && regionalS2==false){
									if(singleradius==true && imagenames[images]!="defocus"){
										if(radnames[radloop]=="regional_size1"){
											selectWindow(saveimages[2]);
											saveAs("Tiff",processing+saveimages[2]);
											close();
										}
									}
									if(singleradius==false){
										if(radnames[radloop]=="regional_size1"){
											selectWindow(saveimages[2]);
											saveAs("Tiff",processing+saveimages[2]);
											close();
										}
									}
									if(radnames[radloop]=="regional_size1" && imagenames[images]=="dark" && Etacalibration==true){
										selectWindow(etasaveimages[2]);
										for(coloop=0;coloop<colorlength;coloop++){
											makeRectangle(coloop,0,1,imagelengthSR);
											getRawStatistics(nPixels, mean, min, max, std, histogram);
											RReta[coloop]=mean;
											RRetaStd[coloop]=std;
											print("RR eta "+coloop+"= "+RReta[coloop]);
											print("RR eta StdDev "+coloop+"= "+RRetaStd[coloop]);
										}
										selectWindow(etasaveimages[2]);
										saveAs("Tiff",processing+etasaveimages[2]);
										close();
									}
								}
								if(regionalS1==true && regionalS2==true){
									if(singleradius==true && imagenames[images]!="defocus"){
										if(radnames[radloop]=="regional_size2"){
											selectWindow(saveimages[2]);
											saveAs("Tiff",processing+saveimages[2]);
											close();
										}
									}
									if(singleradius==false){
										if(radnames[radloop]=="regional_size2"){
											selectWindow(saveimages[2]);
											saveAs("Tiff",processing+saveimages[2]);
											close();
										}
									}
									if(radnames[radloop]=="regional_size2" && imagenames[images]=="dark" && Etacalibration==true){
										selectWindow(etasaveimages[2]);
										for(coloop=0;coloop<colorlength;coloop++){
											makeRectangle(coloop,0,1,imagelengthSR);
											getRawStatistics(nPixels, mean, min, max, std, histogram);
											RReta[coloop]=mean;
											RRetaStd[coloop]=std;
											print("RR eta mean"+coloop+"= "+RReta[coloop]);
											print("RR eta StdDev "+coloop+"= "+RRetaStd[coloop]);
										}
										selectWindow(etasaveimages[2]);
										saveAs("Tiff",processing+etasaveimages[2]);
										close();
									}
								}
							}
						}
					}
				}
			}
		}
	}
	
	////////////////////////////////////////////////////////
	
	setBatchMode(true);
	
	///////////////////Image Particle Labelling
	if((singleradius==true && doubleradius==false && regional==false) || (singleradius==false && doubleradius==true && regional==false) || (singleradius==false && doubleradius==false && regional==true)){
		marking=newArray("'","''"); SRmarking=""; DRmarking=""; RRmarking="";
	}
	if((singleradius==true && doubleradius==true && regional==true) || (singleradius==true && doubleradius==true) || (singleradius==true && regional==true) || (doubleradius==true && regional==true)){
		marking=newArray("'","''"); SRmarking="S"; DRmarking="D"; RRmarking="R";
	}
	///////////////////Color Filtering
	for(sizeloop=0;sizeloop<2;sizeloop++){
		if(sizesets[sizeloop]==true){
			if(poloop==0){
				open(mainparticle[poloop]);
			}
			for(radloop=0;radloop<4;radloop++){
				if(radsets[radloop]==true){
					if(radloop<3){open(processing+brightsaveimages[radloop]+".tif");}
					if(radloop==3){open(processing+brightsaveimages[radloop-1]+".tif");}
					rename("currentframe");
					if(radnames[radloop]=="singleradius" || radnames[radloop]=="regional_size1" || radnames[radloop]=="regional_size2"){
						if(radnames[radloop]=="singleradius"){extcol=ecSR; scatcol=scSR; xname="X-SR"; yname="Y-SR";}
						if(radnames[radloop]=="regional_size1" || radnames[radloop]=="reginal_size2"){extcol=ecRR; scatcol=scRR; xname="X-RR"; yname="Y-RR";}
						if(sizenames[sizeloop]=="size1"){numpart=xs1SR.length; xpart=xs1SR; ypart=ys1SR; row=0;}
						if(sizenames[sizeloop]=="size2"){numpart=xs2SR.length; xpart=xs2SR; ypart=ys2SR; row=xs1SR.length;}
						colorfilter1=newArray(numpart); ColorFilter=colorfilter1;
					}
					if(radnames[radloop]=="doubleradius"){
						extcol=ecDR; scatcol=scDR; xname="X-DR"; yname="Y-DR";
						if(sizenames[sizeloop]=="size1"){numpart=xs1DR.length; xpart=xs1DR; ypart=ys1DR; row=0;}
						if(sizenames[sizeloop]=="size2"){numpart=xs2DR.length; xpart=xs2DR; ypart=ys2DR; row=xs1DR.length;}
						colorfilter2=newArray(numpart); ColorFilter=colorfilter2;
					}
					for(i=0;i<numpart;i++){
						if(matchoice!=mat[2] && type==ImageType[0]){
							selectWindow("currentframe");
							blue=getPixel(0,i+row);
							green=getPixel(1,i+row);
							red=getPixel(2,i+row);
						}
						if(matchoice==mat[0]){
							if(blue>green && green>red){
								ColorFilter[i]=1;
							}
							else{
								ColorFilter[i]=0;
							}
						}
						if(matchoice==mat[1]){
							if(red>blue && green>blue){
								ColorFilter[i]=1;
							}
							else{
								ColorFilter[i]=0;
							}
						}
						if(matchoice==mat[2] || type==ImageType[1]){
							ColorFilter[i]=1;
						}
					}
					selectWindow("currentframe");
					close();
					finalfilter=arrayvalueindices(ColorFilter,1);
					finalcount=arrayvalue(ColorFilter,1);
					runlength=1;
					if(darkfield==true){runlength=2;}
					if(radnames[radloop]=="singleradius"){
						radmark=SRmarking; markspotx=-10; markspoty=-10;
						if(sizenames[sizeloop]=="size1"){
							extnew=newArray(finalcount); extcross=extnew; newImage("SR_pol_ext_size1","32-bit",colorlength,finalcount,1); polextname="SR_pol_ext_size1";
							if(darkfield==true){scatnew=newArray(finalcount); darkcross=scatnew; newImage("SR_pol_dark_size1","32-bit",colorlength,finalcount,1); poldarkname="SR_pol_dark_size1";}
						}
						if(sizenames[sizeloop]=="size2"){
							extnew4=newArray(finalcount); extcross=extnew4; newImage("SR_pol_ext_size2","32-bit",colorlength,finalcount,1); polextname="SR_pol_ext_size2";
							if(darkfield==true){scatnew4=newArray(finalcount); darkcross=scatnew4; newImage("SR_pol_dark_size2","32-bit",colorlength,finalcount,1); poldarkname="SR_pol_dark_size2";}
						}
					}
					if(radnames[radloop]=="doubleradius"){
						radmark=DRmarking; markspotx=12; markspoty=10;
						if(sizenames[sizeloop]=="size1"){
							extnew2=newArray(finalcount); extcross=extnew2; newImage("DR_pol_ext_size1","32-bit",colorlength,finalcount,1); polextname="DR_pol_ext_size1";
							if(darkfield==true){scatnew2=newArray(finalcount); darkcross=scatnew2; newImage("DR_pol_dark_size1","32-bit",colorlength,finalcount,1);  poldarkname="DR_pol_dark_size1";}
						}
						if(sizenames[sizeloop]=="size2"){
							extnew5=newArray(finalcount); extcross=extnew5; newImage("DR_pol_ext_size2","32-bit",colorlength,finalcount,1); polextname="DR_pol_ext_size2";
							if(darkfield==true){scatnew5=newArray(finalcount); darkcross=scatnew5; newImage("DR_pol_dark_size2","32-bit",colorlength,finalcount,1);  poldarkname="DR_pol_dark_size2";}
						}
					}
					if(radnames[radloop]=="regional_size1" && sizenames[sizeloop]=="size1"){
						radmark=RRmarking; markspotx=-10; markspoty=30;
						extnew3=newArray(finalcount); extcross=extnew3; newImage("RR_pol_ext_size1","32-bit",colorlength,finalcount,1); polextname="RR_pol_ext_size1";
						if(darkfield==true){scatnew3=newArray(finalcount); darkcross=scatnew3; newImage("RR_pol_dark_size1","32-bit",colorlength,finalcount,1); poldarkname="RR_pol_dark_size1";}
					}
					if(radnames[radloop]=="regional_size2" && sizenames[sizeloop]=="size2"){
						radmark=RRmarking; markspotx=-10; markspoty=30;
						extnew6=newArray(finalcount); extcross=extnew6; newImage("RR_pol_ext_size2","32-bit",colorlength*polarnum,finalcount,1);  polextname="RR_pol_ext_size2";
						if(darkfield==true){scatnew6=newArray(finalcount); darkcross=scatnew6; newImage("RR_pol_dark_size2","32-bit",colorlength*polarnum,finalcount,1); poldarkname="RR_pol_dark_size2";}
					}
					for(images=0;images<runlength;images++){
						if(images==0){saveimages=brightsaveimages; col=extcol; crosssection=extcross; polname=polextname; etacalmul=newArray(1,1,1);}
						if(images==1){
							saveimages=darksaveimages; col=scatcol; crosssection=darkcross; polname=poldarkname; etacalmul=newArray(1,1,1);
							if(Etacalibration==true && radnames[radloop]=="singleradius"){etacalmul=SReta;}
							if(Etacalibration==true && radnames[radloop]=="doubleradius"){etacalmul=DReta;}
							if(Etacalibration==true && (radnames[radloop]=="regional_size1" || radnames[radloop]=="regional_size2")){etacalmul=RReta;}
						}
						if(radloop<3){
							open(processing+saveimages[radloop]+".tif");
							rename("currentframe");
						}
						if(radloop==3){
							open(processing+saveimages[radloop-1]+".tif");
							rename("currentframe");
						}
						for(coloop=0;coloop<colorlength;coloop++){
							for(i=0;i<finalcount;i++){
								selectWindow("currentframe");
								crosssection[i]=etacalmul[coloop]*getPixel(coloop,finalfilter[i]+row);
								if(coloop==0){
									setResult("Label",i,datatype[0]);
									setResult(xname,i,xpart[finalfilter[i]]);
									setResult(yname,i,ypart[finalfilter[i]]);
								}
								setResult(col[coloop],i,crosssection[i]);
							}
							Array.getStatistics(crosssection,min,max,mean,stdDev);
							crosssectionmean=mean;
							crosssectionstd=stdDev;
							setResult(col[coloop],finalcount,crosssectionmean);
							setResult(col[coloop],finalcount+1,crosssectionstd);
							if(radnames[radloop]=="singleradius" || radnames[radloop]=="doubleradius" || (radnames[radloop]=="regional_size1" && sizenames[sizeloop]=="size1") || (radnames[radloop]=="regional_size2" && sizenames[sizeloop]=="size2")){
								selectWindow(polname);
								for(i=0;i<finalcount;i++){
									setPixel(coloop,i,crosssection[i]);
								}
							}
						}
						selectWindow("currentframe"); close();
						if(radnames[radloop]=="singleradius" || radnames[radloop]=="doubleradius" || (radnames[radloop]=="regional_size1" && sizenames[sizeloop]=="size1") || (radnames[radloop]=="regional_size2" && sizenames[sizeloop]=="size2")){
							selectWindow(polname);
							saveAs("Tiff",processing+polname);
							close();
						}
						if(polarization==false && (radnames[radloop]=="singleradius" || radnames[radloop]=="doubleradius" || (radnames[radloop]=="regional_size1" && sizenames[sizeloop]=="size1") || (radnames[radloop]=="regional_size2" && sizenames[sizeloop]=="size2"))){
							selectWindow(regfiles[poloop]);
							for(i=0;i<finalcount;i++){
								setBackgroundColor(0,0,0);
								setForegroundColor(255,255,255);
								setFont("Times New Roman", 16, " anti-aliased");
								drawString(i+1+radmark+marking[sizeloop], xpart[finalfilter[i]]+markspotx, ypart[finalfilter[i]]+markspoty);
							}
						}
					}
					updateResults();
				}
			}
			selectWindow("Results");
			if(polarization==false){
				saveAs("Results",BASEdirec+"Extinction"+File.separator+sizenames[sizeloop]+"_data.xls");
			}
			if(polarization==true){
				saveAs("Results",BASEdirec+"Extinction"+File.separator+sizenames[sizeloop]+"_data_"+polnames[poloop]+".xls");
			}
			run("Clear Results");
			if(polarization==false){
				if(NPsize==true){
					selectWindow(regfiles[poloop]);
					saveAs("Tiff",BASEdirec+"Extinction"+File.separator+"Labeled_particles");
					close();
				}
				if(NPsize==false){
					if(sizenames[sizeloop]=="size2"){
						selectWindow(regfiles[poloop]);
						saveAs("Tiff",BASEdirec+"Extinction"+File.separator+"Labeled_particles");
						close();
					}
				}
			}
		}
	}
	
	if(Etacalibration==true){
		BGdarksaveimages=newArray("SR_BG_dark.tif","DR_BG_dark.tif","region_BG_dark.tif");
		darkDistimages=newArray("SR_BG_dark_dist.tif","DR_BG_dark_dist.tif","RR_BG_dark_dist.tif");
		open(BASEdirec+"Extinction"+File.separator+savedata[poloop]);
		for(radloop=0;radloop<3;radloop++){
			if(radsets[radloop]==true){
				if(radloop==0){AllEta=SReta; scatcol=scSR;}
				if(radloop==1){AllEta=DReta; scatcol=scDR;}
				if(radloop==2){AllEta=RReta; scatcol=scRR;}
				open(processing+BGdarksaveimages[radloop]);
				imgheight=getHeight();
				open(processing+darkDistimages[radloop]);
				distimgheight=getHeight();
				for(coloop=0;coloop<colorlength;coloop++){
					selectWindow("Results");
					for(i=0;i<meanrow+2;i++){
						unscaled=getResult(scatcol[coloop],i);
						scaled=unscaled*AllEta[coloop];
						setResult(scatcol[coloop],i,scaled);
					}
					selectWindow(BGdarksaveimages[radloop]);
					for(i=0;i<imgheight;i++){
						unscaled=getPixel(coloop,i);
						scaled=unscaled*AllEta[coloop];
						setPixel(coloop,i,scaled);
					}
					selectWindow(darkDistimages[radloop]);
					for(i=0;i<distimgheight;i++){
						unscaled=getPixel(coloop,i);
						scaled=unscaled*AllEta[coloop];
						setPixel(coloop,i,scaled);
					}
				}
				selectWindow(BGdarksaveimages[radloop]);
				saveAs("Tiff",processing+BGdarksaveimages[radloop]);
				close();
				selectWindow(darkDistimages[radloop]);
				saveAs("Tiff",processing+darkDistimages[radloop]);
				close();
			}
		}
		selectWindow("Results");
		saveAs("Results",BASEdirec+"Extinction"+File.separator+"BG_data_scaled.xls");
		run("Clear Results");
	}
	
	/////////////////////////Unpolarized Histograms
	
	////////////////////////Prep
	
	File.makeDirectory(BASEdirec+"Extinction"+File.separator+"Histograms");
	File.makeDirectory(BASEdirec+"Extinction"+File.separator+"Histograms"+File.separator+"size1");
	if(NPsize==false){
		File.makeDirectory(BASEdirec+"Extinction"+File.separator+"Histograms"+File.separator+"size2");
	}
	if((singleradius==true && doubleradius==false && regional==false) || (singleradius==false && doubleradius==true && regional==false) || (singleradius==false && doubleradius==false && regional==true)){
		S1hist=BASEdirec+"Extinction"+File.separator+"Histograms"+File.separator+"size1"+File.separator;
		if(NPsize==false){
			S2hist=BASEdirec+"Extinction"+File.separator+"Histograms"+File.separator+"size2"+File.separator;
		}
		multihist=false;
	}
	if((singleradius==true && doubleradius==true && regional==true) || (singleradius==true && doubleradius==true) || (singleradius==true && regional==true) || (doubleradius==true && regional==true)){
		if(singleradius==true){
			File.makeDirectory(BASEdirec+"Extinction"+File.separator+"Histograms"+File.separator+"size1"+File.separator+"SR");
			s1SRhist=BASEdirec+"Extinction"+File.separator+"Histograms"+File.separator+"size1"+File.separator+"SR"+File.separator;
			if(NPsize==false){
				File.makeDirectory(BASEdirec+"Extinction"+File.separator+"Histograms"+File.separator+"size2"+File.separator+"SR");
				s2SRhist=BASEdirec+"Extinction"+File.separator+"Histograms"+File.separator+"size2"+File.separator+"SR"+File.separator;
			}
		}
		if(doubleradius==true){
			File.makeDirectory(BASEdirec+"Extinction"+File.separator+"Histograms"+File.separator+"size1"+File.separator+"DR");
			s1DRhist=BASEdirec+"Extinction"+File.separator+"Histograms"+File.separator+"size1"+File.separator+"DR"+File.separator;
			if(NPsize==false){
				File.makeDirectory(BASEdirec+"Extinction"+File.separator+"Histograms"+File.separator+"size2"+File.separator+"DR");
				s2DRhist=BASEdirec+"Extinction"+File.separator+"Histograms"+File.separator+"size2"+File.separator+"DR"+File.separator;
			}
		}
		if(regional==true){
			File.makeDirectory(BASEdirec+"Extinction"+File.separator+"Histograms"+File.separator+"size1"+File.separator+"RR");
			s1RRhist=BASEdirec+"Extinction"+File.separator+"Histograms"+File.separator+"size1"+File.separator+"RR"+File.separator;
			if(NPsize==false){
				File.makeDirectory(BASEdirec+"Extinction"+File.separator+"Histograms"+File.separator+"size2"+File.separator+"RR");
				s2RRhist=BASEdirec+"Extinction"+File.separator+"Histograms"+File.separator+"size2"+File.separator+"RR"+File.separator;
			}
		}
		multihist=true;
	}
	
	imagenames=newArray("ext","dark");
	imagesets=newArray(extinctionSet,darkfieldSet);
	
	radnames=newArray("singleradius","doubleradius","regional");
	radsets=newArray(singleradius,doubleradius,regional);
	
	BGextimages=newArray("SR_BG_ext.tif","DR_BG_ext.tif","region_BG_ext.tif");
	BGdarkimages=newArray("SR_BG_dark.tif","DR_BG_dark.tif","region_BG_dark.tif");
	
	s1ext=newArray("SR_pol_ext_size1.tif","DR_pol_ext_size1.tif","RR_pol_ext_size1.tif");
	if(NPsize==false){
		s2ext=newArray("SR_pol_ext_size2.tif","DR_pol_ext_size2.tif","RR_pol_ext_size2.tif");
	}
	if(dark==true){
		s1dark=newArray("SR_pol_dark_size1.tif","DR_pol_dark_size1.tif","RR_pol_dark_size1.tif");
		if(NPsize==false){
			s2dark=newArray("SR_pol_dark_size2.tif","DR_pol_dark_size2.tif","RR_pol_dark_size2.tif");
		}
	}
	
	/////////////////////////////////////////Execution
	for(sizeloop=0;sizeloop<2;sizeloop++){
		if(sizesets[sizeloop]==true){
			for(radloop=0;radloop<3;radloop++){
				if(radsets[radloop]==true){
					for(imloop=0;imloop<2;imloop++){
						if(imagesets[imloop]==true){
							if(sizenames[sizeloop]=="size1"){
								sizetitle=d2s(size1,0)+"nm";
								if(multihist==false){histdir=S1hist;}
								if(multihist==true){
									if(radnames[radloop]=="singleradius"){histdir=s1SRhist;}
									if(radnames[radloop]=="doubleradius"){histdir=s1DRhist;}
									if(radnames[radloop]=="regional"){histdir=s1RRhist;}
								}
								if(imagenames[imloop]=="ext"){BGfile=BGextimages; files=s1ext; crosstitle="Extinction ";}
								if(imagenames[imloop]=="dark"){BGfile=BGdarkimages; files=s1dark; crosstitle="Scattering ";}
							}
							if(sizenames[sizeloop]=="size2"){
								sizetitle=d2s(size2,0)+"nm";
								if(multihist==false){histdir=S2hist;}
								if(multihist==true){
									if(radnames[radloop]=="singleradius"){histdir=s2SRhist;}
									if(radnames[radloop]=="doubleradius"){histdir=s2DRhist;}
									if(radnames[radloop]=="regional"){histdir=s2RRhist;}
								}
								if(imagenames[imloop]=="ext"){BGfile=BGextimages; files=s2ext; crosstitle="Extinction ";}
								if(imagenames[imloop]=="dark"){BGfile=BGdarkimages; files=s2dark; crosstitle="Scattering ";}
							}
	
							open(processing+BGfile[radloop]);
							open(processing+files[radloop]);
	
							BGpixelmax=newArray(colorlength); BGpixelmean=newArray(colorlength); BGpixelstd=newArray(colorlength); BGcountmax=newArray(colorlength); BGpixelmin=newArray(colorlength);
							pixelmax=newArray(colorlength); pixelmean=newArray(colorlength); pixelstd=newArray(colorlength); countmax=newArray(colorlength); pixelmin=newArray(colorlength);
							CrossStd=newArray(colorlength);
	
							for(coloop=0;coloop<colorlength;coloop++){
								selectWindow(BGfile[radloop]);
								BGheight=getHeight();
								nBGbins=round((log(BGheight)/log(2))+1);
								makeRectangle(coloop,0,1,BGheight);
								getRawStatistics(nPixels, mean, min, max, std, histogram);
								BGpixelmax[coloop]=max;
								BGpixelmin[coloop]=min;
								BGpixelmean[coloop]=mean;
								BGpixelstd[coloop]=std;
								getHistogram(values, counts, nBGbins);
								Array.getStatistics(counts,min,max,mean,stdDev);
								BGcountmax[coloop]=max;
	
								selectWindow(files[radloop]);
								histheight=getHeight();
								makeRectangle(coloop,0,1,histheight);
								getRawStatistics(nPixels, mean, min, max, std, histogram);
								pixelmax[coloop]=max;
								pixelmin[coloop]=min;
								pixelmean[coloop]=mean;
								pixelstd[coloop]=std;
	
								CrossStd[coloop]=sqrt(pow(pixelstd[coloop],2)-pow(BGpixelstd[coloop],2));
							}
	
							Array.getStatistics(BGpixelmax,min,max,mean,stdDev);
							bgmaxindex=arrayvalueindices(BGpixelmax,max);
	
							Array.getStatistics(BGpixelmin,min,max,mean,stdDev);
							bgminindex=arrayvalueindices(BGpixelmin,min);
	
							Array.getStatistics(BGcountmax,min,max,mean,stdDev);
							BGcountmaxindex=arrayvalueindices(BGcountmax,max);
	
							Array.getStatistics(pixelmax,min,max,mean,stdDev);
							maxindex=arrayvalueindices(pixelmax,max);
	
							Array.getStatistics(pixelmin,min,max,mean,stdDev);
							minindex=arrayvalueindices(pixelmin,min);
	
							BGbinwidth=(BGpixelmax[bgmaxindex[0]]-BGpixelmin[bgminindex[0]])/nBGbins;
	
							for(coloop=0;coloop<colorlength;coloop++){
								selectWindow(files[radloop]);
								histheight=getHeight();
								nBins=round(log(histheight)/log(2)+1);
								makeRectangle(coloop,0,1,histheight);
								getHistogram(values, counts, nBins);
								Array.getStatistics(counts,min,max,mean,stdDev);
								countmax[coloop]=max;
							}
	
							binwidth=(pixelmax[maxindex[0]]-pixelmin[minindex[0]])/nBins;
	
							Array.getStatistics(countmax,min,max,mean,stdDev);
							countmaxindex=arrayvalueindices(countmax,max);
	
							//if(countmax[countmaxindex[0]]>BGcountmax[BGcountmaxindex[0]])
								MAX=countmax[countmaxindex[0]];
							//if(BGcountmax[BGcountmaxindex[0]]>countmax[countmaxindex[0]])
								BGMAX=BGcountmax[BGcountmaxindex[0]];
							//if(BGbinwidth<1)
							//	increment=0.00001;
							//if(BGbinwidth>1)
							//	increment=1;
							increment=(pixelmax[maxindex[0]]-3*BGpixelmin[bgminindex[0]])/800;
							
							for(coloop=0;coloop<colorlength;coloop++){
								selectWindow(BGfile[radloop]);
								BGheight=getHeight();
								makeRectangle(coloop,0,1,BGheight);
								getHistogram(values, counts, nBGbins);
								BGvalues=values;
								BGcounts=counts;
	
								selectWindow(files[radloop]);
								histheight=getHeight();
								makeRectangle(coloop,0,1,histheight);
								getHistogram(values, counts, nBins);
								values=values;
								counts=counts;
	
								Plot.create(colornames[coloop]+" Channel Cross sections", crosstitle+" Cross Section", "Counts (BG counts are normalized in gray)");
								Plot.setFrameSize(800,400);
								Plot.setLimits(3*BGpixelmin[bgminindex[0]],pixelmax[maxindex[0]],0,MAX+0.3*MAX);
								Plot.setColor("gray");
								for(i=0;i<nBGbins;i++){
									for(j=0;j<BGbinwidth-0.1*BGbinwidth;j+=increment){
										Plot.drawLine(BGvalues[i]+j,0,BGvalues[i]+j,BGcounts[i]*MAX/BGMAX);
									}
								}
								Plot.setColor(signalcolor[coloop]);
								for(i=0;i<nBins;i++){
									for(j=0;j<binwidth-0.1*binwidth;j+=increment){
										Plot.drawLine(values[i]+j,0,values[i]+j,counts[i]);
									}
								}
								Plot.setJustification("right");
								Plot.setColor(signalcolor[coloop]);
								Plot.addText("Mean Cross Section="+pixelmean[coloop],1,0.07);
								Plot.addText("Cross section Std. Dev.= "+CrossStd[coloop],1,0.105);
								Plot.setColor(BGcolor);
								Plot.addText("Total Noise (particles)="+pixelstd[coloop],1,0.14);
								Plot.addText("Background Noise= "+BGpixelstd[coloop],1,0.175);
								Plot.show();
								saveAs("PNG",histdir+colornames[coloop]+"_"+sizetitle+"_"+crosstitle+".png");
								close();
							}
							selectWindow(BGfile[radloop]);
							close();
							selectWindow(files[radloop]);
							close();
						}
					}
				}
			}
		}
	}
	//////////////////////////End Polarization Loop
	}
	}
	///////////////////////////////////////////////Begin Polarization Analysis
	
	////////////////////////////Polarization Folders
	if(polarization==true && partanalysis==true){
		for(radloop=0;radloop<3;radloop++){
			if(radsets[radloop]==true){
				if(radnames[radloop]=="singleradius"){foldname="SR_Method_Data";}
				if(radnames[radloop]=="doubleradius"){foldname="DR_Method_Data";}
				if(radnames[radloop]=="regional"){foldname="RR_Method_Data";}
				File.makeDirectory(polbase+"Polarization_data"+File.separator+foldname);
				File.makeDirectory(polbase+"Polarization_data"+File.separator+foldname+File.separator+"Experimental_Ensemble_Histograms");
				File.makeDirectory(polbase+"Polarization_data"+File.separator+foldname+File.separator+"Simulated_Noise_Data");
				File.makeDirectory(polbase+"Polarization_data"+File.separator+foldname+File.separator+"Particle_Polarization_Dependence");
				if(type==ImageType[0]){
					File.makeDirectory(polbase+"Polarization_data"+File.separator+foldname+File.separator+"Experimental_Ensemble_Histograms"+File.separator+"Ext");
					File.makeDirectory(polbase+"Polarization_data"+File.separator+foldname+File.separator+"Particle_Polarization_Dependence"+File.separator+"Ext");
					File.makeDirectory(polbase+"Polarization_data"+File.separator+foldname+File.separator+"Simulated_Noise_Data"+File.separator+"Ext");
					File.makeDirectory(polbase+"Polarization_data"+File.separator+foldname+File.separator+"Simulated_Noise_Data"+File.separator+"Ext"+File.separator+"B");
					File.makeDirectory(polbase+"Polarization_data"+File.separator+foldname+File.separator+"Simulated_Noise_Data"+File.separator+"Ext"+File.separator+"G");
					File.makeDirectory(polbase+"Polarization_data"+File.separator+foldname+File.separator+"Simulated_Noise_Data"+File.separator+"Ext"+File.separator+"R");
					File.makeDirectory(polbase+"Polarization_data"+File.separator+foldname+File.separator+"Particle_Polarization_Dependence"+File.separator+"Ext"+File.separator+"B");
					File.makeDirectory(polbase+"Polarization_data"+File.separator+foldname+File.separator+"Particle_Polarization_Dependence"+File.separator+"Ext"+File.separator+"G");
					File.makeDirectory(polbase+"Polarization_data"+File.separator+foldname+File.separator+"Particle_Polarization_Dependence"+File.separator+"Ext"+File.separator+"R");
					if(dark==true){
						File.makeDirectory(polbase+"Polarization_data"+File.separator+foldname+File.separator+"Experimental_Ensemble_Histograms"+File.separator+"Dark");
						File.makeDirectory(polbase+"Polarization_data"+File.separator+foldname+File.separator+"Particle_Polarization_Dependence"+File.separator+"Dark");
						File.makeDirectory(polbase+"Polarization_data"+File.separator+foldname+File.separator+"Simulated_Noise_Data"+File.separator+"Dark");
						File.makeDirectory(polbase+"Polarization_data"+File.separator+foldname+File.separator+"Simulated_Noise_Data"+File.separator+"Dark"+File.separator+"B");
						File.makeDirectory(polbase+"Polarization_data"+File.separator+foldname+File.separator+"Simulated_Noise_Data"+File.separator+"Dark"+File.separator+"G");
						File.makeDirectory(polbase+"Polarization_data"+File.separator+foldname+File.separator+"Simulated_Noise_Data"+File.separator+"Dark"+File.separator+"R");
						File.makeDirectory(polbase+"Polarization_data"+File.separator+foldname+File.separator+"Particle_Polarization_Dependence"+File.separator+"Dark"+File.separator+"B");
						File.makeDirectory(polbase+"Polarization_data"+File.separator+foldname+File.separator+"Particle_Polarization_Dependence"+File.separator+"Dark"+File.separator+"G");
						File.makeDirectory(polbase+"Polarization_data"+File.separator+foldname+File.separator+"Particle_Polarization_Dependence"+File.separator+"Dark"+File.separator+"R");
					}
				}
				if(type==ImageType[1]){
					File.makeDirectory(polbase+"Polarization_data"+File.separator+foldname+File.separator+"Experimental_Ensemble_Histograms"+File.separator+"Ext");
					File.makeDirectory(polbase+"Polarization_data"+File.separator+foldname+File.separator+"Simulated_Noise_Data"+File.separator+"Ext");
					File.makeDirectory(polbase+"Polarization_data"+File.separator+foldname+File.separator+"Particle_Polarization_Dependence"+File.separator+"Ext");
					if(dark==true){
						File.makeDirectory(polbase+"Polarization_data"+File.separator+foldname+File.separator+"Experimental_Ensemble_Histograms"+File.separator+"Dark");
						File.makeDirectory(polbase+"Polarization_data"+File.separator+foldname+File.separator+"Simulated_Noise_Data"+File.separator+"Dark");
						File.makeDirectory(polbase+"Polarization_data"+File.separator+foldname+File.separator+"Particle_Polarization_Dependence"+File.separator+"Dark");
					}
				}
			}
		}
	
	//////////////////////////////////Collect Polarization Data
		if(singleradius==true){
			newImage("SR_BG_ext_fulldist","32-bit",colorlength*polarnum,1,1);
			if(dark==true){
				newImage("SR_BG_dark_fulldist","32-bit",colorlength*polarnum,1,1);
			}
		}
		if(doubleradius==true){
			newImage("DR_BG_ext_fulldist","32-bit",colorlength*polarnum,1,1);
			if(dark==true){
				newImage("DR_BG_dark_fulldist","32-bit",colorlength*polarnum,1,1);
			}
		}
		if(regional==true){
			newImage("RR_BG_ext_fulldist","32-bit",colorlength*polarnum,1,1);
			if(dark==true){
				newImage("RR_BG_dark_fulldist","32-bit",colorlength*polarnum,1,1);
			}
		}
	
		distext=newArray("SR_BG_ext_fulldist","DR_BG_ext_fulldist","RR_BG_ext_fulldist");
		distdark=newArray("SR_BG_dark_fulldist","DR_BG_dark_fulldist","RR_BG_dark_fulldist");
	
		S1polext=newArray("SRS1extpol","DRS1extpol","RRS1extpol");
		S1poldark=newArray("SRS1darkpol","DRS1darkpol","RRS1darkpol");
		S2polext=newArray("SRS2extpol","DRS2extpol","RRS2extpol");
		S2poldark=newArray("SRS2darkpol","DRS2darkpol","RRS2darkpol");
	
	
		for(sizeloop=0;sizeloop<2;sizeloop++){
			if(sizesets[sizeloop]==true){
				for(radloop=0;radloop<3;radloop++){
					if(radsets[radloop]==true){
						for(imloop=0;imloop<2;imloop++){
							if(imagesets[imloop]==true){
								if(sizenames[sizeloop]=="size1"){
									if(imagenames[imloop]=="ext"){Distimage=distext; BGfile=BGextimages; files=s1ext; polimagename=S1polext;}
									if(imagenames[imloop]=="dark"){Distimage=distdark; BGfile=BGdarkimages; files=s1dark; polimagename=S1poldark;}
								}
								if(sizenames[sizeloop]=="size2"){
									if(imagenames[imloop]=="ext"){Distimage=distext; BGfile=BGextimages;files=s2ext; polimagename=S2polext;}
									if(imagenames[imloop]=="dark"){Distimage=distdark; BGfile=BGdarkimages;files=s2dark; polimagename=S2poldark;}
								}
	
								for(coloop=0;coloop<colorlength;coloop++){
									for(poloop=0;poloop<polarnum;poloop++){
	
										BASEdirec=polbase+polnames[poloop]+File.separator;
										processing=BASEdirec+"Processing"+File.separator;
										open(processing+BGfile[radloop]);
										BGheight=getHeight();
	
										makeRectangle(coloop,0,1,BGheight);
										getRawStatistics(nPixels, mean, min, max, std, histogram);
										colstd=std;
										selectWindow(BGfile[radloop]);
										close();
										selectWindow(Distimage[radloop]);
										setPixel(coloop*polarnum+poloop,0,colstd);
										if(coloop==colorlength-1 && poloop==polarnum-1){
											selectWindow(Distimage[radloop]);
											saveAs("Tiff",polproc+Distimage[radloop]);
											close();
										}
	
										open(processing+files[radloop]);
										polheight=getHeight();
	
										if(poloop==0){
											newImage(polimagename[radloop],"32-bit",colorlength*polarnum,polheight,1);
											activeimage=getTitle();
										}
										if(poloop>0){
											selectWindow(activeimage);
										}
										for(p=0;p<polheight;p++){
											selectWindow(files[radloop]);
											CS=getPixel(coloop,p);
											selectWindow(activeimage);
											setPixel(coloop*polarnum+poloop,p,CS);
										}
										selectWindow(files[radloop]);
										close();
										if(coloop==colorlength-1 && poloop==polarnum-1){
											selectWindow(polimagename[radloop]);
											saveAs("Tiff",polproc+polimagename[radloop]);
											close();
										}
									}
								}
							}
						}
					}
				}
			}
		}
	////////////////////////////////Begin Analysis
		imagenames=newArray("ext","dark");
		imagesets=newArray(extinctionSet,darkfieldSet);
	
		radnames=newArray("singleradius","doubleradius","regional");
		radsets=newArray(singleradius,doubleradius,regional);
	
		if((singleradius==true && doubleradius==false && regional==false) || (singleradius==false && doubleradius==true && regional==false) || (singleradius==false && doubleradius==false && regional==true)){
			marking=newArray("'","''"); SRmarking=""; DRmarking=""; RRmarking="";
		}
		if((singleradius==true && doubleradius==true && regional==true) || (singleradius==true && doubleradius==true) || (singleradius==true && regional==true) || (doubleradius==true && regional==true)){
			marking=newArray("'","''"); SRmarking="S"; DRmarking="D"; RRmarking="R";
		}
	
		distext=newArray("SR_BG_ext_fulldist.tif","DR_BG_ext_fulldist.tif","RR_BG_ext_fulldist.tif");
		distdark=newArray("SR_BG_dark_fulldist.tif","DR_BG_dark_fulldist.tif","RR_BG_dark_fulldist.tif");
	
		s1polext=newArray("SRS1extpol.tif","DRS1extpol.tif","RRS1extpol.tif");
		s1poldark=newArray("SRS1darkpol.tif","DRS1darkpol.tif","RRS1darkpol.tif");
		s2polext=newArray("SRS2extpol.tif","DRS2extpol.tif","RRS2extpol.tif");
		s2poldark=newArray("SRS2darkpol.tif","DRS2darkpol.tif","RRS2darkpol.tif");
	
		polproc=polbase+"Polarization_data"+File.separator+"Pol_Processing"+File.separator;
		polSR=polbase+"Polarization_data"+File.separator+"SR_Method_Data"+File.separator;
		polDR=polbase+"Polarization_data"+File.separator+"DR_Method_Data"+File.separator;
		polRR=polbase+"Polarization_data"+File.separator+"RR_Method_Data"+File.separator;
		polhist="Experimental_Ensemble_Histograms"+File.separator;
		if(type==ImageType[0]){
			polextcurveB="Particle_Polarization_Dependence"+File.separator+"Ext"+File.separator+"B"+File.separator;
			polextcurveG="Particle_Polarization_Dependence"+File.separator+"Ext"+File.separator+"G"+File.separator;
			polextcurveR="Particle_Polarization_Dependence"+File.separator+"Ext"+File.separator+"R"+File.separator;
			poldarkcurveB="Particle_Polarization_Dependence"+File.separator+"Dark"+File.separator+"B"+File.separator;
			poldarkcurveG="Particle_Polarization_Dependence"+File.separator+"Dark"+File.separator+"G"+File.separator;
			poldarkcurveR="Particle_Polarization_Dependence"+File.separator+"Dark"+File.separator+"R"+File.separator;
			histextb="Simulated_Noise_Data"+File.separator+"Ext"+File.separator+"B"+File.separator;
			histextg="Simulated_Noise_Data"+File.separator+"Ext"+File.separator+"G"+File.separator;
			histextr="Simulated_Noise_Data"+File.separator+"Ext"+File.separator+"R"+File.separator;
			histdarkb="Simulated_Noise_Data"+File.separator+"Dark"+File.separator+"B"+File.separator;
			histdarkg="Simulated_Noise_Data"+File.separator+"Dark"+File.separator+"G"+File.separator;
			histdarkr="Simulated_Noise_Data"+File.separator+"Dark"+File.separator+"R"+File.separator;
		}
		if(type==ImageType[1]){
			polextcurve="Particle_Polarization_Dependence"+File.separator+"Ext"+File.separator;
			poldarkcurve="Particle_Polarization_Dependence"+File.separator+"Dark"+File.separator;
			histext="Simulated_Noise_Data"+File.separator+"Ext"+File.separator;
			histdark="Simulated_Noise_Data"+File.separator+"Dark"+File.separator;
		}
	
		xValues=newArray(polarnum);
		for(m=0;m<polarnum;m++){
			xValues[m]=(polend/(polarnum-1))*m;
		}
		Array.getStatistics(xValues, min, max, mean, stdDev);
		xmax=max;
		if(type==ImageType[0]){
			blueyValues = newArray(polarnum);
			greenyValues = newArray(polarnum);
			redyValues = newArray(polarnum);
			BGByValues= newArray(polarnum);
			BGGyValues=newArray(polarnum);
			BGRyValues=newArray(polarnum);
		}
		if(type==ImageType[1]){
			BWyValues=newArray(polarnum);
			BGBWyValues=newArray(polarnum);
		}
	
		setBatchMode(true);
		open(mainimage);
		run("Set Scale...", "distance=0 known=0 pixel=1 unit=pixel");
		for(sizeloop=0;sizeloop<2;sizeloop++){
			if(sizesets[sizeloop]==true){
				for(radloop=0;radloop<3;radloop++){
					if(radsets[radloop]==true){
						for(imloop=0;imloop<2;imloop++){
							if(imagesets[imloop]==true){
								if(radnames[radloop]=="singleradius"){pol=polSR; xcol="X-SR"; ycol="Y-SR"; radmark=SRmarking; markspotx=15; markspoty=-10;}
								if(radnames[radloop]=="doubleradius"){pol=polDR; xcol="X-DR"; ycol="Y-DR"; radmark=DRmarking; markspotx=37; markspoty=10;}
								if(radnames[radloop]=="regional"){pol=polRR; xcol="X-RR"; ycol="Y-RR"; radmark=RRmarking; markspotx=15; markspoty=30;}
								if(sizenames[sizeloop]=="size1"){
									sizes=d2s(size1,0)+"nm_";
									titlesize=d2s(size1,0)+"nm";
									if(imagenames[imloop]=="ext"){
										poltitle=s1polext; CS="Ext"; CSdir="Ext";
										if(type==ImageType[0]){BGdist=distext; polcurveB=polextcurveB; polcurveG=polextcurveG; polcurveR=polextcurveR; histb=histextb; histg=histextg; histr=histextr;}
										if(type==ImageType[1]){BGdist=distext; polcurve=polextcurve; histBW=histext;}
									}
									if(imagenames[imloop]=="dark"){
										poltitle=s1poldark; CS="Scat"; CSdir="Dark";
										if(type==ImageType[0]){BGdist=distdark; polcurveB=poldarkcurveB; polcurveG=poldarkcurveG; polcurveR=poldarkcurveR; histb=histdarkb; histg=histdarkg; histr=histdarkr;}
										if(type==ImageType[1]){BGdist=distdark; polcurve=poldarkcurve; histBW=histdark;}
									}
								}
								if(sizenames[sizeloop]=="size2"){
									sizes=d2s(size2,0)+"nm_";
									titlesize=d2s(size1,0)+"nm";
									if(imagenames[imloop]=="ext"){
										poltitle=s2polext; CS="Ext"; CSdir="Ext";
										if(type==ImageType[0]){polcurveB=polextcurveB; polcurveG=polextcurveG; polcurveR=polextcurveR; histb=histextb; histg=histextg; histr=histextr;}
										if(type==ImageType[1]){polcurve=polextcurve; histBW=histdext;}
									}
									if(imagenames[imloop]=="dark"){
										poltitle=s2poldark; CS="Scat"; CSdir="Dark";
										if(type==ImageType[0]){polcurveB=poldarkcurveB; polcurveG=poldarkcurveG; polcurveR=poldarkcurveR; histb=histdarkb; histg=histdarkg; histr=histdarkr;}
										if(type==ImageType[1]){polcurve=poldarkcurve; histBW=histdark;}
									}
								}
	
								if(sizenames[sizeloop]=="size1"){
									open(polproc+BGdist[radloop]);
									if(type==ImageType[0]){
										makeRectangle(0,0,polarnum,1);
										getRawStatistics(nPixels, mean, min, max, std, histogram);
										avgBGB=mean;
										makeRectangle(polarnum,0,polarnum,1);
										getRawStatistics(nPixels, mean, min, max, std, histogram);
										avgBGG=mean;
										makeRectangle(2*polarnum,0,polarnum,1);
										getRawStatistics(nPixels, mean, min, max, std, histogram);
										avgBGR=mean;
										run("Select None");
										for(p=0;p<polarnum;p++){
											BGByValues[p]=getPixel(p,0);
											BGGyValues[p]=getPixel(p+polarnum,0);
											BGRyValues[p]=getPixel(p+2*polarnum,0);
										}
									}
									if(type==ImageType[1]){
										makeRectangle(0,0,polarnum,1);
										getRawStatistics(nPixels, mean, min, max, std, histogram);
										avgBG=mean;
										run("Select None");
										for(p=0;p<polarnum;p++){
											BGBWyValues[p]=getPixel(p,0);
										}
									}
									close();
								}
	
								polimage=polproc+poltitle[radloop];
								run("Clear Results");
	
								if(imagenames[imloop]=="ext"){
									open(polimage);
									particlecount=getHeight();
									close();
	
									open(polbase+"0"+File.separator+"Extinction"+File.separator+sizenames[sizeloop]+"_data_0.xls");
									x=newArray(particlecount);
									y=newArray(particlecount);
									for(p=0;p<particlecount;p++){
										x[p]=getResult(xcol,p);
										y[p]=getResult(ycol,p);
									}
									run("Clear Results");
	
									colorcheck=newArray(particlecount);
									for(z=0;z<particlecount;z++){
	
										for(b=0;b<polarnum;b++){
											if(type==ImageType[0]){
												open(polimage);
												blueyValues[b]=getPixel(b,z);
												selectWindow(poltitle[radloop]);
												greenyValues[b]=getPixel(b+polarnum,z);
												selectWindow(poltitle[radloop]);
												redyValues[b]=getPixel(b+(2*polarnum),z);
												close();
											}
											if(type==ImageType[1]){
												open(polimage);
												BWyValues[b]=getPixel(b,z);
												close();
											}
										}
	
										if(type==ImageType[0]){
											polarizedplasmon = "y = c*(1+a*cos(2*0.017453292519943*(x-b)))";
									  		Fit.doFit(polarizedplasmon, xValues, blueyValues);
									  		xfit=newArray(1001);
											yfit=newArray(1001);
											afitB=Fit.p(0);
											bfitB=Fit.p(1);
											cfitB=Fit.p(2);
											rfit=Fit.rSquared;
											if(afitB<0){
												afitB=-afitB;
												bfitB=bfitB+90;
												while(bfitB<0){
													bfitB=bfitB+180;
												}
												while(bfitB>=180){
													bfitB=bfitB-180;
												}
											}
											if(afitB>0){
												while(bfitB<0){
													bfitB=bfitB+180;
												}
												while(bfitB>=180){
													bfitB=bfitB-180;
												}
											}
	
									  		Fit.doFit(polarizedplasmon, xValues, greenyValues);
									  		xfit=newArray(1001);
											yfit=newArray(1001);
											afitG=Fit.p(0);
											bfitG=Fit.p(1);
											cfitG=Fit.p(2);
											rfit=Fit.rSquared;
	
											if(afitG<0){
												afitG=-afitG;
												bfitG=bfitG+90;
												while(bfitG<0){
													bfitG=bfitG+180;
												}
												while(bfitG>=180){
													bfitG=bfitG-180;
												}
											}
											if(afitG>0){
												while(bfitG<0){
													bfitG=bfitG+180;
												}
												while(bfitG>=180){
													bfitG=bfitG-180;
												}
											}
	
									  		Fit.doFit(polarizedplasmon, xValues, redyValues);
									  		xfit=newArray(1001);
											yfit=newArray(1001);
											afitR=Fit.p(0);
											bfitR=Fit.p(1);
											cfitR=Fit.p(2);
											rfit=Fit.rSquared;
	
											if(afitR<0){
												afitR=-afitR;
												bfitR=bfitR+90;
												while(bfitR<0){
													bfitR=bfitR+180;
												}
												while(bfitR>=180){
													bfitR=bfitR-180;
												}
											}
											if(afitR>0){
												while(bfitR<0){
													bfitR=bfitR+180;
												}
												while(bfitR>=180){
													bfitR=bfitR-180;
												}
											}
	
											if(polarizedfilter=="Silver"){
												if(cfitB>cfitG && cfitG>cfitR  && cfitB>avgBGB && cfitG>avgBGG && cfitR>avgBGR){
													colorcheck[z]=1;
												}
												else{
													colorcheck[z]=0;
												}
											}
											if(polarizedfilter=="Gold"){
												if(cfitG>cfitB && cfitR>cfitB && cfitB>avgBGB && cfitG>avgBGG && cfitR>avgBGR){
													colorcheck[z]=1;
												}
												else{
													colorcheck[z]=0;
												}
											}
											if(polarizedfilter=="Non-Metallic"){
												if(cfitB>avgBGB && cfitG>avgBGG && cfitR>avgBGR){
													colorcheck[z]=1;
												}
												else{
													colorcheck[z]=0;
												}
											}
											polfilter=arrayvalueindices(colorcheck,1);
										}
	
										if(type==ImageType[1]){
											polarizedplasmon = "y = c*(1+a*cos(2*0.017453292519943*(x-b)))";
									  		Fit.doFit(polarizedplasmon, xValues, BWyValues);
									  		xfit=newArray(1001);
											yfit=newArray(1001);
											afit=Fit.p(0);
											bfit=Fit.p(1);
											cfit=Fit.p(2);
											rfit=Fit.rSquared;
											if(afit<0){
												afit=-afit;
												bfit=bfit+90;
												while(bfit<0){
													bfit=bfit+180;
												}
												while(bfit>=180){
													bfit=bfit-180;
												}
											}
											if(afit>0){
												while(bfit<0){
													bfit=bfit+180;
												}
												while(bfit>=180){
													bfit=bfit-180;
												}
											}
											if(cfit>avgBG){
												colorcheck[z]=1;
											}
											else{
												colorcheck[z]=0;
											}
											polfilter=arrayvalueindices(colorcheck,1);
										}
									}
								}
	
								//Array.print(colorcheck);
								//Array.print(polfilter);
								//print(particlecount);
	
								if(type==ImageType[0]){
									meanB=newArray(polfilter.length);
									meanG=newArray(polfilter.length);
									meanR=newArray(polfilter.length);
									alphaB=newArray(polfilter.length);
									alphaG=newArray(polfilter.length);
									alphaR=newArray(polfilter.length);
								}
								if(type==ImageType[1]){
									meanBW=newArray(polfilter.length);
									alphaBW=newArray(polfilter.length);
								}
								xfilt=newArray(polfilter.length);
								yfilt=newArray(polfilter.length);
	
								pos=polfilter;
								sizelength=polfilter.length;
								sizename=sizes;
	
								//print(sizelength);
								//print(sizename);
								//print(pos.length);
	
	
								for(z=0;z<sizelength;z++){
									xfilt[z]=x[pos[z]];
									yfilt[z]=y[pos[z]];
									for(b=0;b<polarnum;b++){
										if(type==ImageType[0]){
											open(polimage);
											blueyValues[b]=getPixel(b,pos[z]);
											selectWindow(poltitle[radloop]);
											greenyValues[b]=getPixel(b+polarnum,pos[z]);
											selectWindow(poltitle[radloop]);
											redyValues[b]=getPixel(b+(2*polarnum),pos[z]);
											close();
										}
										if(type==ImageType[1]){
											open(polimage);
											BWyValues[b]=getPixel(b,pos[z]);
											close();
										}
									}
	
									if(type==ImageType[0]){
										Array.getStatistics(blueyValues, min, max, mean, stdDev);
										blueymax=max;
										blueymin=min;
										Array.getStatistics(greenyValues, min, max, mean, stdDev);
										greenymax=max;
										greenymin=min;
										Array.getStatistics(redyValues, min, max, mean, stdDev);
										redymax=max;
										redymin=min;
	
										Fit.doFit(polarizedplasmon, xValues, blueyValues);
									  	xfit=newArray(1001);
										yfit=newArray(1001);
										afitB=Fit.p(0);
										bfitB=Fit.p(1);
										cfitB=Fit.p(2);
										rfit=Fit.rSquared;
	
										if(afitB<0){
											afitB=-afitB;
											bfitB=bfitB+90;
											while(bfitB<0){
												bfitB=bfitB+180;
											}
											while(bfitB>=180){
												bfitB=bfitB-180;
											}
										}
										if(afitB>0){
											while(bfitB<0){
												bfitB=bfitB+180;
											}
											while(bfitB>=180){
												bfitB=bfitB-180;
											}
										}
	
										for(i=0;i<1001;i++){
											xfit[i]=i*180/1000;
										}
										for(i=0;i<1001;i++){
											yfit[i]=cfitB*(1+afitB*cos(2*0.017453292519943*(xfit[i]-bfitB)));
										}
										Array.getStatistics(yfit, min, max, mean, stdDev);
										ampB=max-min;
										RMSEdifsq=newArray(xValues.length);
										for(i=0;i<xValues.length;i++){
											RMSEdifsq[i]=pow(yfit[xValues[i]]-blueyValues[i],2);
										}
										RMSEsum=arraySum(RMSEdifsq);
										RMSEb=sqrt(RMSEsum/(xValues.length));
	
										Plot.create(CS+" Vs. Theta", "Polarization Angle (Deg)", "Sigma_"+CS+" (nm^2)");
									        Plot.setLimits(0, xmax, blueymin-0.1*blueymin, blueymax+0.1*blueymax);
									        Plot.setLineWidth(2);
									        Plot.setColor("gray");
									        Plot.add("line",xfit, yfit);
									        Plot.setColor("blue");
									        Plot.add("circles",xValues, blueyValues);
									        Plot.setColor("black");
									        setJustification("center");
									        Plot.addText(CS+" Cross-Section vs. Polarisation Angle", 0.5, 0.0);
									        Plot.setJustification("right");
									        Plot.addText("a="+afitB,1,0.075);
									        Plot.addText("b="+bfitB,1,0.130);
									        Plot.addText("c="+cfitB,1,0.185);
									        Plot.addText("R^2="+rfit,1,0.250);
									        Plot.addText("RMSE="+RMSEb,1,0.305);
									        Plot.addText("Amp(PtoP)="+ampB,1,0.36);
									        Plot.show();
									        saveAs("PNG",pol+polcurveB+sizename+z+1+"_"+CS+"vsTheta.png");
									        close();
	
									  	Fit.doFit(polarizedplasmon, xValues, greenyValues);
									  	xfit=newArray(1001);
										yfit=newArray(1001);
										afitG=Fit.p(0);
										bfitG=Fit.p(1);
										cfitG=Fit.p(2);
										rfit=Fit.rSquared;
	
										if(afitG<0){
											afitG=-afitG;
											bfitG=bfitG+90;
											while(bfitG<0){
												bfitG=bfitG+180;
											}
											while(bfitG>=180){
												bfitG=bfitG-180;
											}
										}
										if(afitG>0){
											while(bfitG<0){
												bfitG=bfitG+180;
											}
											while(bfitG>=180){
												bfitG=bfitG-180;
											}
										}
	
										for(i=0;i<1001;i++){
											xfit[i]=i*180/1000;
										}
										for(i=0;i<1001;i++){
											yfit[i]=cfitG*(1+afitG*cos(2*0.017453292519943*(xfit[i]-bfitG)));
										}
										Array.getStatistics(yfit, min, max, mean, stdDev);
										ampG=max-min;
										RMSEdifsq=newArray(xValues.length);
										for(i=0;i<xValues.length;i++){
											RMSEdifsq[i]=pow(yfit[xValues[i]]-greenyValues[i],2);
										}
										RMSEsum=arraySum(RMSEdifsq);
										RMSEg=sqrt(RMSEsum/(xValues.length));
	
										Plot.create(CS+" Vs. Theta", "Polarization Angle (Deg)", "Sigma_"+CS+" (nm^2)");
									        Plot.setLimits(0, xmax, greenymin-0.1*greenymin, greenymax+0.1*greenymax);
									        Plot.setLineWidth(2);
									        Plot.setColor("gray");
									        Plot.add("line",xfit, yfit);
									        Plot.setColor("green");
									        Plot.add("circles",xValues, greenyValues);
									        Plot.setColor("black");
									        setJustification("center");
									        Plot.addText(CS+" Cross-Section vs. Polarisation Angle", 0.5, 0.0);
									        Plot.setJustification("right");
									        Plot.addText("a="+afitG,1,0.075);
									        Plot.addText("b="+bfitG,1,0.130);
									        Plot.addText("c="+cfitG,1,0.185);
									        Plot.addText("R^2="+rfit,1,0.250);
									        Plot.addText("RMSE="+RMSEg,1,0.305);
									        Plot.addText("Amp(PtoP)="+ampG,1,0.36);
									        Plot.show();
									        saveAs("PNG",pol+polcurveG+sizename+z+1+"_"+CS+"vsTheta.png");
									        close();
	
	
									  	Fit.doFit(polarizedplasmon, xValues, redyValues);
									  	xfit=newArray(1001);
										yfit=newArray(1001);
										afitR=Fit.p(0);
										bfitR=Fit.p(1);
										cfitR=Fit.p(2);
										rfit=Fit.rSquared;
	
										if(afitR<0){
											afitR=-afitR;
											bfitR=bfitR+90;
											while(bfitR<0){
												bfitR=bfitR+180;
											}
											while(bfitR>=180){
												bfitR=bfitR-180;
											}
										}
										if(afitR>0){
											while(bfitR<0){
												bfitR=bfitR+180;
											}
											while(bfitR>=180){
												bfitR=bfitR-180;
											}
										}
	
										for(i=0;i<1001;i++){
											xfit[i]=i*180/1000;
										}
										for(i=0;i<1001;i++){
											yfit[i]=cfitR*(1+afitR*cos(2*0.017453292519943*(xfit[i]-bfitR)));
										}
										Array.getStatistics(yfit, min, max, mean, stdDev);
										ampR=max-min;
										RMSEdifsq=newArray(xValues.length);
										for(i=0;i<xValues.length;i++){
											RMSEdifsq[i]=pow(yfit[xValues[i]]-redyValues[i],2);
										}
										RMSEsum=arraySum(RMSEdifsq);
										RMSEr=sqrt(RMSEsum/(xValues.length));
	
										Plot.create(CS+" Vs. Theta", "Polarization Angle (Deg)", "Sigma_"+CS+" (nm^2)");
									        Plot.setLimits(0, xmax, redymin-0.1*redymin, redymax+0.1*redymax);
									        Plot.setLineWidth(2);
									        Plot.setColor("gray");
									        Plot.add("line",xfit, yfit);
									        Plot.setColor("red");
									        Plot.add("circles",xValues, redyValues);
									        Plot.setColor("black");
									        setJustification("center");
									        Plot.addText(CS+" Cross-Section vs. Polarisation Angle", 0.5, 0.0);
									        Plot.setJustification("right");
									        Plot.addText("a="+afitR,1,0.075);
									        Plot.addText("b="+bfitR,1,0.130);
									        Plot.addText("c="+cfitR,1,0.185);
									        Plot.addText("R^2="+rfit,1,0.250);
									        Plot.addText("RMSE="+RMSEr,1,0.305);
									        Plot.addText("Amp(PtoP)="+ampR,1,0.36);
									        Plot.show();
									        saveAs("PNG",pol+polcurveR+sizename+z+1+"_"+CS+"vsTheta.png");
									        close();
	
										meanB[z]=cfitB;
										meanG[z]=cfitG;
										meanR[z]=cfitR;
										alphaB[z]=afitB;
										alphaG[z]=afitG;
										alphaR[z]=afitR;
	
										if(z==sizelength-1){
											Array.getStatistics(meanB, min, max, mean, stdDev);
											bpartm=mean;
											bpartstd=stdDev;
											Array.getStatistics(meanG, min, max, mean, stdDev);
											gpartm=mean;
											gpartstd=stdDev;
											Array.getStatistics(meanR, min, max, mean, stdDev);
											rpartm=mean;
											rpartstd=stdDev;
										}
	
	
										if(z==0){
											run("Clear Results");
										}
	
										setResult("X",z,xfilt[z]);
										setResult("Y",z,yfilt[z]);
	
										for(r=0;r<polarnum;r++){
											setResult("Blue "+CS+" (nm^2) @"+xValues[r],z,blueyValues[r]);
										}
										setResult("Blue Alpha",z,afitB);
										setResult("Blue Phase (Deg)",z,bfitB);
										setResult("Blue Particle Mean"+CS+" (nm^2)",z,cfitB);
	
										setResult("Blank",z,"");
	
										for(r=0;r<polarnum;r++){
											setResult("Green "+CS+" (nm^2) @"+xValues[r],z,greenyValues[r]);
										}
										setResult("Green Alpha",z,afitG);
										setResult("Green Phase (Deg)",z,bfitG);
										setResult("Green Particle Mean"+CS+" (nm^2)",z,cfitG);
										setResult("Blank2",z,"");
	
										for(r=0;r<polarnum;r++){
											setResult("Red "+CS+" (nm^2) @"+xValues[r],z,redyValues[r]);
										}
										setResult("Red Alpha",z,afitR);
										setResult("Red Phase (Deg)",z,bfitR);
										setResult("Red Particle Mean"+CS+" (nm^2)",z,cfitR);
	
										if(z==sizelength-1){
											setResult("Blue Ensemble Mean (nm^2)",z+1,bpartm);
											setResult("Blue Ensemble StDev (nm^2)",z+1,bpartstd);
											setResult("Green Ensemble Mean (nm^2)",z+1,gpartm);
											setResult("Green Ensemble StDev (nm^2)",z+1,gpartstd);
											setResult("Red Ensemble Mean (nm^2)",z+1,rpartm);
											setResult("Red Ensemble StDev (nm^2)",z+1,rpartstd);
										}
									}
									if(type==ImageType[1]){
										Array.getStatistics(BWyValues, min, max, mean, stdDev);
										BWymax=max;
										BWymin=min;
	
	
										Fit.doFit(polarizedplasmon, xValues, BWyValues);
									  	xfit=newArray(1001);
										yfit=newArray(1001);
										afit=Fit.p(0);
										bfit=Fit.p(1);
										cfit=Fit.p(2);
										rfit=Fit.rSquared;
	
										if(afit<0){
											afit=-afit;
											//bfit=bfit+90;
											//if(bfit<-90){
											//	while(bfit<-90){
											//		bfit=bfit+180;
												//}
											//}
											//if(bfit>90){
												//while(bfit>90){
													//bfit=bfit-180;
												//}
											//}
											bfit=bfit+90;
											while(bfit<0){
												bfit=bfit+180;
											}
											while(bfit>=180){
												bfit=bfit-180;
											}
										}
										if(afit>0){
											while(bfit<0){
												bfit=bfit+180;
											}
											while(bfit>=180){
												bfit=bfit-180;
											}
										}
										
										for(i=0;i<1001;i++){
											xfit[i]=i*180/1000;
										}
										for(i=0;i<1001;i++){
											yfit[i]=cfit*(1+afit*cos(2*0.017453292519943*(xfit[i]-bfit)));
										}
										Array.getStatistics(yfit, min, max, mean, stdDev);
										amp=max-min;
										RMSEdifsq=newArray(xValues.length);
										for(i=0;i<xValues.length;i++){
											RMSEdifsq[i]=pow(yfit[xValues[i]]-BWyValues[i],2);
										}
										RMSEsum=arraySum(RMSEdifsq);
										RMSE=sqrt(RMSEsum/(xValues.length));
	
										Plot.create(CS+" Vs. Theta", "Polarization Angle (Deg)", "Sigma_"+CS+" (nm^2)");
									        Plot.setLimits(0, xmax, BWymin-0.1*BWymin, BWymax+0.1*BWymax);
									        Plot.setLineWidth(2);
									        Plot.setColor("gray");
									        Plot.add("line",xfit, yfit);
									        Plot.setColor("blue");
									        Plot.add("circles",xValues, BWyValues);
									        Plot.setColor("black");
									        setJustification("center");
									        Plot.addText(CS+" Cross-Section vs. Polarisation Angle", 0.5, 0.0);
									        Plot.setJustification("right");
									        Plot.addText("a="+afit,1,0.075);
									        Plot.addText("b="+bfit,1,0.130);
									        Plot.addText("c="+cfit,1,0.185);
									        Plot.addText("R^2="+rfit,1,0.250);
									        Plot.addText("RMSE="+RMSE,1,0.305);
									        Plot.addText("Amp(PtoP)="+amp,1,0.36);
									        Plot.show();
									        saveAs("PNG",pol+polcurve+sizename+z+1+"_"+CS+"vsTheta.png");
									        close();
	
									        meanBW[z]=cfit;
										alphaBW[z]=afit;
	
										if(z==sizelength-1){
											Array.getStatistics(meanBW, min, max, mean, stdDev);
											partm=mean;
											partstd=stdDev;
										}
	
	
										if(z==0){
											run("Clear Results");
										}
										setResult("X",z,xfilt[z]);
										setResult("Y",z,yfilt[z]);
										for(r=0;r<polarnum;r++){
											setResult(CS+" (nm^2) @ "+xValues[r],z,BWyValues[r]);
										}
										setResult("Alpha",z,afit);
										setResult("Phase (Deg)",z,bfit);
										setResult("Particle Mean "+CS+" (nm^2)",z,cfit);
	
										if(z==sizelength-1){
											setResult("Ensemble Mean (nm^2)",z+1,partm);
											setResult("Ensemble StDev (nm^2)",z+1,partstd);
										}
									}
								}
								updateResults();
	
								wait(2000);
								selectWindow("Results");
								saveAs("Results", pol+"polarized_"+sizename+CS+"_data"+".xls");
								run("Clear Results");
	
								if(imagenames[imloop]=="ext"){
									selectWindow(maintitle);
									for(z=0;z<sizelength;z++){
										setBackgroundColor(0,0,0);
										setForegroundColor(255,255,255);
										setFont("Times New Roman", 16, " anti-aliased");
										drawString(z+1+radmark+marking[sizeloop], xfilt[z]+markspotx, yfilt[z]+markspoty);
									}
								}
	
								if(type==ImageType[0]){newImage(sizename+" Size/Shape Data","32-bit",sizelength,6,1);}
								if(type==ImageType[1]){newImage(sizename+" Size/Shape Data","32-bit",sizelength,2,1);}
								selectWindow(sizename+" Size/Shape Data");
									for(m=0;m<sizelength;m++){
										if(type==ImageType[0]){
											setPixel(m,0,alphaB[m]);
											setPixel(m,1,alphaG[m]);
											setPixel(m,2,alphaR[m]);
											setPixel(m,3,meanB[m]);
											setPixel(m,4,meanG[m]);
											setPixel(m,5,meanR[m]);
										}
										if(type==ImageType[1]){
											setPixel(m,0,alphaBW[m]);
											setPixel(m,1,meanBW[m]);
										}
									}
	
								if(type==ImageType[0]){numhist=6;}
								if(type==ImageType[1]){numhist=2;}
								for(m=0;m<numhist;m++){
									makeRectangle(0,m,sizelength,1);
									run("Measure");
									histmin=getResult("Min");
									histmax=getResult("Max");
									if(type==ImageType[0]){
										if(m==0){alphamaxb=histmax; alphaminb=histmin;}
										if(m==1){alphamaxg=histmax; alphaming=histmin;}
										if(m==2){alphamaxr=histmax; alphaminr=histmin;}
										if(m==3){meanmaxb=histmax; meanminb=histmin;}
										if(m==4){meanmaxg=histmax; meanming=histmin;}
										if(m==5){meanmaxr=histmax; meanminr=histmin;}
									}
									if(type==ImageType[1]){
										if(m==0){alphamax=histmax; alphamin=histmin;}
										if(m==1){meanmax=histmax; meanmin=histmin;}
									}
								}
									run("Select None");
									run("Clear Results");
									saveAs("Tiff",polproc+sizename+CS+"_alphameans.tif");
	
									if(type==ImageType[0]){
										alpha1=titlesize+"_Blue_Ensemble_Alpha_Parameter"; alpha2=titlesize+"_Green_Ensemble_Alpha_Parameter"; alpha3=titlesize+"_Red_Ensemble_Alpha_Parameter";
										meanname1=titlesize+"_Blue_Ensemble_Mean_Cross-sections"; meanname2=titlesize+"_Green_Ensemble_Mean_Cross-sections"; meanname3=titlesize+"_Red_Ensemble_Mean_Cross-sections";
										plotsave=newArray(alpha1,alpha2,alpha3,meanname1,meanname2,meanname3);
										//alpha1=titlesize+"_Balpha"; alpha2=titlesize+"_Galpha"; alpha3=titlesize+"_Ralpha";
										//meanname1=titlesize+"_Bmean"; meanname2=titlesize+"_GMean"; meanname3=titlesize+"_RMean";
										//plotsave=newArray(alpha1,alpha2,alpha3,meanname1,meanname2,meanname3);
									}
									if(type==ImageType[1]){
										alpha1=titlesize+"_Ensemble_Alpha_Parameter";
										meanname1=titlesize+"_Ensemble_Mean_Cross-sections";
										plotsave=newArray(alpha1,meanname1);
										//alpha1=titlesize+"_BWalpha";
										//meanname1=titlesize+"_BWmean";
										//plotsave=newArray(alpha1,meanname1);
									}
	
								nbins=round(log(sizelength)/log(2))+1;
								for(k=0;k<colorlength;k++){
									if(type==ImageType[0]){
										if(k==0){plotmax=alphamaxb; plotmin=alphaminb; color="blue"; col="Blue";}
										if(k==1){plotmax=alphamaxg; plotmin=alphaming; color="green"; col="Green";}
										if(k==2){plotmax=alphamaxr; plotmin=alphaminr; color="red"; col="Red";}	
									}
									if(type==ImageType[1]){
										plotmax=alphamax; plotmin=alphamin; alphas=alphaBW; color="black"; col="";
									}
									binwidth=(plotmax-plotmin)/nbins;
									//if(binwidth<1) increment=0.00001;
									//if(binwidth>1) increment=1;
									increment=(plotmax-plotmin)/800;
									selectWindow(sizename+CS+"_alphameans.tif");
									makeRectangle(0,k,sizelength,1);
									getHistogram(values,counts,nbins);
									values1=values;
									counts1=counts;
									getRawStatistics(nPixels, mean, min, max, std, histogram);
									meanhist=mean;
									stdhist=std;
									//run("Histogram", "bins=&nbins x_min=&plotmin x_max=&plotmax y_max=&sizelength");
									Array.getStatistics(counts1,min,max,mean,stdDev);
									maxhist=max;
								    Plot.create(col+CS+plotsave[k], "Alpha Parameter", "Counts");
									Plot.setFrameSize(800,400);
									Plot.setLimits(plotmin,plotmax,0,maxhist+0.3*maxhist);
									Plot.setColor(color);
									for(i=0;i<nbins;i++){
										for(j=0;j<binwidth-0.1*binwidth;j+=increment){
											Plot.drawLine(values1[i]+j,0,values1[i]+j,counts1[i]);
										}
									}
									Plot.setJustification("right");
									Plot.setColor(color);
									Plot.addText("Mean Alpha="+meanhist,1,0.07);
									Plot.addText("Alpha Std. Dev.= "+stdhist,1,0.105);
									Plot.show();
									saveAs("PNG",pol+polhist+CSdir+File.separator+CS+"_"+plotsave[k]+".png");
								    close();
								}
								run("Select None");
	
								for(k=colorlength;k<2*colorlength;k++){
									if(type==ImageType[0]){
										if(k==0){plotmin=meanminb; plotmax=meanmaxb; means=meanB; color="blue";}
										if(k==1){plotmin=meanming; plotmax=meanmaxg; means=meanG; color="green";}
										if(k==2){plotmin=meanminr; plotmax=meanmaxr; means=meanR; color="red"}	
									}
									if(type==ImageType[1]){
										plotmin=meanmin; plotmax=meanmax; means=meanBW; color="black";
									}
								    binwidth=(plotmax-plotmin)/nbins;
									//if(binwidth<1) increment=0.00001;
									//if(binwidth>1) increment=1;
									increment=(plotmax-plotmin)/800;
									selectWindow(sizename+CS+"_alphameans.tif");
									makeRectangle(0,k,sizelength,1);
									getHistogram(values,counts,nbins);
									values1=values;
									counts1=counts;
									getRawStatistics(nPixels, mean, min, max, std, histogram);
									meanhist=mean;
									stdhist=std;
									Array.getStatistics(counts1,min,max,mean,stdDev);
									maxhist=max;
									//run("Histogram", "bins=&nbins x_min=&plotmin x_max=&plotmax y_max=&sizelength");
	
								    Plot.create(col+CS+plotsave[k], "Mean Cross-section (nm^2)", "Counts");
									Plot.setFrameSize(800,400);
									Plot.setLimits(plotmin,plotmax,0,maxhist+0.3*maxhist);
									Plot.setColor(color);
									for(i=0;i<nbins;i++){
										for(j=0;j<binwidth-0.1*binwidth;j+=increment){
											Plot.drawLine(values1[i]+j,0,values1[i]+j,counts1[i]);
										}
									}
									Plot.setJustification("right");
									Plot.setColor(color);
									Plot.addText("Mean "+CS+" Cross-section (nm^2)="+meanhist,1,0.07);
									Plot.addText(CS+" Cross-section StDev (nm^2)="+stdhist,1,0.105);
									Plot.show();
									saveAs("PNG",pol+polhist+CSdir+File.separator+CS+"_"+plotsave[k]+".png");
								    close();
								}
								run("Select None");
								selectWindow(sizename+CS+"_alphameans.tif");
								close();
	
								//////////////////////////////Simulated Noise for Fit distributions
								Ymax=sizelength*noisefits;
								if(type==ImageType[0]){newImage("Full Noise Fits","32-bit",9,Ymax,1);}
								if(type==ImageType[1]){newImage("Full Noise Fits","32-bit",3,Ymax,1);}
								fullnoise=getTitle();
								nbins=round(log(noisefits)/log(2))+1;
								NBINS=round(log(noisefits*sizelength)/log(2))+1;
								open(polimage);
								for(P=0;P<colorlength;P++){
									if(P==0){
										if(type==ImageType[0]){dataset=BGByValues; color="B"; col="blue"; histdir=histb;}
										if(type==ImageType[1]){dataset=BGBWyValues; color=""; col="black"; histdir=histBW;}
									}
									if(P==1){
										dataset=BGGyValues; color="G"; col="green"; histdir=histg;
									}
									if(P==2){
										dataset=BGRyValues; color="R"; col="red"; histdir=histr;
									}
	
									height=sizelength;
									particledata=newArray(polarnum);
									newImage("Test","32-bit",noisefits,3*height,1);
	
									for(q=0;q<height;q++){
										selectWindow(poltitle[radloop]);
										for(i=0;i<polarnum;i++){
											particledata[i]=getPixel(i+P*polarnum,pos[q]);
										}
	
										fits=newArray(noisefits);
										fits1=newArray(noisefits);
										fits2=newArray(noisefits);
										randata=newArray(polarnum);
										Eq="y=c*(1+a*(cos(2*0.0174533*(x-b))))";
	
										for(z=0;z<noisefits;z++){
											randommean=newArray(polarnum);
											for(jr=0;jr<polarnum;jr++){
												randomvalue=0; for(kr=0;kr<12;kr++) randomvalue+=(random()-0.5);
												randommean[jr]=randomvalue*dataset[jr];
											}
	
											for(i=0;i<polarnum;i++){
												randata[i]=particledata[i]+randommean[i];
											}
	
											Fit.doFit(Eq,xValues,randata);
											a=Fit.p(0);
											b=Fit.p(1);
											c=Fit.p(2);
											if(a<0){
												a=a*(-1);
												b=b+90;
												while(b<0){
													b=b+180;
												}
												while(b>=180){
													b=b-180;
												}
											}
											if(a>0){
												while(b<0){
													b=b+180;
												}
												while(b>=180){
													b=b-180;
												}
											}
											fits[z]=a;
											fits1[z]=b;
											fits2[z]=c;
											selectWindow("Full Noise Fits");
											setPixel(P,z+q*noisefits,fits[z]);
											setPixel(colorlength+P,z+q*noisefits,fits1[z]);
											setPixel(2*colorlength+P,z+q*noisefits,fits2[z]);
										}
	
										selectWindow("Test");
										Array.getStatistics(fits, min, max, mean,stdDev);
										fitmean=mean;
										fitstd=stdDev;
										fitmax=max;
										fitmin=min;
										//finhistmax=fitmax+0.05*fitmax;
										//finhistmin=fitmin-0.05*fitmin;
										Array.getStatistics(fits1,min,max,mean,stdDev);
										fit1mean=mean;
										fit1std=stdDev;
										Array.getStatistics(fits2,min,max,mean,stdDev);
										fit2mean=mean;
										fit2std=stdDev;
										for(i=0;i<noisefits;i++){
											setPixel(i,q,fits[i]);
											setPixel(i,q+height,fits1[i]);
											setPixel(i,q+2*height,fits2[i]);
											setResult("Alpha Fit"+i+1,q,fits[i]);
										}
										setResult("Mean Alpha",q,fitmean);
										setResult("Alpha StDev",q,fitstd);
										setResult("N/A",q,"");
										for(i=0;i<noisefits;i++){
											setResult("Phase Fit"+i+1,q,fits1[i]);
										}
										setResult("Mean Phase",q,fit1mean);
										setResult("Phase StDev",q,fit1std);
										setResult("N/A2",q,"");
										for(i=0;i<noisefits;i++){
											setResult("Cross-section Fit"+i+1,q,fits1[i]);
										}
										setResult("Mean Cross-section",q,fit2mean);
										setResult("Cross-section StDev",q,fit2std);
										binwidth=(fitmax-fitmin)/nbins;
										//if(binwidth<1) increment=0.00001;
										//if(binwidth>1) increment=1;
										increment=(fitmax-fitmin)/800;
										selectWindow("Test");
										makeRectangle(0,q,noisefits,1);
										//run("Histogram", "bins=&nbins use x_min=&finhistmin x_max=&finhistmax y_max=&noisefits");
										getHistogram(values,counts,nbins);
										values1=values;
										counts1=counts;
										getRawStatistics(nPixels, mean, min, max, std, histogram);
										meanhist=mean;
										stdhist=std;
										Array.getStatistics(counts1,min,max,mean,stdDev);
										maxhist=max;
									    Plot.create(color+CS+"Alpha", "Alpha Parameter", "Counts");
										Plot.setFrameSize(800,400);
										Plot.setLimits(fitmin,fitmax,0,maxhist+0.3*maxhist);
										Plot.setColor(col);
										for(i=0;i<nbins;i++){
											for(j=0;j<binwidth-0.1*binwidth;j+=increment){
												Plot.drawLine(values1[i]+j,0,values1[i]+j,counts1[i]);
											}
										}
										Plot.setJustification("right");
										Plot.setColor(col);
										Plot.addText("Mean Simulated Alpha = "+meanhist,1,0.07);
										Plot.addText("Simulated Alpha StDev = "+stdhist,1,0.105);
										Plot.show();
										saveAs("PNG",pol+histdir+sizename+color+"_"+CS+"_"+q+1+"_Simulated_Alpha.png");
										close();
									}
	
									selectWindow("Test");
									saveAs("Tiff",polproc+sizename+color+"_"+CS+"_"+"fitdata"+".tif");
									close();
									updateResults();
									selectWindow("Results");
									saveAs("Results",pol+histdir+sizename+color+"_"+CS+"_"+"fitdata"+".xls");
	
									selectWindow("Full Noise Fits");
									makeRectangle(P,0,1,Ymax);
									run("Clear Results");
									run("Measure");
									finhistmax=getResult("Max");
									finhistmin=getResult("Min");
									//finhistmax=finhistmax+0.05*finhistmax;
									//finhistmin=finhistmin-0.05*finhistmin;
									run("Clear Results");
									//run("Histogram", "bins=&NBINS use x_min=&finhistmin x_max=&finhistmax y_max=&Ymax");
									//selectWindow("Histogram of Full");
									binwidth=(finhistmax-finhistmin)/NBINS;
									//if(binwidth<1) increment=0.00001;
									//if(binwidth>1) increment=1;
									increment=(finhistmax-finhistmin)/800;
									getHistogram(values,counts,NBINS);
									values1=values;
									counts1=counts;
									getRawStatistics(nPixels, mean, min, max, std, histogram);
									meanhist=mean;
									stdhist=std;
									Array.getStatistics(counts1,min,max,mean,stdDev);
									maxhist=max;
								    Plot.create(color+CS+"Ensemble Alpha", "Alpha Parameter", "Counts");
									Plot.setFrameSize(800,400);
									Plot.setLimits(finhistmin,finhistmax,0,maxhist+0.3*maxhist);
									Plot.setColor(col);
									for(i=0;i<NBINS;i++){
										for(j=0;j<binwidth-0.1*binwidth;j+=increment){
											Plot.drawLine(values1[i]+j,0,values1[i]+j,counts1[i]);
										}
									}
									Plot.setJustification("right");
									Plot.setColor(col);
									Plot.addText("Ensemble Mean Simulated Alpha = "+meanhist,1,0.07);
									Plot.addText("Ensemble Simulated Alpha StDev = "+stdhist,1,0.105);
									Plot.show();
									saveAs("PNG",pol+histdir+sizename+CS+"_"+color+"_Ensemble_Simulated_Alpha_Parameter.png");
									close();
	
									selectWindow("Full Noise Fits");
									makeRectangle(P+colorlength,0,1,Ymax);
									finhistmax=180;
									finhistmin=0;
									run("Clear Results");
									//run("Histogram", "bins=&NBINS use x_min=&finhistmin x_max=&finhistmax y_max=&Ymax");
									//selectWindow("Histogram of Full");
									binwidth=(finhistmax-finhistmin)/NBINS;
									//if(binwidth<1) increment=0.00001;
									//if(binwidth>1) increment=0.5;
									increment=(finhistmax-finhistmin)/800;
									getHistogram(values,counts,NBINS);
									values1=values;
									counts1=counts;
									getRawStatistics(nPixels, mean, min, max, std, histogram);
									meanhist=mean;
									stdhist=std;
									Array.getStatistics(counts1,min,max,mean,stdDev);
									maxhist=max;
								    Plot.create(color+CS+"Ensemble Phase", "Phase (Deg)", "Counts");
									Plot.setFrameSize(800,400);
									Plot.setLimits(finhistmin,finhistmax,0,maxhist+0.3*maxhist);
									Plot.setColor(col);
									for(i=0;i<NBINS;i++){
										for(j=0;j<binwidth-0.1*binwidth;j+=increment){
											Plot.drawLine(values1[i]+j,0,values1[i]+j,counts1[i]);
										}
									}
									Plot.setJustification("right");
									Plot.setColor(col);
									Plot.addText("Ensemble Mean Simulated Phase = "+meanhist,1,0.07);
									Plot.addText("Ensemble Simulated Phase StDev = "+stdhist,1,0.105);
									Plot.show();
									saveAs("PNG",pol+histdir+sizename+CS+"_"+color+"_Ensemble_Simulated_Phase.png");
									close();
	
									selectWindow("Full Noise Fits");
									makeRectangle(P+2*colorlength,0,1,Ymax);
									run("Clear Results");
									run("Measure");
									finhistmax=getResult("Max");
									finhistmin=getResult("Min");
									//finhistmax=finhistmax+0.05*finhistmax;
									//finhistmin=finhistmin-0.05*finhistmin;
									run("Clear Results");
									//run("Histogram", "bins=&NBINS use x_min=&finhistmin x_max=&finhistmax y_max=&Ymax");
									//selectWindow("Histogram of Full");
									binwidth=(finhistmax-finhistmin)/NBINS;
									//if(binwidth<1) increment=0.00001;
									//if(binwidth>1) increment=1;
									increment=(finhistmax-finhistmin)/800;
									getHistogram(values,counts,NBINS);
									values1=values;
									counts1=counts;
									getRawStatistics(nPixels, mean, min, max, std, histogram);
									meanhist=mean;
									stdhist=std;
									Array.getStatistics(counts1,min,max,mean,stdDev);
									maxhist=max;
								    Plot.create(color+CS+"Ensemble Mean Cross-section", "Phase (Deg)", "Counts");
									Plot.setFrameSize(800,400);
									Plot.setLimits(finhistmin,finhistmax,0,maxhist+0.3*maxhist);
									Plot.setColor(col);
									for(i=0;i<NBINS;i++){
										for(j=0;j<binwidth-0.1*binwidth;j+=increment){
											Plot.drawLine(values1[i]+j,0,values1[i]+j,counts1[i]);
										}
									}
									Plot.setJustification("right");
									Plot.setColor(col);
									Plot.addText("Ensemble Mean Simulated "+CS+" cross-section = "+meanhist,1,0.07);
									Plot.addText("Ensemble Simulated "+CS+" cross-section StDev = "+stdhist,1,0.105);
									Plot.show();
									saveAs("PNG",pol+histdir+sizename+CS+"_"+color+"_Ensemble_Simulated_Cross-section.png");
									close();
								}
								selectWindow("Full Noise Fits");
								run("Select None");
								saveAs("Tiff",polproc+sizename+CS+"_"+"Full_Noise_Fits.tif");
								close();
								selectWindow(poltitle[radloop]);
								close();
							}
						}
					}
				}
				if(NPsize==true){
					selectWindow(maintitle);
					saveAs("Tiff",polbase+"Polarization_data"+File.separator+"Labeled_particles.tif");
					close();
				}
				if(NPsize==false){
					if(sizenames[sizeloop]=="size2"){
						selectWindow(maintitle);
						saveAs("Tiff",polbase+"Polarization_data"+File.separator+"Labeled_particles.tif");
						close();
					}
				}
			}
		}
		run("Clear Results");
	}
	beep();
	showMessage("Operation Complete");
}